$DEBUG
C     AMMIWINS:  AMMI winners.  HGG July 2007.
C     This program reads an output from MATMODEL
C     and is explained in the MATMODEL documentation.
      INTEGER*4    L(16000), LAM(7)
      CHARACTER*4  C(16000)
      CHARACTER*1  T
      CHARACTER*40  FN1, FN2
      COMMON  /DATA/  V(16000)
      COMMON  /PARM/  NG, NE, GM
      COMMON  /SETO/  KEM,KGC,KEC,KGL,KEL,KGW,KEW,KGT,KGA,KEA
      EQUIVALENCE  (V(1), L(1), C(1))
      LNGTH4 = 16000
C     Memory model.
C     Data     Vector  Subscr   Offset
C     GEN means     V     IGM     0
C     ENV means     V     IEM     KEM =  G
C     GEN names     C     IGC     KGC =  G +  E
C     ENV names     C     IEC     KEC = 2G +  E
C     GEN mark      L     IGL     KGL = 2G + 2E
C     ENV ranks     LV    IEL     KEL = 3G + 2E
C     GEN winners   LV    IGR     KGW = 3G + 3E
C     ENV winners   L     IER     KEW = 4G + 3E
C     GEN index     L     IGT     KGT = 4G + 4E
C     GEN IPCA      V     IGA     KGA = 5G + 4E
C     ENV IPCA      V     IEA     KEA = (5+NXL)G + 4E
C     Also presumes that NE<=4900 and Megaenvs<=1600.
C     Open files.
      WRITE (*, '('' AMMIWINS:  AMMI Winners.'', /, '' This is'',
     1 '' open source as explained in the MATMODEL documentation.'',
     2 /, '' Hugh Gauch, Crop and Soil Sciences, Cornell'',
     3 '' University.'', /)')
   10 WRITE (*, '('' Enter input filename.'')')
      READ (*, '(A40)')  FN1
      WRITE (*, '('' Enter output filename.'')')
      READ (*, '(A40)')  FN2
      WRITE (*, '('' Are these filenames correct?  <Y, N>'')')
      READ (*, '(A1)')  T
      IF (.NOT. (T.EQ.'Y' .OR. T.EQ.'y'))  GO TO 10
      OPEN (1, FILE=FN1, STATUS='OLD')
      OPEN (2, FILE=FN2, STATUS='NEW')
      WRITE (2, '(''AMMIWINS Analysis'')')
      WRITE (2, '(''Input  file:  '', A40)')  FN1
      WRITE (2, '(''Output file:  '', A40, /)')  FN2
C     Read MATMODEL machine-readable input file, first line.
      READ (1, '(4X,I1,21X,I5,6X,I5,31X,F17.5)')  NAX, NG, NE, GM
C     Take user requests.
      WRITE (*, '('' The input file provides'', I2,
     1 '' AMMI IPCA axes.'', /)')  NAX
      LAM(2) = 0
      DO 200 ND=1,NAX
   11 WRITE (*, '('' Do you want AMMI'', I1, '' mega-environment'',
     1 '' analysis?  <Y, N>'')')  ND
      READ (*, '(A1)')  T
      IF     (T .EQ. 'Y'  .OR.  T .EQ. 'y')  THEN
      LAM(ND) = 1
      NXL = ND
      GO TO 200
      ELSEIF (T .EQ. 'N'  .OR.  T .EQ. 'n')  THEN
      LAM(ND) = 0
      GO TO 200
      ELSE
      GO TO 11
      ENDIF
  200 CONTINUE
C     Check and assign memory.
      NMR = NG + ((4 + NXL) * (NG + NE))
      IF (NMR .LE. LNGTH4)  THEN
      KEM = NG
      KGC = KEM + NE
      KEC = KGC + NG
      KGL = KEC + NE
      KEL = KGL + NG
      KGW = KEL + NE
      KEW = KGW + NG
      KGT = KEW + NE
      KGA = KGT + NG
      KEA = KGA + (NG * NXL)
      ELSE
      WRITE (*, '('' Data requirements exceed memory limits.'')')
      GO TO 70
      ENDIF
C     Read remainder of input file.
      DO 100 NGD=1,NG
  100 READ (1, '(7X, A4, F20.5, 7F20.7)')  C(KGC+NGD), V(NGD),
     1 (V(KGA+((NX-1)*NG)+NGD), NX=1,NXL)
      DO 101 NED=1,NE
  101 READ (1, '(7X, A4, F20.5, 7F20.7)')  C(KEC+NED), V(KEM+NED),
     1 (V(KEA+((NX-1)*NE)+NED), NX=1,NXL)
      CLOSE (1)
C     AMMI winners analysis.
      IF (LAM(1) .EQ. 1)  CALL AMMI1
      IF (LAM(2) .EQ. 1)  CALL AMMI2
      IF (NXL .GE. 3)  THEN
      DO 110 ND=3,NXL
  110 IF (LAM(ND) .EQ. 1)  CALL AMMIH(ND)
      ENDIF
C     Conclude analysis.
   70 CLOSE (2)
      WRITE (*, '(/, '' AMMIWINS Analysis completed.'')')
      STOP
      END
      SUBROUTINE AMMI1
      REAL*4       SEP(1600), BP(1700)
      INTEGER*4    L(16000), NGW(1600)
      CHARACTER*4  C(16000)
      COMMON  /DATA/  V(16000)
      COMMON  /PARM/  NG, NE, GM
      COMMON  /SETO/  KEM,KGC,KEC,KGL,KEL,KGW,KEW,KGT,KGA,KEA
      COMMON  /MATR/  M(4900)
      EQUIVALENCE  (V(1), L(1), C(1))
      EQUIVALENCE  (M(1),NGW(1)), (M(1601),SEP(1)), (M(3201),BP(1))
C     Notify.
      WRITE (*, '('' AMMI1 analysis begins.'')')
C     Initially accept all genotypes.
      NEL = 0
      DO 100 NGD=1,NG
  100 L(KGL+NGD) = 1
C     Find genotype with highest mean.
      GHM = 0.0
      DO 101 NGD=1,NG
      IF (L(KGL+NGD) .NE. 1)  GO TO 101
      R = V(NGD)
      IF (R .LE. GHM)  GO TO 101
      GHM = R
      NGH = NGD
  101 CONTINUE
C     Identify genotype left and right winners.
      NN = 0
      DO 102 NGD=1,NG
      IF (L(KGL+NGD) .NE. 1)  GO TO 102
      R = V(KGA+NGD)
      NN = NN + 1
      IF (NN .EQ. 1)  THEN
      GXL = R
      GXR = R
      NGXL = NGD
      NGXR = NGD
      ELSE
      IF (R .LT. GXL)  THEN
      GXL = R
      NGXL = NGD
      ENDIF
      IF (R .GT. GXR)  THEN
      GXR = R
      NGXR = NGD
      ENDIF
      ENDIF
  102 CONTINUE
C     Find all switches.
      IGW = 1
      NGW(IGW) = NGXR
      L(KGL+NGXR) = -1
      AR = V(NGXR)
      BR = V(KGA+NGXR)
   20 NGSW = 0
      DO 110 NGD=1,NG
      IF (L(KGL+NGD) .NE. 1)  GO TO 110
      S = (AR - V(NGD)) / (V(KGA+NGD) - BR)
      IF (IGW .GE. 2  .AND.  S .GE. SR)  GO TO 110
      IF (NGSW .EQ. 0)  THEN
      SN = S
      NGSW = NGD
      ELSE
      IF (S.GT.SN .OR. (S.EQ.SN .AND. V(KGA+NGD).LT.V(KGA+NGSW)))  THEN
      SN = S
      NGSW = NGD
      ENDIF
      ENDIF
  110 CONTINUE
      IGW = IGW + 1
      NGW(IGW) = NGSW
      SEP(IGW) = SN
      L(KGL+NGSW) = -1
      IF (NGSW .EQ. NGXL)  GO TO 30
      AR = V(NGSW)
      BR = V(KGA+NGSW)
      SR = SN
      GO TO 20
C     Rank environment IPCA 1 scores; find max and min.
   30 CALL INDEX(NE, KEA, KEL)
      NEMIN = L(KEL+NE)
      NEMAX = L(KEL+1)
      EMIN = V(KEA+NEMIN)
      EMAX = V(KEA+NEMAX)
C     Count wins for each genotype.
      NLD = 1
      NGWIN = NGW(NLD)
      SWITCH = SEP(NLD+1)
      DO 130 NGD=1,NG
  130 L(KGW+NGD) = 0
      DO 131 NED=1,NE
      NER = L(KEL+NED)
      EP1 = V(KEA+NER)
      IF (EP1 .GE. SWITCH  .OR.  NLD .EQ. IGW)  GO TO 36
   35 NLD = NLD + 1
      NGWIN = NGW(NLD)
      SWITCH = SEP(NLD+1)
      IF (EP1 .LT. SWITCH  .AND.  NLD .LT. IGW)  GO TO 35
   36 L(KEW+NER) = NGWIN
  131 L(KGW+NGWIN) = L(KGW+NGWIN) + 1
      NGMIN = L(KEW+NEMIN)
      NGMAX = L(KEW+NEMAX)
C     Write list of genotype winners.
      WRITE (2, '(''AMMI1 Winners, Results for Genotypes'')')
      R = V(NGMAX) + (V(KGA+NGMAX) * EMAX)
      WRITE (2, '(''Win at score'', F20.7, '' is'', I6, 2X, A4,
     1 ''  Nom. Yld.'', F15.5)')  EMAX, NGMAX, C(KGC+NGMAX), R
      WRITE (2, '(''Win by main effect at score  0.0 is'',
     1 I6, 2X, A4, ''  Nom. Yld.'', F15.5)')  NGH, C(KGC+NGH), GHM
      R = V(NGMIN) + (V(KGA+NGMIN) * EMIN)
      WRITE (2, '(''Win at score'', F20.7, '' is'', I6, 2X, A4,
     1 ''  Nom. Yld.'', F15.5)')  EMIN, NGMIN, C(KGC+NGMIN), R
      WRITE (2, '(/, ''Gen. Winners     Env. IPCA 1 Switch'',
     1 ''     Nominal Yield'')')
      DO 135 NLD=1,IGW
      NWN = NGW(NLD)
      IF     (NWN .EQ. NGMIN)  THEN
      YN = V(NGMIN) + (V(KGA+NGMIN) * EMIN)
      WRITE (2, '(I5, 2X, A4, 24X, F15.5, '' at'', F20.7)')
     1 NGMIN, C(KGC+NGMIN), YN, EMIN
      ELSEIF (NWN .EQ. NGH)  THEN
      WRITE (2, '(I5, 2X, A4, 24X, F15.5, '' at'', 11X, ''0.0'')')
     1 NGH, C(KGC+NGH), GHM
      ELSEIF (NWN .EQ. NGMAX)  THEN
      YN = V(NGMAX) + (V(KGA+NGMAX) * EMAX)
      WRITE (2, '(I5, 2X, A4, 24X, F15.5, '' at'', F20.7)')
     1 NGMAX, C(KGC+NGMAX), YN, EMAX
      ELSE
      WRITE (2, '(I5, 2X, A4)')  NWN, C(KGC+NWN)
      ENDIF
      IF (NLD .EQ. IGW)  GO TO 135
      R = SEP(NLD+1)
      IF (R .GE. EMIN  .AND.  R .LE. EMAX)  THEN
      YN = V(NWN) + (V(KGA+NWN) * SEP(NLD+1))
      WRITE (2, '(11X, F20.7, F19.5)')  R, YN
      ELSE
      WRITE (2, '(11X, F20.7, ''   (Outside range.)'')')  R
      ENDIF
  135 CONTINUE
C     Write results for mega-environments, also to screen.
      WRITE (2, '(/, ''AMMI1 Winners, Results for'',
     1 '' Mega-environments'')')
      WRITE (*, '(/, '' AMMI1 Winners, Results for'',
     1 '' Mega-environments'')')
      WRITE (2, '(/, ''Genotype       Wins      Exp. Yield | '',
     1 A4, '' Exp. Yield    Boost (%)'')')  C(KGC+NGH)
      WRITE (*, '(/, '' Genotype       Wins      Exp. Yield | '',
     1 A4, '' Exp. Yield    Boost (%)'')')  C(KGC+NGH)
      V0A = GHM - GM
      V0M = V(KGA+NGH)
      Y1T = 0.0
      DO 141 NLD=1,IGW
      NGWIN = NGW(NLD)
      IF (L(KGW+NGWIN) .EQ. 0)  GO TO 141
      V1A = V(NGWIN) - GM
      V1M = V(KGA+NGWIN)
      Y0 = 0.0
      Y1 = 0.0
      DO 142 NED=1,NE
      IF (L(KEW+NED) .NE. NGWIN)  GO TO 142
      R = V0A + V(KEM+NED) + (V0M * V(KEA+NED))
      Y0 = Y0 + R
      R = V1A + V(KEM+NED) + (V1M * V(KEA+NED))
      Y1 = Y1 + R
  142 CONTINUE
      Y1T = Y1T + Y1
      R = L(KGW+NGWIN)
      Y0 = Y0 / R
      Y1 = Y1 / R
      YB = 100.0 * ((Y1 - Y0) / Y0)
      BP(NLD) = YB
      WRITE (2, '(I5,2X,A4,I8,F16.5,'' |'',F16.5,F10.2)')
     1 NGWIN, C(KGC+NGWIN), L(KGW+NGWIN), Y1, Y0, YB
      WRITE (*, '(I6,2X,A4,I8,F16.5,'' |'',F16.5,F10.2)')
     1 NGWIN, C(KGC+NGWIN), L(KGW+NGWIN), Y1, Y0, YB
  141 CONTINUE
      FE = NE
      Y0T = V(NGH)
      Y1T = Y1T / FE
      YB = 100.0 * ((Y1T - Y0T) / Y0T)
      WRITE (2, '(''Overall    '',I8,F16.5,'' |'',F16.5,
     1 F10.2)')  NE, Y1T, Y0T, YB
      WRITE (*, '('' Overall    '',I8,F16.5,'' |'',F16.5,
     1 F10.2)')  NE, Y1T, Y0T, YB 
C     Eliminate minor winners.
      KGEL = 0
      WRITE (*, '(/, '' Enter minor genotypes to eliminate,'',
     1 '' 0 to end list;'', /, ''OR enter negative -N to'',
     2 '' eliminate entries with N or fewer wins.'')')
   40 READ (*, *)  NGEL
      IF (KGEL .EQ. 0  .AND.  NGEL .EQ. 0)  GO TO 50
      IF (KGEL .GE. 1  .AND.  NGEL .EQ. 0)  GO TO 41
      IF (NGEL .LT. 0)  THEN
      MIN = -NGEL
      DO 143 NGD=1,NG
  143 L(KGL+NGD) = 1
      DO 144 NLD=1,IGW
      NGWIN = NGW(NLD)
      IF (L(KGW+NGWIN) .GT. MIN)  GO TO 144
      L(KGL+NGWIN) = -3
      KGEL = KGEL + 1
  144 CONTINUE    
      ELSE
      IF (NGEL .LE. NG)  THEN
      L(KGL+NGEL) = -3
      KGEL = KGEL + 1
      ELSE
      WRITE (*, '('' Entry outside range; try again.'')')
      ENDIF
      GO TO 40
      ENDIF
C     Write list of eliminated winners.
   41 WRITE (2, '(/, ''AMMI1 Winners, Genotypes'',
     1 '' Eliminated by User''''s Requests'')')
      IF (KGEL .EQ. 0)  THEN
      WRITE (2, '(''No winners were eliminated.'')')
      ELSE
      WRITE (2, '(I3, '' winners were eliminated,'',
     1 '' as follows:'')')  KGEL
      DO 150 NLD=1,IGW
      NGWIN = NGW(NLD)
      IF (L(KGL+NGWIN) .NE. -3)  GO TO 150
      WRITE (2, '(I5, 2X, A4)')  NGWIN, C(KGC+NGWIN)
  150 CONTINUE
      ENDIF
      WRITE (2, '('' '')')
C     Skip second analysis if no winners were eliminated.
      IF (KGEL .EQ. 0)  GO TO 50
C     Repeat analysis, allowing no new winners.
      IF (L(KGL+NGH) .EQ. -3)  THEN
      GHM = 0.0
      DO 160 NGD=1,NG
      IF (L(KGL+NGD) .EQ. -3)  GO TO 160
      R = V(NGD)
      IF (R .LE. GHM)  GO TO 160
      GHM = R
      NGH = NGD
  160 CONTINUE
      ENDIF
      NLD = 0
      DO 161 ND=1,IGW
      NWN = NGW(ND)
      IF (L(KGL+NWN) .EQ. -3)  GO TO 161
      NLD = NLD + 1
      NGW(NLD) = NWN
  161 CONTINUE
      IGW = IGW - KGEL
      DO 162 NLD=1,IGW-1
      NL2 = NLD + 1
      NG1 = NGW(NLD)
      NG2 = NGW(NL2)
  162 SEP(NL2) = (V(NG1)-V(NG2))/(V(KGA+NG2)-V(KGA+NG1))
      WRITE (2, '(/, ''AMMI1 Winners, Results for'',
     1 '' Genotypes After Eliminations'')')
      R = V(NGMAX) + (V(KGA+NGMAX) * EMAX)
      WRITE (2, '(''Win at score'', F20.7, '' is'', I6, 2X, A4,
     1 ''  Nom. Yld.'', F15.5)')  EMAX, NGMAX, C(KGC+NGMAX), R
      WRITE (2, '(''Win by main effect at score  0.0 is'',
     1 I6, 2X, A4, ''  Nom. Yld.'', F15.5)')  NGH, C(KGC+NGH), GHM
      R = V(NGMIN) + (V(KGA+NGMIN) * EMIN)
      WRITE (2, '(''Win at score'', F20.7, '' is'', I6, 2X, A4,
     1 ''  Nom. Yld.'', F15.5)')  EMIN, NGMIN, C(KGC+NGMIN), R
      WRITE (2, '(/, ''Gen. Winners     Env. IPCA 1 Switch'',
     1 ''     Nominal Yield'')')
      DO 163 NLD=1,IGW
      NWN = NGW(NLD)
      IF     (NWN .EQ. NGMIN)  THEN
      YN = V(NGMIN) + (V(KGA+NGMIN) * EMIN)
      WRITE (2, '(I5, 2X, A4, 24X, F15.5, '' at'', F20.7)')
     1 NGMIN, C(KGC+NGMIN), YN, EMIN
      ELSEIF (NWN .EQ. NGH)  THEN
      WRITE (2, '(I5, 2X, A4, 24X, F15.5, '' at'', 11X, ''0.0'')')
     1 NGH, C(KGC+NGH), GHM
      ELSEIF (NWN .EQ. NGMAX)  THEN
      YN = V(NGMAX) + (V(KGA+NGMAX) * EMAX)
      WRITE (2, '(I5, 2X, A4, 24X, F15.5, '' at'', F20.7)')
     1 NGMAX, C(KGC+NGMAX), YN, EMAX
      ELSE
      WRITE (2, '(I5, 2X, A4)')  NWN, C(KGC+NWN)
      ENDIF
      IF (NLD .EQ. IGW)  GO TO 163
      R = SEP(NLD+1)
      IF (R .GE. EMIN  .AND.  R .LE. EMAX)  THEN
      YN = V(NWN) + (V(KGA+NWN) * SEP(NLD+1))
      WRITE (2, '(11X, F20.7, F19.5)')  R, YN
      ELSE
      WRITE (2, '(11X, F20.7, ''   (Outside range.)'')')  R
      ENDIF
  163 CONTINUE
      NLD = 1
      NGWIN = NGW(NLD)
      SWITCH = SEP(NLD+1)
      DO 165 NGD=1,NG
  165 L(KGW+NGD) = 0
      DO 166 NED=1,NE
      NER = L(KEL+NED)
      EP1 = V(KEA+NER)
      IF (EP1 .GE. SWITCH  .OR.  NLD .EQ. IGW)  GO TO 46
   45 NLD = NLD + 1
      NGWIN = NGW(NLD)
      SWITCH = SEP(NLD+1)
      IF (EP1 .LT. SWITCH  .AND.  NLD .LT. IGW)  GO TO 45
   46 L(KEW+NER) = NGWIN
  166 L(KGW+NGWIN) = L(KGW+NGWIN) + 1
      NGMIN = L(KEW+NEMIN)
      NGMAX = L(KEW+NEMAX)
      WRITE (2, '(/, ''AMMI1 Winners, Results for'',
     1 '' Mega-environments After Eliminations'')')
      WRITE (2, '(/, ''Genotype       Wins      Exp. Yield | '',
     1 A4, '' Exp. Yield    Boost (%)'')')  C(KGC+NGH)
      V0A = GHM - GM
      V0M = V(KGA+NGH)
      Y1T = 0.0
      DO 170 NLD=1,IGW
      NGWIN = NGW(NLD)
      IF (L(KGW+NGWIN) .EQ. 0)  GO TO 170
      V1A = V(NGWIN) - GM
      V1M = V(KGA+NGWIN)
      Y0 = 0.0
      Y1 = 0.0
      DO 171 NED=1,NE
      IF (L(KEW+NED) .NE. NGWIN)  GO TO 171
      R = V0A + V(KEM+NED) + (V0M * V(KEA+NED))
      Y0 = Y0 + R
      R = V1A + V(KEM+NED) + (V1M * V(KEA+NED))
      Y1 = Y1 + R
  171 CONTINUE
      Y1T = Y1T + Y1
      R = L(KGW+NGWIN)
      Y0 = Y0 / R
      Y1 = Y1 / R
      YB = 100.0 * ((Y1 - Y0) / Y0)
      BP(NLD) = YB
      WRITE (2, '(I5,2X,A4,I8,F16.5,'' |'',F16.5,F10.2)')
     1 NGWIN, C(KGC+NGWIN), L(KGW+NGWIN), Y1, Y0, YB
  170 CONTINUE
      FE = NE
      Y0T = V(NGH)
      Y1T = Y1T / FE
      YB = 100.0 * ((Y1T - Y0T) / Y0T)
      WRITE (2, '(''Overall    '',I8,F16.5,'' |'',F16.5,
     1 F10.2)')  NE, Y1T, Y0T, YB
C     Write results for environments.
   50 IF (KGEL .EQ. 0)  THEN
      WRITE (2, '(/, ''AMMI1 Winners, Results for'',
     1 '' Environments'')')
      ELSE
      WRITE (2, '(/, ''AMMI1 Winners, Results for'',
     1 '' Environments After Eliminations'')')
      ENDIF
      WRITE (2, '(''Environments are grouped in'', I3,
     1 '' mega-environments.'')')  IGW
      V0A = V(NGH) - GM
      V0M = V(KGA+NGH)
      NED = 0
      DO 180 NLD=1,IGW
      NWN = NGW(NLD)
      V1A = V(NWN) - GM
      V1M = V(KGA+NWN)
      NME = L(KGW+NWN)
      WRITE (2, '(/, ''Mega-environment'', I3, '' with'', I5,
     1 '' wins by '', A4, '' gives boost of'', F7.2, ''%'')')
     2 NLD, NME, C(KGC+NWN), BP(NLD)
      WRITE (2, '(''Environment        IPCA 1 Score     '',
     1 ''Exp. Yield      Boost (%)'')')
      DO 181 NMD=1,NME
      NED = NED + 1
      NER = L(KEL+NED)
      RV = V(KEA+NER)
      YN = V1A + V(KEM+NER) + (V1M * RV)
      Y0 = V0A + V(KEM+NER) + (V0M * RV)
      R = 100.0 * ((YN - Y0) / Y0)
  181 WRITE (2, '(I5, 2X, A4, F20.7, F15.5, F11.2)')
     1 NER, C(KEC+NER), RV, YN, R
  180 CONTINUE
C     Notify.
      WRITE (*, '(/, '' AMMI1 analysis completed.'')')
      RETURN
      END
      SUBROUTINE AMMI2
      INTEGER*4    L(16000), IGW(26), LGW(26)
      REAL*4  Y0(26), Y2(26)
      CHARACTER*4  C(16000)
      CHARACTER*1  T, S(26)
      COMMON  /DATA/  V(16000)
      COMMON  /PARM/  NG, NE, GM
      COMMON  /SETO/  KEM,KGC,KEC,KGL,KEL,KGW,KEW,KGT,KGA,KEA
      COMMON  /MATR/  M(4900)
      EQUIVALENCE  (V(1), L(1), C(1))
      DATA  S  /'1', '2', '3', '4', '5', '6', '7', '8', '9',
     1 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K',
     2 'L', 'M', 'N', 'O', 'P', 'X'/
C     Notify.
      WRITE (*, '(/, '' AMMI2 analysis begins.'')')
C     Take default range from environments; offer choice.
      NE1N = 1
      NE1X = 1
      NE2N = 1
      NE2X = 1
      PC1N = V(KEA+1)
      PC1X = V(KEA+1)
      PC2N = V(KEA+NE+1)
      PC2X = V(KEA+NE+1)
      DO 100 NED=1,NE
      R = V(KEA+NED)
      IF (R .LT. PC1N)  THEN
      PC1N = R
      NE1N = NED
      ENDIF
      IF (R .GT. PC1X)  THEN
      PC1X = R
      NE1X = NED
      ENDIF
      R = V(KEA+NE+NED)
      IF (R .LT. PC2N)  THEN
      PC2N = R
      NE2N = NED
      ENDIF
      IF (R .GT. PC2X)  THEN
      PC2X = R
      NE2X = NED
      ENDIF
  100 CONTINUE
      WRITE (*, '(/, '' Default range for IPCA 1 is'', /,
     1 F20.7, '' for environment '', A4, '' to'', /,
     2 F20.7, '' for environment '', A4, ''.'')')
     3 PC1N, C(KEC+NE1N), PC1X, C(KEC+NE1X)
      WRITE (*, '('' Default range for IPCA 2 is'', /,
     1 F20.7, '' for environment '', A4, '' to'', /,
     2 F20.7, '' for environment '', A4, ''.'')')
     3 PC2N, C(KEC+NE2N), PC2X, C(KEC+NE2X)
   10 WRITE (*, '(/, '' Do you want these defaults?  <Y, N>'')')
      READ (*, '(A1)')  T
      IF (T .EQ. 'Y'  .OR.  T .EQ. 'y')  GO TO 15
      IF (.NOT. (T .EQ. 'N'  .OR.  T .EQ. 'n'))  GO TO 10
   11 WRITE (*, '('' Enter minimum for IPCA 1.'')')
      READ (*, *)  PC1N
      WRITE (*, '('' Enter maximum for IPCA 1.'')')
      READ (*, *)  PC1X
      WRITE (*, '('' Enter minimum for IPCA 2.'')')
      READ (*, *)  PC2N
      WRITE (*, '('' Enter maximum for IPCA 2.'')')
      READ (*, *)  PC2X
   12 WRITE (*, '(/, '' Are these four values correct?  <Y, N>'')')
      IF (T .EQ. 'Y'  .OR.  T .EQ. 'y')  GO TO 15
      IF (T .EQ. 'N'  .OR.  T .EQ. 'n')  GO TO 11
      GO TO 12
C     Initially accept all genotypes.
   15 KGEL = 0
      DO 110 NGD=1,NG
  110 L(KGL+NGD) = 1
C     Find genotype with highest mean.
      GHM = 0.0
      DO 112 NGD=1,NG
      IF (L(KGL+NGD) .EQ. 0)  GO TO 112
      R = V(NGD)
      IF (R .LE. GHM)  GO TO 112
      GHM = R
      NGH = NGD
  112 CONTINUE
C     Zero genotype wins; real.
      DO 115 NGD=1,NG
  115 V(KGW+NGD) = 0.0
C     Determine winner at each pixel; count wins.
      R1 = (PC1X - PC1N) / 69.0
      R2 = (PC2X - PC2N) / 69.0
      NP = 0
      DO 120 N2D=1,70
      PC2 = PC2X - (FLOAT(N2D - 1) * R2)
      DO 121 N1D=1,70
      PC1 = PC1N + (FLOAT(N1D - 1) * R1)
      YX = 0.0
      DO 122 NGD=1,NG
      IF (L(KGL+NGD) .EQ. 0)  GO TO 122
      R = V(NGD) + (V(KGA+NGD) * PC1) + (V(KGA+NG+NGD) * PC2)
      IF (R .LE. YX)  GO TO 122
      YX = R
      NWN = NGD
  122 CONTINUE
      NP = NP + 1
      M(NP) = NWN
  121 V(KGW+NWN) = V(KGW+NWN) + 1.0
  120 CONTINUE
C     Rank winners and construct key.
      CALL INDEX(NG, KGW, KGT)
      NME = 0
      NPX = 0
      LGW(26) = 0
      DO 130 NMD=1,NG
      IF (NMD .LT. 26)  THEN
      I = L(KGT+NMD)
      IGW(NMD) = I
      I = V(KGW+I)
      LGW(NMD) = I
      ELSE
      I = V(KGW+L(KGT+NMD))
      LGW(26) = LGW(26) + I
      ENDIF
      NPX = NPX + I
      NME = NME + 1
      IF (NPX .EQ. 4900)  GO TO 25
  130 CONTINUE
   25 NMEP = NME
      IF (NMEP .GT. 26)  NMEP = 26
      DO 131 NMD=1,NME
      I = L(KGT+NMD)
      NTR = NMD
      IF (NTR .GT. 26)  NTR = 26
  131 L(KGW+I) = NTR
C     Calculate expected yields.
      DO 135 NMD=1,NMEP
      Y0(NMD) = 0.0
  135 Y2(NMD) = 0.0
      V1H = V(KGA+NGH)
      V2H = V(KGA+NG+NGH)
      NX = 0
      DO 136 N2D=1,70
      PC2 = PC2X - (FLOAT(N2D - 1) * R2)
      DO 137 N1D=1,70
      PC1 = PC1N + (FLOAT(N1D - 1) * R1)
      NX = NX + 1
      NGW = M(NX)
      NET = L(KGW+NGW)
      Y = GHM + (V1H * PC1) + (V2H * PC2)
      Y0(NET) = Y0(NET) + Y
      Y = V(NGW) + (V(KGA+NGW) * PC1) + (V(KGA+NG+NGW) * PC2)
  137 Y2(NET) = Y2(NET) + Y
  136 CONTINUE
      Y0T = 0.0
      Y2T = 0.0
      DO 138 NMD=1,NMEP
      R = LGW(NMD)
      Y = Y0(NMD)
      Y0(NMD) = Y / R
      Y0T = Y0T + Y
      Y = Y2(NMD)
      Y2(NMD) = Y / R
  138 Y2T = Y2T + Y
      Y0T = Y0T / 4900.0
      Y2T = Y2T / 4900.0
C     Write results for genotypes.
      WRITE (2, '(/, ''AMMI2 Winners, Results for'',
     1 '' Genotypes'')')
      WRITE (2, '(''Calculated using a 70x70 grid over'', /,
     1 ''IPCA 1 from '', F20.7, '' to '', F20.7, /,
     2 ''IPCA 2 from '', F20.7, '' to '', F20.7)')
     3 PC1N, PC1X, PC2N, PC2X
      WRITE (2, '(/, ''Key  Genotype       Wins      '',
     1 ''Exp. Yield | '', A4, '' Exp. Yield    Boost (%)'')')
     2 C(KGC+NGH) 
      DO 140 NMD=1,NMEP
      YB = 100.0 * ((Y2(NMD) - Y0(NMD)) / Y0(NMD))
      IF (NMD .LT. 26)  THEN
      NGW = IGW(NMD)
      WRITE (2, '('' '', A1, I8, 2X, A4, I8, F16.5, '' |'',
     1 F16.5, F10.2)')  S(NMD), NGW, C(KGC+NGW), LGW(NMD),
     2 Y2(NMD), Y0(NMD), YB
      ELSE
      WRITE (2, '('' '', A1, I8, '' Other'', I8, F16.5, '' |'',
     1 F16.5, F10.2)')  S(26), NME-25, LGW(NMD), Y2(NMD),
     2 Y0(NMD), YB
      ENDIF
  140 CONTINUE
      YB = 100.0 * ((Y2T - Y0T) / Y0T)
      WRITE (2, '(''Overall             4900'', F16.5, '' |'',
     1 F16.5, F10.2, /)')  Y2T, Y0T, YB
      DO 141 N1D=1,70
      NS = 1 + ((N1D - 1) * 70)
  141 WRITE (2, '(70A1)')  (S(L(KGW+M(ND))), ND=NS,NS+69)
C     Results for mega-environments, also to screen on first pass.
   30 CONTINUE
      DO 150 NGD=1,NG
  150 V(KGW+NGD) = 0.0
      DO 151 NED=1,NE
      AD = V(KEM+NED) - GM
      PC1 = V(KEA+NED)
      PC2 = V(KEA+NE+NED)
      YX = 0.0
      DO 152 NGD=1,NG
      IF (L(KGL+NGD) .EQ. 0)  GO TO 152
      R = V(NGD) + AD + (V(KGA+NGD)*PC1) + (V(KGA+NG+NGD)*PC2)
      IF (R .LE. YX)  GO TO 152
      YX = R
      NWN = NGD
  152 CONTINUE
      L(KEW+NED) = NWN
  151 V(KGW+NWN) = V(KGW+NWN) + 1.0
      CALL INDEX(NG, KGW, KGT)
      NME = 0
      NEX = 0
      LGW(26) = 0
      DO 155 NMD=1,NG
      IF (NMD .LT. 26)  THEN
      I = L(KGT+NMD)
      IGW(NMD) = I
      I = V(KGW+I)
      LGW(NMD) = I
      ELSE
      I = V(KGW+L(KGL+NMD))
      LGW(26) = LGW(26) + I
      ENDIF
      NEX = NEX + I
      NME = NME + 1
      IF (NEX .EQ. NE)  GO TO 31
  155 CONTINUE
   31 NMEP = NME
      IF (NMEP .GT. 26)  NMEP = 26
      DO 156 NMD=1,NME
      I = L(KGT+NMD)
      NTR = NMD
      IF (NTR .GT. 26)  NTR = 26
  156 L(KGW+I) = NTR
      DO 157 NMD=1,NMEP
      Y0(NMD) = 0.0
  157 Y2(NMD) = 0.0
      V1H = V(KGA+NGH)
      V2H = V(KGA+NG+NGH)
      DO 158 NED=1,NE
      AD = V(KEM+NED) - GM
      PC1 = V(KEA+NED)
      PC2 = V(KEA+NE+NED)
      NGW = L(KEW+NED)
      NET = L(KGW+NGW)
      Y = GHM + AD + (V1H * PC1) + (V2H * PC2)
      Y0(NET) = Y0(NET) + Y
      Y = V(NGW) + AD + (V(KGA+NGW)*PC1) + (V(KGA+NG+NGW)*PC2)
  158 Y2(NET) = Y2(NET) + Y
      Y0T = 0.0
      Y2T = 0.0
      DO 159 NMD=1,NMEP
      R = LGW(NMD)
      Y = Y0(NMD)
      Y0(NMD) = Y / R
      Y0T = Y0T + Y
      Y = Y2(NMD)
      Y2(NMD) = Y / R
  159 Y2T = Y2T + Y
      Y0T = Y0T / FLOAT(NE)
      Y2T = Y2T / FLOAT(NE)
      IF (KGEL .EQ. 0)  WRITE (2, '(/, ''AMMI2 Winners,'',
     1 '' Results for Mega-environments'')')
      IF (KGEL .NE. 0)  WRITE (2, '(/, ''AMMI2 Winners,'',
     1 '' Results for Mega-environments After Eliminations'')')
      WRITE (2, '(/, ''Genotype       Wins      '',
     1 ''Exp. Yield | '', A4, '' Exp. Yield    Boost (%)'')')
     2 C(KGC+NGH) 
      DO 160 NMD=1,NMEP
      YB = 100.0 * ((Y2(NMD) - Y0(NMD)) / Y0(NMD))
      IF (NMD .LT. 26)  THEN
      NGW = IGW(NMD)
      WRITE (2, '(I5, 2X, A4, I8, F16.5, '' |'',
     1 F16.5, F10.2)')  NGW, C(KGC+NGW), LGW(NMD),
     2 Y2(NMD), Y0(NMD), YB
      ELSE
      WRITE (2, '(I5 ,'' Other'', I8, F16.5, '' |'', F16.5,
     1 F10.2)')  NME-25, LGW(NMD), Y2(NMD), Y0(NMD), YB
      ENDIF
  160 CONTINUE
      YB = 100.0 * ((Y2T - Y0T) / Y0T)
      WRITE (2, '(''Overall       '', I5, F16.5, '' |'',
     1 F16.5, F10.2, /)')  NE, Y2T, Y0T, YB
      IF (KGEL .EQ. 0)  THEN
      WRITE (*, '(/, '' AMMI2 Winners, Results for'',
     1 '' Mega-environments'')')
      WRITE (*, '(/, '' Genotype       Wins      '',
     1 ''Exp. Yield | '', A4, '' Exp. Yield    Boost (%)'')')
     2 C(KGC+NGH) 
      DO 161 NMD=1,NMEP
      YB = 100.0 * ((Y2(NMD) - Y0(NMD)) / Y0(NMD))
      IF (NMD .LT. 26)  THEN
      NGW = IGW(NMD)
      WRITE (*, '(I6, 2X, A4, I8, F16.5, '' |'',
     1 F16.5, F10.2)')  NGW, C(KGC+NGW), LGW(NMD),
     2 Y2(NMD), Y0(NMD), YB
      ELSE
      WRITE (*, '(I8, I10, F16.5, '' |'', F16.5,
     1 F10.2)')  NME-25, LGW(NMD), Y2(NMD), Y0(NMD), YB
      ENDIF
  161 CONTINUE
      YB = 100.0 * ((Y2T - Y0T) / Y0T)
      WRITE (*, '('' Overall       '', I5, F16.5, '' |'',
     1 F16.5, F10.2, /)')  NE, Y2T, Y0T, YB
      ENDIF
C     Eliminate minor winners.
      IF (KGEL .EQ. 0)  THEN
      WRITE (*, '(/, '' Enter minor genotypes to eliminate,'',
     1 '' 0 to end list;'', /, ''OR enter negative -N to'',
     2 '' eliminate entries with N or fewer wins.'')')
   40 READ (*, *)  NGEL
      IF (KGEL .EQ. 0  .AND.  NGEL .EQ. 0)  GO TO 50
      IF (KGEL .GE. 1  .AND.  NGEL .EQ. 0)  GO TO 41
      IF (NGEL .LT. 0)  THEN
      MIN = -NGEL
      DO 170 NGD=1,NG
  170 L(KGL+NGD) = 1
      DO 171 NLD=1,NME
      NGWIN = IGW(NLD)
      IF (LGW(NLD) .GT. MIN)  GO TO 171
      L(KGL+NGWIN) = 0
      KGEL = KGEL + 1
  171 CONTINUE    
      ELSE
      IF (NGEL .LE. NG)  THEN
      L(KGL+NGEL) = 0
      KGEL = KGEL + 1
      ELSE
      WRITE (*, '('' Entry outside range; try again.'')')
      ENDIF
      GO TO 40
      ENDIF
   41 IF (L(KGL+NGH) .EQ. 0)  THEN
      GHM = 0.0
      DO 175 NGD=1,NG
      IF (L(KGL+NGD) .EQ. 0)  GO TO 175
      R = V(NGD)
      IF (R .LE. GHM)  GO TO 175
      GHM = R
      NGH = NGD
  175 CONTINUE
      ENDIF
      DO 177 NGD=1,NG
  177 IF (V(KGW+NGD) .EQ. 0.0)  L(KGL+NGD) = 0
C     Write list of eliminated winners.
      WRITE (2, '(/, ''AMMI2 Winners, Genotypes'',
     1 '' Eliminated by User''''s Requests'')')
      IF (KGEL .EQ. 0)  THEN
      WRITE (2, '(''No winners were eliminated.'')')
      ELSE
      WRITE (2, '(I3, '' winners were eliminated,'',
     1 '' as follows:'')')  KGEL
      DO 172 NLD=1,NME
      NGWIN = IGW(NLD)
      IF (L(KGL+NGWIN) .NE. 0)  GO TO 172
      WRITE (2, '(I5, 2X, A4)')  NGWIN, C(KGC+NGWIN)
  172 CONTINUE
      ENDIF
      WRITE (2, '('' '')')
      GO TO 30
      ENDIF
C     Write results for environments.
   50 IF (KGEL .EQ. 0)  THEN
      WRITE (2, '(/, ''AMMI2 Winners, Results for'',
     1 '' Environments'')')
      ELSE
      WRITE (2, '(/, ''AMMI2 Winners, Results for'',
     1 '' Environments After Eliminations'')')
      ENDIF
      WRITE (2, '(''Environments are grouped in'', I3,
     1 '' mega-environments.'')')  NME
      V0A = V(NGH) - GM
      V0M1 = V(KGA+NGH)
      V0M2 = V(KGA+NG+NGH)
      DO 180 NLD=1,NME
      NGW = L(KGT+NLD)
      V2A = V(NGW) - GM
      V2M1 = V(KGA+NGW)
      V2M2 = V(KGA+NG+NGW)
      NMW = 0
      Y0T = 0.0
      Y2T = 0.0
      DO 181 NED=1,NE
      IF (L(KEW+NED) .NE. NGW)  GO TO 181
      NMW = NMW + 1
      R1 = V(KEA+NED)
      R2 = V(KEA+NE+NED)
      V0 = V0A + V(KEM+NED) + (V0M1*R1) + (V0M2*R2)
      V2 = V2A + V(KEM+NED) + (V2M1*R1) + (V2M2*R2)
      Y0T = Y0T + V0
      Y2T = Y2T + V2
  181 CONTINUE
      YB = 100.0 * ((Y2T - Y0T) / Y0T)
      WRITE (2, '(/, ''Mega-environment'', I3, '' with'', I5,
     1 '' wins by '', A4, '' gives boost of'', F7.2, ''%'')')
     2 NLD, NMW, C(KGC+NGW), YB
      WRITE (2, '(''Environment        '',
     1 ''Exp. Yield      Boost (%)'')')
      DO 182 NED=1,NE
      IF (L(KEW+NED) .NE. NGW)  GO TO 182
      R1 = V(KEA+NED)
      R2 = V(KEA+NE+NED)
      V0 = V0A + V(KEM+NED) + (V0M1*R1) + (V0M2*R2)
      V2 = V2A + V(KEM+NED) + (V2M1*R1) + (V2M2*R2)
      YB = 100.0 * ((V2 - V0) / V0)
      WRITE (2, '(I5, 2X, A4, F18.5, F11.2)')
     1 NED, C(KEC+NED), V2, YB
  182 CONTINUE
  180 CONTINUE
C     Notify.
      WRITE (*, '(/, '' AMMI2 analysis completed.'')')
      RETURN
      END
      SUBROUTINE AMMIH(NX)
      INTEGER*4    L(16000)
      REAL*4  YA(4900)
      CHARACTER*4  C(16000)
      COMMON  /DATA/  V(16000)
      COMMON  /PARM/  NG, NE, GM
      COMMON  /SETO/  KEM,KGC,KEC,KGL,KEL,KGW,KEW,KGT,KGA,KEA
      COMMON  /MATR/  M(4900)
      EQUIVALENCE  (V(1), L(1), C(1))
      EQUIVALENCE  (M(1), YA(1))
C     Notify.
      WRITE (*, '(/, '' AMMI'', I1, '' analysis begins.'')')  NX
C     Initially accept all genotypes.
      KGEL = 0
      DO 100 NGD=1,NG
  100 L(KGL+NGD) = 1
C     Find genotype with highest mean.
      GHM = 0.0
      DO 101 NGD=1,NG
      R = V(NGD)
      IF (R .LE. GHM)  GO TO 101
      GHM = R
      NGH = NGD
  101 CONTINUE
C     Determine and count winners.
      DO 111 NED=1,NE
      AD = V(KEM+NED) - GM
      Y0 = AD + GHM
      DO 112 ND=1,NX
      X = V(KEA+((ND-1)*NE)+NED)
  112 Y0 = Y0 + (V(KGA+((ND-1)*NG)+NGH) * X)
      YA(NED) = Y0
      YX = 0.0
      DO 113 NGD=1,NG
      IF (L(KGL+NGD) .NE. 1)  GO TO 113
      YH = AD + V(NGD)
      DO 114 ND=1,NX
      X = V(KEA+((ND-1)*NE)+NED)
  114 YH = YH + (V(KGA+((ND-1)*NG)+NGD) * X)
      IF (YH .LE. YX)  GO TO 113
      YX = YH
      NWN = NGD
  113 CONTINUE
      L(KEW+NED) = NWN
      V(KGW+NWN) = V(KGW+NWN) + 1.0
  111 V(KEL+NED) = YX
      CALL INDEX(NG, KGW, KGT)
      DO 115 NGD=1,NG
  115 IF (V(KGW+NGD) .EQ. 0.0)  L(KGL+NGD) = 0
C     Results for mega-environments, also to screen on first pass.
      WRITE (2, '(/, ''AMMI'', I1, '' Winners, Results for'',
     1 '' Mega-environments'')')  NX
      WRITE (*, '(/, '' AMMI'', I1, '' Winners, Results for'',
     1 '' Mega-environments'')')  NX
      WRITE (2, '(/, ''Genotype       Wins      Exp. Yield | '',
     1 A4, '' Exp. Yield    Boost (%)'')')  C(KGC+NGH)
      WRITE (*, '(/, '' Genotype       Wins      Exp. Yield | '',
     1 A4, '' Exp. Yield    Boost (%)'')')  C(KGC+NGH)
      NME = 0
      DO 120 NGD=1,NG
      NGW = L(KGT+NGD)
      NWINS = V(KGW+NGW)
      IF (NWINS .EQ. 0)  GO TO 20
      NME = NME + 1
      Y0T = 0.0
      YHT = 0.0
      DO 121 NED=1,NE
      IF (L(KEW+NED) .NE. NGW)  GO TO 121
      Y0T = Y0T + YA(NED)
      YHT = YHT + V(KEL+NED)
  121 CONTINUE
      R = NWINS
      Y0T = Y0T / R
      YHT = YHT / R
      YB = 100.0 * ((YHT - Y0T) / Y0T)
      WRITE (2, '(I5, 2X, A4, I8, F16.5, '' |'', F16.5, F10.2)')
     1 NGW, C(KGC+NGW), NWINS, YHT, Y0T, YB
  120 WRITE (*, '(I6, 2X, A4, I8, F16.5, '' |'', F16.5, F10.2)')
     1 NGW, C(KGC+NGW), NWINS, YHT, Y0T, YB
C     Eliminate minor winners.
   20 WRITE (*, '(/, '' Enter minor genotypes to eliminate,'',
     1 '' 0 to end list;'', /, ''OR enter negative -N to'',
     2 '' eliminate entries with N or fewer wins.'')')
   25 READ (*, *)  NGEL
      IF (NGEL .EQ. 0)  GO TO 26
      IF (NGEL .LT. 0)  THEN
      MIN = -NGEL
      DO 130 NGD=1,NG
      NWINS = V(KGW+NGD)
      IF (NWINS .EQ. 0  .OR.  NWINS .GT. MIN)  GO TO 130
      L(KGL+NGD) = -1
      V(KGW+NGD) = 0.0
      KGEL = KGEL + 1
  130 CONTINUE    
      GO TO 26
      ELSE
      IF (NGEL .LE. NG)  THEN
      L(KGL+NGEL) = -1
      V(KGW+NGEL) = 0.0
      KGEL = KGEL + 1
      ELSE
      WRITE (*, '('' Entry outside range; try again.'')')
      ENDIF
      GO TO 25
      ENDIF
   26 WRITE (2, '(/, ''AMMI'', I1, '' Winners, Genotypes'',
     1 '' Eliminated by User''''s Requests'')')  NX
      IF (KGEL .EQ. 0)  THEN
      WRITE (2, '(''No winners were eliminated.'')')
      ELSE
      NME = NME - KGEL
      WRITE (2, '(I3, '' winners were eliminated,'',
     1 '' as follows:'')')  KGEL
      DO 135 NGD=1,NG
  135 IF (L(KGL+NGD) .EQ. -1)  WRITE (2, '(I5, 2X, A4)')
     1 NGD, C(KGC+NGD)
      ENDIF
      WRITE (2, '('' '')')
C     Repeat analysis.
      IF (L(KGL+NGH) .EQ. -1)  THEN
      GHM = 0.0
      DO 140 NGD=1,NG
      IF (L(KGL+NGD) .NE. 1)  GO TO 140
      R = V(NGD)
      IF (R .LE. GHM)  GO TO 140
      GHM = R
      NGH = NGD
  140 CONTINUE
      DO 150 NED=1,NE
      Y0 = GHM + V(KEM+NED) - GM
      DO 151 ND=1,NX
      X = V(KEA+((ND-1)*NE)+NED)
  151 Y0 = Y0 + (V(KGA+((ND-1)*NG)+NGH) * X)
  150 YA(NED) = Y0
      ENDIF
      DO 155 NED=1,NE
      NGW = L(KEW+NED)
      IF (L(KGL+NGW) .NE. -1)  GO TO 155
      AD = V(KEM+NED) - GM
      YX = 0.0
      DO 156 NGD=1,NG
      IF (L(KGL+NGD) .NE. 1)  GO TO 156
      YH = AD + V(NGD)
      DO 157 ND=1,NX
      X = V(KEA+((ND-1)*NE)+NED)
  157 YH = YH + (V(KGA+((ND-1)*NG)+NGD) * X)
      IF (YH .LE. YX)  GO TO 156
      YX = YH
      NWN = NGD
  156 CONTINUE
      L(KEW+NED) = NWN
      V(KGW+NWN) = V(KGW+NWN) + 1.0
      V(KEL+NED) = YX
  155 CONTINUE
      CALL INDEX(NG, KGW, KGT)
      WRITE (2, '(/, ''AMMI'', I1, '' Winners, Results for'',
     1 '' Mega-environments After Eliminations'')')  NX
      WRITE (2, '(/, ''Genotype       Wins      Exp. Yield | '',
     1 A4, '' Exp. Yield    Boost (%)'')')  C(KGC+NGH)
      DO 160 ND=1,NME
      NGW = L(KGT+ND)
      NWINS = V(KGW+NGW)
      Y0T = 0.0
      YHT = 0.0
      DO 161 NED=1,NE
      IF (L(KEW+NED) .NE. NGW)  GO TO 161
      Y0T = Y0T + YA(NED)
      YHT = YHT + V(KEL+NED)
  161 CONTINUE
      R = NWINS
      Y0T = Y0T / R
      YHT = YHT / R
      YB = 100.0 * ((YHT - Y0T) / Y0T)
  160 WRITE (2, '(I5, 2X, A4, I8, F16.5, '' |'', F16.5, F10.2)')
     1 NGW, C(KGC+NGW), NWINS, YHT, Y0T, YB
C     Results for environments.
      IF (KGEL .EQ. 0)  THEN
      WRITE (2, '(/, ''AMMI2 Winners, Results for'',
     1 '' Environments'')')
      ELSE
      WRITE (2, '(/, ''AMMI2 Winners, Results for'',
     1 '' Environments After Eliminations'')')
      ENDIF
      WRITE (2, '(''Environments are grouped in'', I3,
     1 '' mega-environments.'')')  NME
      DO 170 NMD=1,NME
      NGW = L(KGT+NMD)
      NWINS = V(KGW+NGW)
      Y0 = 0.0
      YH = 0.0
      DO 171 NED=1,NE
      IF (L(KEW+NED) .NE. NGW)  GO TO 171
      Y0 = Y0 + YA(NED)
      YH = YH + V(KEL+NED)
  171 CONTINUE
      YB = 100.0 * ((YH - Y0) / Y0)
      WRITE (2, '(/, ''Mega-environment'', I3, '' with'', I5,
     1 '' wins by '', A4, '' gives boost of'', F7.2, ''%'')')
     2 NMD, NWINS, C(KGC+NGW), YB
      WRITE (2, '(''Environment        '',
     1 ''Exp. Yield      Boost (%)'')')
      DO 172 NED=1,NE
      IF (L(KEW+NED) .NE. NGW)  GO TO 172
      Y0 = YA(NED)
      YH = V(KEL+NED)
      YB = 100.0 * ((YH - Y0) / Y0)
      WRITE (2, '(I5, 2X, A4, F18.5, F11.2)')
     1 NED, C(KEC+NED), YH, YB
  172 CONTINUE
  170 CONTINUE
      RETURN
      END
      SUBROUTINE INDEX(N, KBV, KBL)
C     Index N elements of vector V with base address KBV, placing
C     the indices in decending order in vector L with base address
C     KBL, using the Heapsort sorting algorithm with indirect
C     indexing through L in all references to V.
C     Hence the first index, L(KBL+1), points to V(KBV+L(KBL+1))
C     containing the largest value in vector V.
C     This subroutine is adapted from William H. Press, Brian P.
C     Flannery, Saul A. Teukolsky, and William T. Vetterling,
C     1986, Numerical Recipes, Cambridge University Press,
C     Cambridge, England, pages 232-233.
      INTEGER*4  L(16000)
      COMMON  /DATA/  V(16000)
      EQUIVALENCE  (V(1), L(1))
      DO 100 J=1,N
      L(KBL+J) = J
  100 CONTINUE
      K = (N / 2) + 1
      IR = N
   10 CONTINUE
      IF (K .GT. 1)  THEN
      K = K - 1
      LT = L(KBL+K)
      X = V(KBV+LT)
      ELSE
      LT = L(KBL+IR)
      X = V(KBV+LT)
      L(KBL+IR) = L(KBL+1)
      IR = IR - 1
      IF (IR .EQ. 1)  THEN
      L(KBL+1) = LT
      GO TO 30
      ENDIF
      ENDIF
      I = K
      J = K + K
   20 IF (J .LE. IR)  THEN
      IF (J .LT. IR)  THEN
      KS = KBL + J
      IF (V(KBV+L(KS)) .LT. V(KBV+L(KS+1)))  J = J + 1
      ENDIF
      IF (X .LT. V(KBV+L(KBL+J)))  THEN
      L(KBL+I) = L(KBL+J)
      I = J
      J = J + J
      ELSE
      J = IR + 1
      ENDIF
      GO TO 20
      ENDIF
      L(KBL+I) = LT
      GO TO 10
   30 KS = KBL + N + 1
      DO 101 J=1,N/2
      I = KBL + J
      K = KS - J
      LT = L(I)
      L(I) = L(K)
  101 L(K) = LT
      RETURN
      END
