C     MATMODEL Version 3.0.  Hugh G. Gauch, Jr.  July 2007.
C     This FORTRAN77 program fits AMMI, ANOVA, PCA, and linear
C     regression models for two-way data matrices, with or
C     without replication.  The AMMI model allows missing data, and
C     some of the data may be reserved for validation calculations
C     of predictive success.
C
C     Modified December 1990.
C     Modified March 1991.
C     Modified June 1993.
C     Modified October 1995.
C     Upgraded from Version 2.1 to 2.2 in September 2002.
C     Upgraded from Version 2.2 to 3.0 in July 2007.
C     Compiled with 32-bit DIGITAL Visual FORTRAN Optimizing
C     Compiler, Version 5.0.
C
C     Memory requirement is about 32 megabytes.
C
C     (C) Copyright 1990, 1993, 1998, 2002, 2007 Hugh G. Gauch, Jr.
C     All rights reserved.
C     Crop and Soil Sciences, 519 Bradfield Hall, Cornell University,
C     Ithaca, New York  14851
C     (Telephone 607-255-7764; e-mail hgg1@cornell.edu)
C
C     An accompanying documentation is available, as well as
C     sample data files and output files.
C
C     This software is open source and may be redistributed or
C     modified under the GNU General Public License as published
C     by the Free Software Foundation.
C
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA,   Q
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      LOGICAL     EX
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /MESW/  KME
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /RAND/  RAN, IXS, IYS, IZS, IX, IY, IZ
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C
C     Memory map for data vector with equivalenced arrays V Real*8,
C     Y Real*4, L Integer*4, C Character*4, and M Integer*2.
C     G genotypes, E environments, R replicates, and N IPCA or PCA axes.
C
C     Data          Vector Subscript  Offset          Length
C
C     TRT means, IPCA    V   IAGE    0                   GE
C     TRT means, keep    V   IDGE    KDGE=GE             GE
C     GEN means          V   IBG     KBG=KDGE+GE         G
C     ENV means          V   IBE     KBE=KBG+G           E
C     GEN scratch        V   ISG     KSG=KBE+E           G
C     ENV scratch        V   ISE     KSE=KSG+G           E
C     GEN IPCA           V   IPGN    KPGN=KSE+E          GN
C     ENV IPCA           V   IPEN    KPEN=KPGN+GN        EN
C
C     REP data           Y   IYRGE   KYRGE=2*(KPEN+EN)   RGE
C     GEN counts         L   ICG     KCG=KYRGE+RGE       G
C     ENV counts         L   ICE     KCE=KCG+G           E
C     GEN scratch        L   IIG     KIG=KCE+E           G
C     ENV scratch        L   IIE     KIE=KIG+G           E
C     GEN names          C   ING     KNG=KIE+E           G
C     ENV names          C   INE     KNE=KNG+G           E
C
C     REP status         M   IKRGE   KKRGE=2*(KNE+E)     RGE
C     TRT status         M   IKGE    KKGE=KKRGE+RGE      GE
C
C     Memory total in *4 units is    KTL=(KKGE+GE+1)/2
C     which equals 1.5GE(4.5+R) + (7+N)(G+E), rounded up.
C
C     The storage order within vectors having two or three
C     implicit subscripts is as follows.
C     For the REP data Y(IYRGE) and REP status M(IKRGE) vectors,
C     the storage order is to increment REP fastest, GEN next,
C     and ENV slowest.  Hence REP 1, GEN 1, ENV 1 is entry 1;
C     REP 2, GEN 1, ENV 1 is entry 2; REP 1, GEN 2, ENV 1 is
C     entry R+1; and REP 1, GEN 1, ENV 2 is entry RG+1.
C     For the two TRT vectors V(IAGE) and V(IDGE), and for the TRT
C     status vector M(IKGE), the storage order is to increment
C     GEN fastest and ENV slowest.
C     For the GEN IPCA and ENV IPCA vectors V(IPGN) and V(IPEN),
C     the storage order is to increment GEN (or ENV) fastest and
C     the index number of the IPCA axis slowest.
C
C     Set program constants.
C          Set vector length n in *4 units.  Accordingly the vector
C          dimensions should be V(n/2), Y(n), L(n), C(n), and M(2n).
      LNGTH4 = 8000000
C          Set maximum number of validation runs (except that this
C          limit is not imposed when analyzing indirect information).
      MXRUN = 50000
C          Set default name for treatments.
      TNM = 'TRT '
C          Set defaults for controlling IPCA iterations.
C          If MXIPCA is increased above its original default value of 7,
C          increase the dimension of vector SSP in common /SSQR/ from 7
C          to the new value of MXIPCA.  Also in common /PRED/ increase
C          the original dimensions of RPD(8,3) to RPD(MXIPCA+1,3) and
C          of KRPD(8) to KRPD(MXIPCA+1).
      MXIPCA = 7
      MNIPD = 0
      MXIPD = 5
      MFIPD = 1
      MIIPD = 1
      MXCYCL = 50
      RSHFT = 1.0D-6
      MXITER = 100
      DEL = 1.0D-9
C          Initialize random number generator.
      IXS = 15718
      IYS = 28357
      IZS = 12275
C     Select program mode:  MODE=1 Validating; 2 Fitting.
C     Validating mode requires 1 or more validation runs, fits a
C     family of AMMI models, and produces minimal output.
C     The special value MRUN=-1, encountered in validating mode,
C     sets KII=1 for indirect information.
C     Fitting mode requires 0 or 1 validation run, fits a
C     single AMMI model, and produces extensive output.
      WRITE (*, '('' MATMODEL Version 3.0:  AMMI and Related'',
     1 '' Analyses.'', /, '' This is open source software as'',
     2 '' explained in its documentation.'', /, '' Hugh Gauch,'',
     3 '' Crop and Soil Sciences, Cornell University.'', /, /)')
   10 WRITE (*, '('' Enter program mode:  Validating model or'',
     1 '' Fitting model.  <V, F>'')')
      READ (*, '(A1)')  Q
      MODE = 1
      IF (Q .EQ. 'V'  .OR.  Q .EQ. 'v')  GO TO 11
      IF (.NOT. (Q .EQ. 'F'  .OR.  Q .EQ. 'f'))  GO TO 10
      MODE = 2
C     Enter file names and open files.
      KVAL = 0
   11 WRITE (*, '(/, '' Enter data file name.'')')
      READ (*, '(A60)')  FNDAT
      INQUIRE (FILE=FNDAT, EXIST=EX)
      IF (EX)  GO TO 12
      WRITE (*, '('' WARNING:  Cannot find file; try again.'')')
      GO TO 11
   12 MRUN = 1
      IF (MODE .EQ. 1)  GO TO 14
   13 WRITE (*, '(/, '' Do you want to read a validation file with'',
     1 '' 1 run?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  GO TO 20
      IF (.NOT. (Q .EQ. 'Y'  .OR.  Q .EQ. 'y'))  GO TO 13
   14 WRITE (*, '(/, '' Enter validation file name.'')')
      READ (*, '(A60)')  FNVAL
      INQUIRE (FILE=FNVAL, EXIST=EX)
      KVAL = 1
      IF (EX)  GO TO 15
      WRITE (*, '('' WARNING:  Cannot find file; try again.'')')
      GO TO 14
   15 OPEN (2, FILE=FNVAL, STATUS='OLD')
      READ (2, '(I5, 1X, A72)', ERR=16)  MRUN, NMVA
      GO TO 17
   16 WRITE (*, '('' WARNING:  Number of runs is unreadable;'',
     1 '' try again.'')')
      GO TO 14
   17 IF (MRUN .NE. 0)  GO TO 18
      WRITE (*, '('' WARNING:  Number of runs is 0; try again.'')')
      GO TO 14
   18 IF (MODE .EQ. 2)  GO TO 19
      IF (MRUN .LE. MXRUN)  GO TO 20
      WRITE (*, '('' WARNING:  Number of runs exceeds limit of'', I5,
     1 ''; try again.'')')  MXRUN
      GO TO 14
   19 IF (MRUN .EQ. 1)  GO TO 20
      WRITE (*, '('' WARNING:  Number of runs is not 1; try again.'')')
      GO TO 14
   20 WRITE (*, '(/, '' Enter output file name.'')')
      READ (*, '(A60)')  FNOUT
      INQUIRE (FILE=FNOUT, EXIST=EX)
      IF (.NOT. EX)  GO TO 75
      WRITE (*, '('' WARNING:  File already exists; try again.'')')
      GO TO 20
   75 KQ = 0
      IF (MODE .EQ. 1)  GO TO 21
      WRITE (*, '(/, '' Do you want to write a machine-readable'',
     1 '' file?  <Y, N, *>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      GO TO 21
      ELSEIF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      KQ = 1
      GO TO 76
      ELSEIF (Q .EQ. '*')  THEN
      KQ = 2
      GO TO 76
      ELSE
      GO TO 75
      ENDIF
   76 WRITE (*, '(/, '' Enter machine-readable file name.'')')
      READ (*, '(A60)')  FNMCR
      INQUIRE (FILE=FNMCR, EXIST=EX)
      IF (.NOT. EX)  GO TO 77
      WRITE (*, '('' WARNING:  File already exists; try again.'')')
      GO TO 76
   77 IF (KQ .EQ. 2)  THEN
      WRITE (*, '(/, '' Enter format for expected values,'',
     1 '' such as (10F8.3).'')')
      READ (*, '(A60)')  FZ
      ENDIF
   21 WRITE (*, '(/, '' Are all of these file names correct?'',
     1 ''  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  GO TO 22
      IF (.NOT. (Q .EQ. 'N'  .OR.  Q .EQ. 'n'))  GO TO 21
      GO TO 11
   22 OPEN (1, FILE=FNDAT, STATUS='OLD')
      OPEN (3, FILE=FNOUT, STATUS='NEW')
C     Enter run title.
      KRTL = 0
   23 WRITE (*, '(/, '' Do you want to enter a run title?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  GO TO 24
      IF (.NOT. (Q .EQ. 'Y'  .OR.  Q .EQ. 'y'))  GO TO 23
      KRTL = 1
      WRITE (*, '(/, '' Enter run title.'', /, ''          1'',
     1 ''         2         3         4         5         6'',
     2 ''         7  |'', /, '' 123456789012345678901234567890'',
     3 ''123456789012345678901234567890123456789012|'')')
      READ (*, '(A72)')  NMRN
C     Offer default or customized analysis.
   24 KII = 0
      IF (MRUN .EQ. -1)  KII = 1
      MIIP = MIIPD
      MNIP = MNIPD
      MXIP = MXIPD
      MFIP = MFIPD
      KPII = 1
      KPAM = 1
      KPC = -1
      KLR = -1
      KPPC = 1
      KPLR = 1
      KLV = 1
      KME = 1
      WRITE (*, '(/, '' Do you want all defaults?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  GO TO 44
      IF (.NOT. (Q .EQ. 'N'  .OR.  Q .EQ. 'n'))  GO TO 24
C     Enter number of IPCA axes.
      IF (KII .EQ. 0)  GO TO 27
   25 WRITE (*, '(/, '' Analysis of indirect information has been'',
     1 '' selected.'', /, '' Do you want the default of'', I3,
     2 '' IPCA axis?  <Y, N>'')')  MIIPD
      READ (*, '(A1)')  Q
      IF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  GO TO 33
      IF (.NOT. (Q .EQ. 'N'  .OR.  Q .EQ. 'n'))  GO TO 25
   26 WRITE (*, '('' Enter number of IPCA axes within 0 to'', I2,
     1 ''.'')')  MXIPCA
      READ (*, *)  MIIP
      IF (MIIP .GE. 0  .AND.  MIIP .LE. MXIPCA)  GO TO 33
      WRITE (*, '('' WARNING:  Illegal value; try again.'')')
      GO TO 26
   27 IF (MODE .EQ. 2)  GO TO 31
   28 WRITE (*, '(/, '' The defaults for validation mode are'', I3,
     1 '' to'', I3, '' IPCA axes.'', /, '' Do you want these'',
     2 '' defaults?  <Y, N>'')')  MNIPD, MXIPD
      READ (*, '(A1)')  Q
      IF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  GO TO 44
      IF (.NOT. (Q .EQ. 'N'  .OR.  Q .EQ. 'n'))  GO TO 28
   29 WRITE (*, '('' Enter minimum number of IPCA axes within 0 to'',
     1 I2, ''.'')')  MXIPCA
      READ (*, *)  MNIP
      IF (MNIP .GE. 0  .AND.  MNIP .LE. MXIPCA)  GO TO 30
      WRITE (*, '('' WARNING:  Illegal value; try again.'')')
      GO TO 29
   30 WRITE (*, '('' Enter maximum number of IPCA axes within'', I2,
     1 '' to'', I2, ''.'')')  MNIP, MXIPCA
      READ (*, *)  MXIP
      IF (MXIP .GE. MNIP  .AND.  MXIP .LE. MXIPCA)  GO TO 44
      WRITE (*, '('' WARNING:  Illegal value; try again.'')')
      GO TO 30
   31 WRITE (*, '(/, '' The default for fitting mode is'', I3,
     1 '' IPCA axis.'', /, '' Do you want this default?  <Y, N>'')')
     2 MFIPD
      READ (*, '(A1)')  Q
      IF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  GO TO 33
      IF (.NOT. (Q .EQ. 'N'  .OR.  Q .EQ. 'n'))  GO TO 31
   32 WRITE (*, '('' Enter number of IPCA axes within 0 to'', I2,
     1 ''.'')')  MXIPCA
      READ (*, *)  MFIP
      IF (MFIP .GE. 0  .AND.  MFIP .LE. MXIPCA)  GO TO 33
      WRITE (*, '('' WARNING:  Illegal value; try again.'')')
      GO TO 32
C     Specify whether to produce extensive table for indirect
C     information and for AMMI.
   33 IF (MODE .EQ. 2)  GO TO 35
      IF (KII .EQ. 0)  GO TO 44
   34 WRITE (*, '(/, '' Do you want the extensive table for indirect'',
     1 '' information?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      KPII = -1
      GO TO 44
      ELSEIF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      GO TO 44
      ELSE
      GO TO 34
      ENDIF
   35 WRITE (*, '(/, '' Do you want the extensive table for the AMMI'',
     1 '' model?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      KPAM = -1
      GO TO 96
      ELSEIF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      GO TO 96
      ELSE
      GO TO 35
      ENDIF
C     Specify whether to perform mega-environment analysis.
   96 IF (MFIP .LT. 1)  GO TO 36
      WRITE (*, '(/, '' Do you want mega-environment analysis'',
     1 '' for AMMI1?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      KME = -1
      GO TO 36
      ELSEIF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      GO TO 36
      ELSE
      GO TO 96
      ENDIF
C     Specify whether to produce PCA and linear regression models.
   36 WRITE (*, '(/, '' Do you want the PCA model?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      KPC = 1
      GO TO 38
      ELSEIF (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      GO TO 39
      ELSE
      GO TO 36
      ENDIF
   38 WRITE (*, '(/, '' Do you want the extensive table for the PCA'',
     1 '' model?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      KPPC = -1
      GO TO 39
      ELSEIF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      GO TO 39
      ELSE
      GO TO 38
      ENDIF
   39 WRITE (*, '(/, '' Do you want the linear regression model?'',
     1 ''  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      KLR = 1
      GO TO 40
      ELSEIF (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      GO TO 44
      ELSE
      GO TO 39
      ENDIF
   40 WRITE (*, '(/, '' Do you want the extensive table for the'',
     1 '' linear regression?  <Y, N>'')')
      READ (*, '(A1)')  Q
      IF     (Q .EQ. 'N'  .OR.  Q .EQ. 'n')  THEN
      KPLR = -1
      GO TO 44
      ELSEIF (Q .EQ. 'Y'  .OR.  Q .EQ. 'y')  THEN
      GO TO 44
      ELSE
      GO TO 40
      ENDIF
C     Notify that terminal input is completed.
   44 WRITE (*, '(/, '' Terminal input is now completed.'')')
      WRITE (*, '(/, '' MATMODEL analysis begins.'')')
C     Read data matrix.
      CALL READ
C     AMMI analysis for KII=1 indirect information.
      IF (KII .EQ. 0)  GO TO 50
      IRUN = 0
      NFE0 = 0
      CALL ANOVA(0)
      IRUN = 1
      CALL VALID
      IF (KNUL .EQ. 1)  CALL ANOVA(1)
      CALL INDIR
      GO TO 70
C     AMMI analysis for MODE=1 model validating.
   50 IF (MODE .EQ. 2)  GO TO 51
      DO 100 IRUN=1,MRUN
      CALL VALID
      CALL ANOVA(IRUN)
      CALL AMMI(MNIP, MXIP)
      WRITE (*, '('' Validation'', I7, '' of'', I7, ''.'')')
     1 IRUN, MRUN
  100 CONTINUE
      WRITE (*, '(/, '' Completed prediction statistics.'')')
      GO TO 70
C     AMMI analysis for MODE=2 model fitting.
   51 IRUN = 1
      CALL ANOVA(0)
      IF (KVAL .EQ. 0)  GO TO 52
      CALL VALID
      CALL ANOVA(1)
   52 CALL AMMI(MFIP, MFIP)
C     Conclude program run.
   70 CLOSE (1)
      IF (KVAL .EQ. 1)  CLOSE (2)
      ENDFILE 3
      CLOSE (3)
      STOP
      END
      SUBROUTINE READ
C     Read data file and check memory requirements.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA,   SEQA, SEQB, SEQC, FT
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM,   TNX
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN,   FMT
      CHARACTER*80     LNE
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C
C     The first line of the data file (FNDAT) contains two items.
C     (1)  Columns  1-72, title for the data matrix.
C     (2)  Columns 77-80, brief name for the data matrix.
C     The second line contains thirteen items.
C     (1)  Columns  1-20, name for the data's measurement and units.
C     (2)  Columns 21-25, scaling integer N, multiplies data by 10**N.
C     (3)  Columns 26-30, number of matrix rows (such as genotypes).
C     (4)  Columns 32-35, name for matrix rows (such as GEN).
C     (5)  Columns 36-40, number of matrix columns (environments).
C     (6)  Columns 42-45, name for matrix columns (ENV).
C     (7)  Columns 46-50, number of replications.
C     (8)  Columns 52-55, name for replications (REP).
C     (9)  Columns 58-60, order of data matrix entries, specified by
C          the first letters of the subscript names (from columns
C          32, 42, and 52), the first changing fastest and the last
C          changing slowest. For example, for order 'RGE' the second
C          matrix entry is for REP 2, GEN 1, and ENV 1.
C     (10) Columns 62-65, name for treatments (default is TRT).
C     (11) Columns 68-70, experimental design ('RAN' or other).
C     (12) Columns 71-75, missing value indicator.
C     (13) Column 80, if 'N', no format supplied;
C          if 'F', format supplied for reading matrix entries;
C          if 'S', format supplied for reading single-entry lines.
C     The optional format line, given 'F' or 'S' in column 80 of
C     line 2, contains the FORTRAN variable format (format A72).
C     The data matrix entries then follow, in free format if no format
C     is supplied, in fixed format if 'F', and in single-entry lines
C     if 'S'.  Single-entry lines contain the ENV, GEN, and REP
C     subscripts (in the order indicated in columns 58-60 of line 2)
C     followed by the data value.  The end of single-entry lines is
C     indicated by a line containing all zeros.
C     Row (GEN) code names and then column (ENV) names follow
C     (format 15(A4, 1X)), each list starting a new line.
C     For all of the above data, enter full lines completing all
C     items called for, even if this requires blanks or zeros
C     at the end of a line.
C     Finally, any number of optional comment lines may follow,
C     of which up to 50 lines will be written in fitting mode.
C
C     Initialize error flag.
      KERR = 0
C     Read title card, matrix dimensions, and format if supplied.
      READ (1, '(A72, 4X, A4)')  NMDA, NMDB
      READ (1, '(A20, I5, 3(I5,1X, A1, A3), 2X, 3A1, 1X, A4, 2X, A3,
     1 F5.0, 4X, A1)')  NMUN, KSCAL, NG, GNA, GNB, NE, ENA, ENB, NR,
     2 RNA, RNB, SEQA, SEQB, SEQC, TNX, EXDS, YMV, FT
      IF (KRTL .EQ. 0)  NMRN = NMDA
      IF (TNX .NE. '    ')  TNM = TNX
      KFM = -1
      IF (FT .EQ. 'N'  .OR.  FT .EQ. 'n')  KFM = 0
      IF (FT .EQ. 'F'  .OR.  FT .EQ. 'f')  KFM = 1
      IF (FT .EQ. 'S'  .OR.  FT .EQ. 's')  KFM = 2
      IF (KFM .EQ. -1)  THEN
      KERR = 1
      WRITE (*, '('' ERROR:  Format specification '', A1,
     1 '' fails to be N or F or S.'')')  FT
      ELSE
      IF (KFM .GE. 1)  READ (1, '(A72)')  FMT
      ENDIF
C     Set several constants.
      NX = NR
      NGE = NG * NE
      NRG = NR * NG
      NRGE = NR * NGE
      FG = NG
      FE = NE
      FR = NR
      FGE = NGE
      FRGE = NRGE
C     Reduce numbers of IPCA and PCA axes if matrix is too small.
      NS = MIN0(NG, NE)
      NXP = 4
      IF (NS .LT. NXP)  NXP = NS
      NS = NS - 1
      IF     (MODE. EQ. 1  .AND.  KII .EQ. 0)  THEN
      IF (MNIP .GT. NS)  MNIP = NS
      IF (MXIP .GT. NS)  MXIP = NS
      NP = MXIP
      ELSEIF (MODE .EQ. 1  .AND.  KII .EQ. 1)  THEN
      IF (MIIP .GT. NS)  MIIP = NS
      NP = MIIP
      ELSEIF (MODE .EQ. 2)  THEN
      IF (MFIP .GT. NS)  MFIP = NS
      NP = MFIP
      IF (KPC .EQ. 1)  NP = MAX0(NP, NXP)
      ENDIF
C     Check availability of adequate array storage.
      KDGE = NGE
      KBG = KDGE + NGE
      KBE = KBG + NG
      KSG = KBE + NE
      KSE = KSG + NG
      KPGN = KSE + NE
      KPEN = KPGN + (NG * NP)
      KYRGE = 2 * (KPEN + (NE * NP))
      KCG = KYRGE + NRGE
      KCE = KCG + NG
      KIG = KCE + NE
      KIE = KIG + NG
      KNG = KIE + NE
      KNE = KNG + NG
      KKRGE = 2 * (KNE + NE)
      KKGE = KKRGE + NRGE
      KTL = (KKGE + NGE + 1) / 2
      WRITE (*, '(/, '' '', A72)')  NMDA
      WRITE (*, '('' '', I5, 1X, A1, A3, ''   and'', I6, 1X, A1, A3,
     1 ''   with'', I6, 1X, A1, A3)')  NG, GNA, GNB, NE, ENA, ENB,
     2 NR, RNA, RNB
      RM = 100.0D0 * (DFLOAT(KTL) / DFLOAT(LNGTH4))
      WRITE (*, '(/, '' Required memory is'', F10.3,
     1 ''% of program specification.'')')  RM
      IF (KTL .LE. LNGTH4)  THEN
      WRITE (*, '('' Program memory specification is adequate.'')')
      ELSE
      KERR = 1
      WRITE (*, '('' Program memory specification is not adequate.'')')
      ENDIF
C     Check variable names for uniqueness of first letters.
      IF (GNA .NE. ENA  .AND.  GNA .NE. RNA  .AND.  ENA .NE. RNA)
     1 GO TO 10
      KERR = 1
      WRITE (*, '('' ERROR:  Variable name first letters not'',
     1 '' unique.'')')
C     If using a validation file KVAL=1, then check that NR>1.
   10 IF (KVAL .EQ. 0  .OR.  NR .GT. 1)  GO TO 20
C     Oct 1995 change to allow indir info with nr=1.
      IF (KII .EQ. 1)  GO TO 20
      KERR = 1
      WRITE (*, '('' ERROR:  No replication despite request for'',
     1 '' validation.'')')
C     Check validity of input order, read data matrix into vector
C     Y(IYRGE), and mark REP status in vector M(IKRGE).
C     SEQA changes fastest and SEQC slowest in the input data matrix
C     order, whereas the standardized storage order in Y(IYRGE) is to
C     increment REP fastest, GEN next, and ENV slowest, which is
C     the target order of RGE.
   20 IF     (SEQA.EQ.RNA .AND. SEQB.EQ.GNA .AND. SEQC.EQ.ENA)  THEN
      MA = 1
      MB = NR
      MC = NRG
      NA = NR
      NB = NG
      NC = NE
      GO TO 21
      ELSEIF (SEQA.EQ.RNA .AND. SEQB.EQ.ENA .AND. SEQC.EQ.GNA)  THEN
      MA = 1
      MB = NRG
      MC = NR
      NA = NR
      NB = NE
      NC = NG
      GO TO 21
      ELSEIF (SEQA.EQ.GNA .AND. SEQB.EQ.ENA .AND. SEQC.EQ.RNA)  THEN
      MA = NR
      MB = NRG
      MC = 1
      NA = NG
      NB = NE
      NC = NR
      GO TO 21
      ELSEIF (SEQA.EQ.GNA .AND. SEQB.EQ.RNA .AND. SEQC.EQ.ENA)  THEN
      MA = NR
      MB = 1
      MC = NRG
      NA = NG
      NB = NR
      NC = NE
      GO TO 21
      ELSEIF (SEQA.EQ.ENA .AND. SEQB.EQ.GNA .AND. SEQC.EQ.RNA)  THEN
      MA = NRG
      MB = NR
      MC = 1
      NA = NE
      NB = NG
      NC = NR
      GO TO 21
      ELSEIF (SEQA.EQ.ENA .AND. SEQB.EQ.RNA .AND. SEQC.EQ.GNA)  THEN
      MA = NRG
      MB = 1
      MC = NR
      NA = NE
      NB = NR
      NC = NG
      GO TO 21
      ENDIF
      KERR = 1
      WRITE (*, '('' ERROR:  Input order not valid.'')')
      GO TO 25
   21 IF (KERR .EQ. 1)  GO TO 25
      KYB = KYRGE - NR - NRG
      IF (KFM .NE. 0)  GO TO 22
      READ (1, *)    (((Y(KYB+MC*NCD+MB*NBD+MA*NAD), NAD=1,NA),
     1 NBD=1,NB), NCD=1,NC)
      GO TO 26
   22 IF (KFM .EQ. 2)  GO TO 23
      READ (1, FMT)  (((Y(KYB+MC*NCD+MB*NBD+MA*NAD), NAD=1,NA),
     1 NBD=1,NB), NCD=1,NC)
      GO TO 26
   23 KCRG = NRG + NR
      DO 100 IKRGE=KKRGE,KKRGE+NRGE
  100 M(IKRGE) = 0
   24 READ (1, FMT)  NA, NB, NC, YD
      IF (NA .EQ. 0)  GO TO 26
      NS = (MA * NA) + (MB * NB) + (MC * NC) - KCRG
      Y(KYRGE+NS) = YD
      IF (YD .NE. YMV)  M(KKRGE+NS) = 1
      GO TO 24
C     Terminate run if an error was encountered.
   25 IF (KERR .EQ. 0)  GO TO 26
      WRITE (*, '(/, '' ERROR:  Termination because of data'',
     1 '' problem.'')')
      STOP
C     Mark REP status.
   26 IF (KFM .EQ. 2)  GO TO 27
      DO 101 ND=1,NRGE
      MK = 1
      IF (Y(KYRGE+ND) .EQ. YMV)  MK = 0
  101 M(KKRGE+ND) = MK
C     Apply scaling factor if needed.
   27 IF (KSCAL .EQ. 0)  GO TO 30
      FS = KSCAL
      FS = 10.0D0 ** FS
      DO 102 ND=1,NRGE
      IF (M(KKRGE+ND) .EQ. 0)  GO TO 102
      NS = KYRGE + ND
      Y(NS) = Y(NS) * FS
  102 CONTINUE
C     Read variable names.
   30 READ (1, '(15(A4,1X))')  (C(KNG+ND), ND=1,NG)
      READ (1, '(15(A4,1X))')  (C(KNE+ND), ND=1,NE)
C     Write matrix name and notes if supplied.
      WRITE (3, '(''MATMODEL Version 3.0:  Data Input.'')')
      WRITE (3, '(/, A72)')  NMRN
      IF (MODE .EQ. 1  .AND.  KII .EQ. 0)  WRITE (3, '(/,
     1 ''Program mode:  Validating AMMI model.'')')
      IF (MODE .EQ. 1  .AND.  KII .EQ. 1)  WRITE (3, '(/,
     1 ''Program mode:  Validating for indirect information.'')')
      IF (MODE .EQ. 2)  WRITE (3, '(/, ''Program mode:'',
     2 ''  Fitting AMMI model.'')')
      WRITE (3, '(/, ''Data file:        '', A60)')  FNDAT
      IF (KVAL .EQ. 1)  WRITE (3, '(''Validation file:  '', A60)')
     1 FNVAL
      WRITE (3, '(''Output file:      '', A60)')  FNOUT
      WRITE (3, '(/, ''Read data matrix:'', /, A72, '' ('',
     1 A4, '')'', /)')  NMDA, NMDB
      DO 103 LD=1,50
      READ (1, '(A80)', END=31)  LNE
  103 WRITE (3, '(A80)')  LNE
   31 WRITE (3, '(/, ''Required memory is'', F10.3,
     1 ''% of program specification.'')')  RM
      WRITE (3, '(''Program memory specification is adequate.'')')
C     Issue terminal progress message.
      WRITE (*, '(/, '' Completed reading data file.'')')
      RETURN
      END
      SUBROUTINE VALID
C     Mark validation observations in the REP status vector M(IKRGE)
C     and the TRT status vector M(IKGE).  Set the number of validation
C     observations NVAL and the maximum number of model replications NX.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA,   A
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /RAND/  RAN, IXS, IYS, IZS, IX, IY, IZ
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C
C     The first line of the validation file (FNVAL), read in the
C     main routine, contains two items (format I5, 1X, A72).
C       (1)  MRUN is the number of validation runs.
C            In fitting mode, MRUN must equal 1.
C            In validating mode, 1 <= MRUN <= MXRUN (default=50000),
C            except indirect information analysis has MRUN = -1.
C       (2)  NMVA is the title for the validation file,
C            actually used only in fitting mode.
C     Each validation run, processed by subroutine VALID, defines a
C     data splitting by one of five methods.
C     Avoid removing all of the data from any ENV or GEN.
C     N:  No marking of validation observations.
C         This single line contains 'N     0' (format A1, I6).
C         This option is allowed only for analysis of indirect
C         information (with MRUN = -1 on the preceding line).
C     R:  Model N reps selected randomly for each TRT, as possible.
C         The first line contains 'R' and N (format A1, I6).
C         The second line contains an integer IRAN (format I5)
C         from 1 to 30000 to initialize the random number generator.
C     P:  Like option R, except N and random number are initialized
C         only once, regardless how large MRUN may be.
C     V:  Mark reps as listed for validation; no randomization.
C         The first line contains 'V' and N (format A1, I6).
C         The following N lines contain the list of reps, each line
C         giving the number of the REP (format I5).
C     T:  Mark TRT as listed for validation.
C         The first line contains 'T' and N (format A1, I6).
C         The following N lines contain the list of TRT, each line
C         giving the number of the ENV and GEN (format 2I5).
C     L:  Mark observations as listed for validation.
C         The first line contains 'L' and N (format A1, I6).
C         The following N lines contain the list of observations,
C         each giving the ENV, GEN, and REP (format 3I5).
C
C     The REP status vector M(IKRGE) entries contain -1 for a
C     validation datum, 0 for no datum, or 1 for a datum currently
C     used for modelling.
C     The TRT status vector M(IKGE) entries, to be set by subroutine
C     ANOVA, contain 0 for no data or N for N observations currently
C     used for modelling.
C     KBAL is set, by subroutine ANOVA, to -1 for missing cells,
C     0 for unbalanced data, or 1 for balanced data.
C     NVAL is the number of validation observations.
C     KVAL is set, by the main routine, to 0 if no validation file
C     is processed, or 1 alternatively.
C
C     If not the first call, first reset the REP status vector.
      IF (IRUN .EQ. 1)  GO TO 10
      DO 100 IKRGE=KKRGE+1,KKRGE+NRGE
  100 IF (M(IKRGE) .EQ. -1)  M(IKRGE) = 1
   10 CONTINUE
C     Read the next validation instruction.
      IF ((A .EQ. 'P'  .OR.  A .EQ. 'p')  .AND.
     1 IRUN .GT. 1)  GO TO 11
      READ (2, '(A1, I6)')  A, N
C     Case N:  No marking of validation observations.
   11 KNUL = 1
      IF     (A .EQ. 'N'  .OR.  A .EQ. 'n')  THEN
      KNUL = 0
      IF (MODE .EQ. 1  .AND.  KII .EQ. 1)  RETURN
      WRITE (*, '('' ERROR:  No validation observations attempted'',
     1 '' while in fitting mode,'', /, '' or without selecting'',
     2 '' analysis of indirect information.'')')
      STOP
C     Case R or P:  Model NRT reps selected randomly for each TRT,
C     as possible.
      ELSEIF (A .EQ. 'R'  .OR.  A .EQ. 'r'
     1 .OR.  A .EQ. 'P'  .OR.  A .EQ. 'p')  THEN
      IF ((A .EQ. 'P'  .OR.  A .EQ. 'p')  .AND. IRUN .GT. 1)
     1 GO TO 12
      NRT = N
      READ (2, '(I5)')  IRAN
      IX = IXS
      IY = IYS
      IZ = IZS
      IF (IRAN .GE. 1  .AND.  IRAN .LE. 30000)  IZ = IRAN
   12 CONTINUE
      DO 110 IGE=1,NGE
      IA = KKRGE + ((IGE - 1) * NR)
      ND = 0
      DO 111 IR=1,NR
      IF (M(IA+IR) .NE. 1)  GO TO 111
      ND = ND + 1
      CALL RANDOM
      V(KSG+IR) = RAN
  111 CONTINUE
      NDEL = ND - NRT
      IF (NDEL .LE. 0)  GO TO 110
      DO 112 NL=1,NDEL
      VM = -1.0D0
      DO 113 IR=1,NR
      IF (M(IA+IR) .NE. 1)  GO TO 113
      VT = V(KSG+IR)
      IF (VT .LE. VM)  GO TO 113
      VM = VT
      NM = IR
  113 CONTINUE
      V(KSG+NM) = -1.0D0
  112 M(IA+NM) = -1
  110 CONTINUE
      GO TO 30
C     Case V:  Mark reps as listed for validation; no randomization.
      ELSEIF (A .EQ. 'V'  .OR.  A .EQ. 'v')  THEN
      DO 120 ND=1,N
      READ (2, '(I5)')  IR
      IF (IR .LT. 1  .OR.  IR .GT. NR)  GO TO 20
      DO 121 IGE=1,NGE
      IA = KKRGE + ((IGE - 1) * NR) + IR
  121 IF (M(IA) .EQ. 1)  M(IA) = -1
  120 CONTINUE
      GO TO 30
C     Case T:  Mark TRT as listed for validation.
      ELSEIF (A .EQ. 'T'  .OR.  A .EQ. 't')  THEN
      KCRG = NRG + NR
      DO 130 ND=1,N
      READ (2, '(2I5)')  IE, IG
      IF (IE .LT. 1  .OR.  IE .GT. NE)  GO TO 20
      IF (IG .LT. 1  .OR.  IG .GT. NG)  GO TO 20
      IA = KKRGE + (IE * NRG) + (IG * NR) - KCRG
      DO 131 IR=1,NR
      IB = IA + IR
  131 IF (M(IB) .EQ. 1)  M(IB) = -1
  130 CONTINUE
      GO TO 30
C     Case L:  Mark observations as listed for validation.
      ELSEIF (A .EQ. 'L'  .OR.  A .EQ. 'l')  THEN
      KCRG = NRG + NR
      DO 140 ND=1,N
      READ (2, '(3I5)')  IE, IG, IR
      IF (IE .LT. 1  .OR.  IE .GT. NE)  GO TO 20
      IF (IG .LT. 1  .OR.  IG .GT. NG)  GO TO 20
      IF (IR .LT. 1  .OR.  IR .GT. NR)  GO TO 20
      IA = KKRGE + (IE * NRG) + (IG * NR) + IR - KCRG
  140 IF (M(IA) .EQ. 1)  M(IA) = -1
      GO TO 30
C     Error conditions.
      ENDIF
      WRITE (*, '('' ERROR:  Validation run has illegal key '',
     1 A1, ''.'')')  A
      STOP
   20 WRITE (*, '('' ERROR:  Validation file faulty.'')')
      READ (2, '(A72)', END=21)  NMRN
      WRITE (*, '('' The next line in the validation file begins'',
     1 '' with:'', /, A72)')  NMRN
   21 STOP
   30 CONTINUE
C     Talley observations in presently unused data vector elements.
      KD = 2 * KBG
      KM = 2 * KSG
      KV = KIG
      DO 200 ND=1,NR+1
      L(KD+ND) = 0
      L(KM+ND) = 0
  200 L(KV+ND) = 0
      NVAL = 0
      NMDL = 0
      DO 201 IGE=1,NGE
      NMC = 0
      NVC = 0
      IA = KKRGE + ((IGE - 1) * NR)
      DO 202 ND=1,NR
      NT = M(IA+ND)
      IF (NT .EQ. 1)  NMC = NMC + 1
  202 IF (NT .EQ. -1)  NVC = NVC + 1
      NDC = NMC + NVC
      NVAL = NVAL + NVC
      NMDL = NMDL + NMC
      NA = KD + NDC + 1
      L(NA) = L(NA) + 1
      NA = KM + NMC + 1
      L(NA) = L(NA) + 1
      NA = KV + NVC + 1
  201 L(NA) = L(NA) + 1
      NX = 0
      DO 203 ND=1,NR
      NA = KM + ND + 1
  203 IF (L(NA) .GT. 0)  NX = ND
C     Write results unless in validating mode.
      IF (MODE .EQ. 1)  RETURN
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:'',
     1 ''  Data Splitting.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, ''Validation file:  '', A60, /, A72)')  FNVAL, NMVA
      WRITE (3, '(/, ''The observation counts are'', I7, '' total,'',
     1 I7, '' model, and'', I7, '' validation.'')')  NMDL+NVAL, NMDL,
     2 NVAL
      IF (NX .LT. NR)  WRITE (3, '(''The number of '', A1, A3,
     1 '' was originally'', I6, '', but has been set to'', I6, ''.'')')
     2 RNA, RNB, NR, NX
      WRITE (3, '(''The following table lists the number of '', A1,
     1 A3, '' available for all'', I6, 1X, A4, /, ''in consequence of'',
     2 '' splitting the data into model and validation subsets.'')')
     3 RNA, RNB, NGE, TNM
      WRITE (3, '(/, 37(''-''), /, 1X, A1, A3, ''     Data     Model'',
     1 ''   Validation'', /, 37(''-''))')  RNA, RNB
      DO 300 ND=1,NR+1
  300 WRITE (3, '(I4, 3I10)')  ND-1, L(KD+ND), L(KM+ND), L(KV+ND)
      WRITE (3, '(37(''-''))')
C     Issue terminal progress message.
      WRITE (*, '(/, '' Completed data splitting for validation.'')')
      RETURN
      END
      SUBROUTINE ANOVA(ICALL)
C     For validating mode, merely compute grand and TRT means and set
C     KBAL.  For fitting mode, compute and write basic ANOVA.
C     ICALL=0 for fitting mode; n for run IRUN in validating mode.
C     Modified December 1990:  If missing cells (KBAL .EQ. -1), then
C     inflate NFR and NFT.  Add note about imputing increasing noise.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM, SI
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /SGNF/  SMS, PF, SI
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Count observations; compute TRT means and basic ANOVA.
C     Set KBAL=1 balanced; 0 unbalanced; -1 missing cells.
C     NX is the largest number of replicates encountered for any TRT,
C     and may be less than NR if a validation file has removed
C     some data from modelling.
      GM = 0.0D0
      GMW = 0.0D0
      NAXGE = 0
      NAGE = 0
      IRGE = 0
      KCL = NR
      KCU = 1
      DO 100 IGE=1,NGE
      NB = 0
      VB = 0.0D0
      DO 101 IR=1,NR
      IRGE = IRGE + 1
      IF (M(KKRGE+IRGE) .NE. 1)  GO TO 101
      NB = NB + 1
      R = Y(KYRGE+IRGE)
      VB = VB + R
  101 CONTINUE
      M(KKGE+IGE) = NB
      IF (NB .EQ. 0)  GO TO 100
      NAGE = NAGE + 1
      NAXGE = NAXGE + NB
      GMW = GMW + VB
      R = NB
      VB = VB / R
      GM = GM + VB
      V(KDGE+IGE) = VB
      IF (NB .LT. KCL)  KCL = NB
      IF (NB .GT. KCU)  KCU = NB
  100 CONTINUE
      NMCL = NGE - NAGE
      R = NAXGE
      GMW = GMW / R
      R = NAGE
      GM = GM / R
      GMU = GM
      NFR = NAGE - 1
      NFE = NAXGE - NAGE
      NFT = NFR + NFE
      KBAL = 1
      NXGE = NX * NGE
      IF (NAXGE .LT. NXGE)  KBAL = 0
      IF (NAGE .LT. NGE)  THEN
      KBAL = -1
      KPC = -1
      KLR = -1
      ENDIF
      SSE = 0.0D0
      SSR = 0.0D0
      SSRW = 0.0D0
      DO 102 IGE=1,NGE
      NB = M(KKGE+IGE)
      IF (NB .LE. 0)  GO TO 102
      VB = V(KDGE+IGE)
      R = VB - GM
      SSR = SSR + (R * R)
      R = VB - GMW
      FB = NB
      SSRW = SSRW + (R * R * FB)
      IA = (IGE - 1) * NR
      IB = KYRGE + IA
      IA = KKRGE + IA
      DO 103 IR=1,NR
      IF (M(IA+IR) .NE. 1)  GO TO 103
      R = Y(IB+IR)
      R = R - VB
      SSE = SSE + (R * R)
  103 CONTINUE
  102 CONTINUE
      R = NX
      SSR = SSR * R
      SST = SSR + SSE
      R = NFE
      EMS = 0.0D0
      IF (NFE .GE. 1)  EMS = SSE / R
      REMS = DSQRT(EMS)
      SSTW = SSRW + SSE
C     If MODE=2 model fitting or analyzing indirect information,
C     write results.
      IF (MODE .EQ. 1  .AND.  KII .EQ. 0)  RETURN
      IF (MODE .EQ. 1  .AND.  KII .EQ. 1  .AND. IRUN .GT. 0)  RETURN
      IF (ICALL .EQ. 0)  THEN
      WRITE (3, '(/, ''The data matrix, as read, is as follows.'')')
      NFE0 = NFE
      EMS0 = EMS
      REMS0 = REMS
      GMU0 = GMU
      ELSE
      WRITE (3, '(/, ''The data matrix, after removing'', I8,
     1 '' validation observations, is as follows.'')')  NVAL
      ENDIF
      WRITE (3, '(/, ''Measurement  '', A20)')  NMUN
      WRITE (3, '(''Number of '', A1, A3, I7)')  GNA, GNB, NG
      WRITE (3, '(''Number of '', A1, A3, I7)')  ENA, ENB, NE
      WRITE (3, '(''Number of '', A1, A3, I7)')  RNA, RNB, NX
      WRITE (3, '(''Number of possible observations'', I12)')  NXGE
      WRITE (3, '(''Number of actual observations'', I14)')  NAXGE
      IF (KBAL .EQ. 1)  WRITE (3, '(''No missing data;'',
     1 '' these data are balanced.'')')
      R = (NXGE - NAXGE) * 100
      PM = NXGE
      PM = R / PM
      IF (KBAL .NE. 1)  WRITE (3, '(''Missing data'', F6.2,
     1 ''%; these data are unbalanced.'')')  PM
      WRITE (3, '(''Number of possible '', A4, I10)')  TNM, NGE
      WRITE (3, '(''Number of actual '', A4, I12)')  TNM, NAGE
      IF (KBAL .GE. 0)  WRITE (3, '(''No missing cells.'')')
      R = NMCL * 100
      PM = NGE
      PM = R / PM
      IF (KBAL .EQ. -1)  WRITE (3, '(''Missing cells'', F6.2,
     1 ''%.'')')  PM
      IF (ICALL .EQ. 0)  THEN
      IF (EXDS .EQ. 'RAN'  .OR.  EXDS .EQ. 'ran')  THEN
      WRITE (3, '(/, ''The experimental design is RAN, completely'',
     1 '' randomized.'')')
      ELSE
      WRITE (3, '(/, ''The experimental design is '', A3,
     1 '', rather than RAN.'')')  EXDS
      ENDIF
      WRITE (3, '(''It may be worthwhile to use other programs to'',
     1 '' analyze the experimental'', /, ''design since this'',
     2 '' program addresses only the treatment design.'')')
      WRITE (3, '(/, ''Basic ANOVA for the data as read.'')')
      ELSE
      WRITE (3, '(/, ''Basic ANOVA for the data after removing'', I8,
     1 '' validation observations.'')')  NVAL
      ENDIF
      WRITE (3, '(/, 79(''-''), /, ''Source'', 11X, ''df'', 12X,
     1 ''SS'', 19X, ''MS'', 12X, ''Probability'', /, 79(''-''))')
      R = NFT
      SMTW = SSTW / R
      WRITE (3, '(''Total'', I15, 2F21.5)')  NFT, SSTW, SMTW
      CALL FTEST(NFR, SSRW, NFE, SSE)
      TRMS = SMS
      WRITE (3, '(''  '', A4, I14, 2F21.5, F13.8, A4)')  TNM, NFR,
     1 SSRW, SMS, PF, SI
      IF (NFE .GT. 0)  WRITE (3, '(''  Error'', I13, 2F21.5)')
     1 NFE, SSE, EMS
      WRITE (3, '(79(''-''))')
      IF (KCL .EQ. KCU)  THEN
      WRITE (3, '(/, ''Grand mean'', F19.5)')  GMW
      ELSE
      WRITE (3, '(/, ''Grand mean'', F19.5, ''  (Weighted by cell'',
     1 '' counts of'', I5, '' to'', I5, ''.)'')')  GMW, KCL, KCU
      ENDIF
      IF (NFE .GT. 0)  WRITE (3, '(''Root Error MS'', F16.5, /)')  REMS
      IF (KBAL .EQ. 1)  WRITE (3, '(''These data are balanced'',
     1 '' and there are no missing cells.'')')
      IF (KCL .EQ. KCU)  GO TO 10  
      WRITE (3, '(''Hereafter these unbalanced data may be'',
     1 '' approximated as balanced data,'', /, ''computing the '', A4,
     2 '' SS as if all of the'', I7, 1X, A4, '' with any data had the'',
     3 '' full'', /, A1, A3, '' ='', I3, '' observations, but leaving'', 
     4 '' the error df and SS unaltered.'', /, ''Note that this'',
     5 '' procedure makes F-tests more significant than they ought'',
     6 '' to be.'', /, ''Therefore interpret all following F-tests'',
     7 '' conservatively, especially if'', /, ''marginally'',
     8 '' significant.'')')  TNM, NAGE, TNM, RNA, RNB, NX
      WRITE (3, '(''The resulting basic ANOVA is as follows.'')')
      WRITE (3, '(/, 79(''-''), /, ''Source'', 11X, ''df'', 12X,
     1 ''SS'', 19X, ''MS'', 12X, ''Probability'', /, 79(''-''))')
      R = NFT
      SMT = SST / R
      WRITE (3, '(''Total'', I15, 2F21.5)')  NFT, SST, SMT
      CALL FTEST(NFR, SSR, NFE, SSE)
      TRMS = SMS
      WRITE (3, '(''  '', A4, I14, 2F21.5, F13.8, A4)')  TNM, NFR,
     1 SSR, SMS, PF, SI
      IF (NFE .GT. 0)  WRITE (3, '(''  Error'', I13, 2F21.5)')
     1 NFE, SSE, EMS
      WRITE (3, '(79(''-''))')
      WRITE (3, '(/, ''Grand mean'', F19.5, ''  (Not weighted by'',
     1 '' cell counts.)'')')  GM
      IF (NFE .GT. 0)  WRITE (3, '(''Root Error MS'', F16.5)')  REMS
C     Assess magnitude of AMMI adjustments.
   10 IF (NFE .EQ. 0  .OR.  TRMS .LT. EMS)  GO TO 12
      PNSE = 100.0D0 * (EMS / TRMS)
      PPAT = 100.0D0 - PNSE
      RESS = DFLOAT(NFR) * EMS
      R = 100.0D0 * (RESS / SSR)
      RERMS = DSQRT((DFLOAT(NFR) / DFLOAT(NX * NGE)) * EMS)
      PGM = 100.0D0 * (RERMS / GM)
      WRITE (3, '(/, ''The '', A4, '' MS contains the Error MS'',
     1 '' variance component, so the '', A4, /, ''contain'', F7.2,
     2 ''% pattern related to the treatment design, and'', /,
     3 ''contain'', F7.2, ''% noise related to the experimental'',
     4 '' design.'', /, ''An ideal statistical analysis will'',
     5 '' recover the pattern in its model,'', /
     6 ''while relegating the noise to a discarded residual,'', /,
     7 ''in order to increase predictive accuracy.'', /)')
     8 TNM, TNM, PPAT, PNSE
      WRITE (3, '(''Discarding a residual SS of'', F21.5, '', or'',
     1 F7.2, ''% of this '', A4, '' SS,'', /, ''causes AMMI'',
     2 '' adjusted means to differ from unadjusted means by a'', /,
     3 ''root mean square of'', F15.5, '', or'', F7.2,
     4 ''% of the unweighted grand mean.'', /, ''Assuming normality,'',
     5 '' 5% of these adjustments exceed this RMS'', /,
     6 ''by a factor of 1.960, 1% by 2.576, and 0.1% by 3.291.'')')
     7 RESS, R, TNM, RERMS, PGM
      IF (KBAL .EQ. -1)  THEN
      WRITE (3, '(/, ''Subsequent AMMI modelling'',
     1 '' will estimate the'', I7, '' missing cells,'', /,
     2 ''thereby inflating the '', A4, '' and total df and SS, and'',
     3 '' also altering the'', /, ''(unweighted) grand mean.'')')
     4 NGE-NAGE, TNM
      NFR = NGE - 1
      NFT = NFR + NFE
      WRITE (3, '(/, ''Given data for'', I7, '' actual '', A4,
     1 '' out of'', I7, '' possible (actual plus imputed),'', /,
     2 ''imputing will increase the number of estimates by a'',
     3 '' factor of'', F7.3, ''.'')')  NAGE, TNM, NGE,
     4 DFLOAT(NGE) / DFLOAT(NAGE)
      WRITE (3, '(''Hence the amount of noise SS indicated above for'',
     1 '' just the actual data'', /, ''is likely to be increased'',
     2 '' by the imputing process by at least this factor,'', /,
     3 ''and perhaps by a somewhat larger factor since naturally'',
     4 '' the imputed estimates'', /, ''tend to be somewhat less'',
     5 '' accurate than the regular estimates with actual data.'')')
      ENDIF
C     Issue terminal progress message.
   12 IF (ICALL .EQ. 0)  WRITE (*, '(/, '' Completed basic ANOVA.'')')
      IF (ICALL .GT. 0)  WRITE (*, '(/, '' Recomputed basic ANOVA.'')')
      RETURN
      END
      SUBROUTINE AMMI(MN, MX)
C     Fit AMMI models with MN to MX IPCA axes.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /PRED/  RPD(8,3), RPDT, KRPD(8), NRPD(2)
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Initialize variables.
      IF (IRUN .GT. 1)  GO TO 10
      DO 100 LD=1,MXIPCA+1
      KRPD(LD) =  0
      RPD(LD,1) = 0.0D0
  100 RPD(LD,2) = 0.0D0
      NRPD(1) = 0
      NRPD(2) = 0
      RPDT = 0.0D0
C     Compute GEN and ENV counts and unweighted means.
   10 CONTINUE 
      DO 101 NGD=1,NG
      NC = 0
      XB = 0.0D0
      DO 102 NED=1,NE
      KS = ((NED - 1) * NG) + NGD
      IF (M(KKGE+KS) .LE. 0)  GO TO 102
      NC = NC + 1
      XB = XB + V(KDGE+KS)
  102 CONTINUE
      IF (NC .EQ. 0)  GO TO 11
      L(KCG+NGD) = NC
  101 V(KBG+NGD) = XB / DBLE(NC)
      GO TO 12
   11 WRITE (*, '('' ERROR:  '', A1, A3, I6, '' has no data.'')')
     1 GNA, GNB, NGD
      STOP
   12 KS = 0
      DO 103 NED=1,NE
      NC = 0
      XB = 0.0D0
      DO 104 NGD=1,NG
      KS = KS + 1
      IF (M(KKGE+KS) .LE. 0)  GO TO 104
      NC = NC + 1
      XB = XB + V(KDGE+KS)
  104 CONTINUE
      IF (NC .EQ. 0)  GO TO 13
      L(KCE+NED) = NC
  103 V(KBE+NED) = XB / DBLE(NC)
      GO TO 14
   13 WRITE (*, '('' ERROR:  '', A1, A3, I6, '' has no data.'')')
     1 ENA, ENB, NED
      STOP
C     For the first run, initialize IPCA GEN scores for axes 1 to MX
C     with vectors of arbitrary but unequal numbers.
   14 IF (MX .EQ. 0  .OR.  IRUN .GT. 1)  GO TO 15
      X = (FG - 1.0D0) / 2.0D0
      DO 105 NGD=1,NG
      DO 106 MA=1,MX
      KS = KPGN + ((MA - 1) * NG) + NGD
  106 V(KS) = X
  105 X = X - 1.0D0
C     Compute AMMI models with MN to MX IPCA axes.
   15 MNA = MN
      IF (KBAL .GE. 0)  MNA = MX
      DO 110 MDI=MNA,MX
C     Initialize interaction residuals in V(IAGE), imputing 0 for
C     missing cells.  Also impute additive model for missing cells
C     in V(IDGE).
      MD = MDI
      KS = 0
      DO 111 NED=1,NE
      X = V(KBE+NED) - GM
      DO 112 NGD=1,NG
      KS = KS + 1
      IF (M(KKGE+KS) .GT. 0)  THEN
      V(KS) = V(KDGE+KS) - (X + V(KBG+NGD))
      ELSE
      V(KS) = 0.0D0
      V(KDGE+KS) = X + V(KBG+NGD)
      ENDIF
  112 CONTINUE
  111 CONTINUE
C     Skip IPCA calculations for AMMI0 (ANOVA) model.
      IF (MD .EQ. 0)  GO TO 30
C     If missing cells, cycle IPCA iterations.
      NCY = 1
      IF (KBAL .EQ. -1)  NCY = MXCYCL
      DO 120 NDI=1,NCY
      ND = NDI
C     Calculate IPCA axes 1 to MD, using the power method.
      CALL IPCA(MN, MX, ND, NCY)
C     If no missing cells, conclude AMMI calculations.
      IF (KBAL .GE. 0)  GO TO 30
C     If missing cells, update imputed data and find maximum shift.
      RS = 0.0D0
      KS = 0
      DO 121 NED=1,NE
      X = V(KBE+NED) - GM
      DO 122 NGD=1,NG
      KS = KS + 1
      IF (M(KKGE+KS) .GT. 0)  GO TO 122
      VH = X + V(KBG+NGD)
      DO 123 ID=1,MD
      IM = ID - 1
  123 VH = VH + (V(KPGN+(IM*NG)+NGD) * V(KPEN+(IM*NE)+NED))
      NS = KDGE + KS
      R = DABS(V(NS) - VH)
      IF (R .GT. RS)  RS = R
      V(NS) = VH
  122 CONTINUE
  121 CONTINUE
C     If missing cells, test convergence.
      XS = RS / GM
      IF (KLV .EQ. 3)  WRITE (*, '(''     Maximum shift'', F17.5,
     1 '' (or'', F11.8, '' times the grand mean).'')')  RS, XS
      IF (XS .LE. RSHFT)  GO TO 30
C     If missing cells, update additive model and initialize
C     interaction residuals for the next iteration.
      GM = 0.0D0
      DO 124 NGD=1,NG
      XB = 0.0D0
      DO 125 NED=1,NE
      KS = ((NED - 1) * NG) + NGD
  125 XB = XB + V(KDGE+KS)
      GM = GM + XB
  124 V(KBG+NGD) = XB / FE
      GM = GM / FGE
      KS = KDGE
      NS = 0
      DO 126 NED=1,NE
      XB = 0.0D0
      DO 127 NGD=1,NG
      KS = KS + 1
  127 XB = XB + V(KS)
      XB = XB / FG
      V(KBE+NED) = XB
      X = XB - GM
      DO 128 NGD=1,NG
      NS = NS + 1
      R = 0.0D0
      IF (M(KKGE+NS) .GT. 0)  R = V(KDGE+NS) - (X + V(KBG+NGD))
  128 V(NS) = R
  126 CONTINUE
  120 CONTINUE
      IF (KLV .EQ. 3)  WRITE (*, '('' WARNING:  AMMI cycling'',
     1 '' terminates while shift'', F12.9, '' exceeds'', F12.9,
     2 ''.'')')  XS, RSHFT
C     If data splitting is used, compute prediction statistics.
   30 IF (KVAL .EQ. 1)  CALL STATS(MX)
C     If MODE=2 model fitting, write detailed results.
      IF (KII .EQ. 1)  GO TO 40
      IF (MODE .EQ. 1)  GO TO 110
      CALL OUT1
      CALL OUT2
      CALL OUT3
      IF (KPC .EQ. 1)  CALL PCA
      IF (KLR .EQ. 1)  CALL LNR
  110 CONTINUE
C     Issue terminal progress message.
      IF (KLV .EQ. 2)  WRITE (*, '('' Run'', I5, '' of'', I5, ''.'')')
     1 IRUN, MRUN
C     If KVAL=1 data splitting, IRUN=MRUN last run, and KII=0,
C     write prediction statistics summary.
   40 IF (KVAL .EQ. 1  .AND.  IRUN .EQ. MRUN  .AND.  KII .EQ. 0)
     1 CALL PREDS(MX)
      RETURN
      END
      SUBROUTINE IPCA(MN, MX, ND, NCY)
C     Compute IPCA or PCA scores using the power method.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /PRED/  RPD(8,3), RPDT, KRPD(8), NRPD(2)
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Calculate scores for axes 1 to MD.
      DO 100 ID=1,MD
      ITER = 0
      KG = KPGN + ((ID - 1) * NG)
      KE = KPEN + ((ID - 1) * NE)
C     Before the first iteration, make the current GEN scores a unit
C     vector, and initialize the old GEN scores to these same values.
      R = 0.0D0
      DO 101 NGD=1,NG
      X = V(KG+NGD)
  101 R = R + (X * X)
      R = DSQRT(R)
      DO 102 NGD=1,NG
      X = V(KG+NGD) / R
      V(KG+NGD) = X
  102 V(KSG+NGD) = X
C     Iterate PCA calculations, using the power method.
      DO 110 IX=1,MXITER
      ITER = ITER + 1
C     Compute ENV scores from GEN scores.
      EV = 0.0D0
      NS = 0
      DO 121 NED=1,NE
      X = 0.0D0
      DO 122 NGD=1,NG
      NS = NS + 1
  122 X = X + (V(NS) * V(KG+NGD))
      EV = EV + (X * X)
  121 V(KE+NED) = X
      EVSQ = DSQRT(EV)
      DO 123 NED=1,NE
      NS = KE + NED
  123 V(NS) = V(NS) / EVSQ
C     Compute GEN scores from ENV scores.
      EV = 0.0D0
      DO 124 NGD=1,NG
      X = 0.0D0
      DO 125 NED=1,NE
      NS = ((NED - 1) * NG) + NGD
  125 X = X + (V(NS) * V(KE+NED))
      EV = EV + (X * X)
  124 V(KG+NGD) = X
      EVSQ = DSQRT(EV)
C     Test convergence, retaining the old GEN scores as a unit vector.
      SFT = 0.0D0
      DO 126 NGD=1,NG
      NS = KG + NGD
      X = V(NS) / EVSQ
      V(NS) = X
      NS = KSG + NGD
      R = DABS(X - V(NS))
      IF (R .GT. SFT)  SFT = R
  126 V(NS) = X
      IF (SFT .LE. DEL)  GO TO 10
  110 CONTINUE
C     Scale IPCA scores and compute residuals.
   10 SSP(ID) = EV * DBLE(NX)
      R = DSQRT(EVSQ)
      DO 127 NGD=1,NG
      KS = KG + NGD
  127 V(KS) = V(KS) * R
      KS = 0
      DO 128 NED=1,NE
      NS = KE + NED
      X = V(NS) * R
      V(NS) = X
      DO 129 NGD=1,NG
      KS = KS + 1
  129 V(KS) = V(KS) - (X * V(KG+NGD))
  128 CONTINUE
C     Issue terminal progress message.
      IF (KLV .NE. 3)  GO TO 100
      IF (MN .GE. 0)  THEN
      WRITE (*, '('' Run'', I5, '' of'', I5, '', model'', I5, '' of'',
     1 I5, '' to'', I5, '','', /, ''     cycle'', I5, '' of'', I5,
     2 '' maximum, axis'', I5, '' of'', I5, '', iterations'', I5,
     3 ''.'')')  IRUN, MRUN, MD, MN, MX, ND, NCY, ID, MD, ITER
      ELSE
      WRITE (*, '('' PCA axis'', I2, '' of'', I2, '', iterations'',
     1 I5, ''.'')')  ID, MD, ITER
      ENDIF
      IF (SFT .GT. DEL)  WRITE (*, '('' WARNING:  PCA iterating'',
     1 '' terminates while shift'', F12.9, '' exceeds'', F12.9,
     2 ''.'')')  SFT, DEL
  100 CONTINUE
      RETURN
      END
      SUBROUTINE OUT1
C     Write AMMI model parameters and ANOVA table.
C     Also write machine-readable file if requested.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*2   SA, SB
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM, SI
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /SGNF/  SMS, PF, SI
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Compute additive ANOVA.
      FNX = NX
      NFR = NGE - 1
      NFGEN = NG - 1
      NFENV = NE - 1
      NFGE = NFGEN * NFENV
      SSGEN = 0.0D0
      DO 100 NGD=1,NG
      R = V(KBG+NGD) - GM
  100 SSGEN = SSGEN + (R * R)
      SSGEN = SSGEN * FE * FNX
      SSENV = 0.0D0
      DO 101 NED=1,NE
      R = V(KBE+NED) - GM
  101 SSENV = SSENV + (R * R)
      SSENV = SSENV * FG * FNX
      SSR = 0.0D0
      SSE = 0.0D0
      KR = 0
      DO 102 IGE=1,NGE
      X = V(KDGE+IGE)
      DO 103 NRD=1,NR
      KR = KR + 1
      IF (M(KKRGE+KR) .LE. 0)  GO TO 103
      R = Y(KYRGE+KR)
      R = R - X
      SSE = SSE + (R * R)
  103 CONTINUE
      R = X - GM
  102 SSR = SSR + (R * R)
      SSR = SSR * FNX
      SST = SSR + SSE
      SSGE = SSR - SSGEN - SSENV
      EMS = 0.0D0
      IF (NFE .GE. 1)  EMS = SSE / DBLE(NFE)
      REMS = DSQRT(EMS)
C     Write additive model.
      CALL INDEX(NG, KBG, KIG)
      CALL INDEX(NE, KBE, KIE)
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' Means for Model AMMI'', I1, ''.'')')  GNA, GNB, MD
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 75(''-''), /, A1, A3, ''   Name       Mean'',
     1 ''      Count    |  Index  Name       Mean      Count'', /,
     2 75(''-''))')  GNA, GNB
      DO 104 NGD=1,NG
      NCA = L(KCG+NGD)
      SA = '  '
      IF (NCA .LT. NE)  SA = ' I'
      NB = L(KIG+NGD)
      NCB = L(KCG+NB)
      SB = '  '
      IF (NCB .LT. NE)  SB = ' I'
  104 WRITE (3, '(I5, 2X, A4, F15.5, I7, A2, ''  |'', I7, 2X, A4,
     1 F15.5, I7, A2)')  NGD, C(KNG+NGD), V(KBG+NGD), NCA, SA, NB,
     2 C(KNG+NB), V(KBG+NB), NCB, SB
      WRITE (3, '(75(''-''))')
      WRITE (3, '(/, ''Grand mean '', F15.5, 2X, A20)')  GM, NMUN
      IF (KBAL .EQ. -1)  WRITE (3, '(/, ''I = Imputed data for ('',
     1 I5, '' - Count) '', A1, A3, '' with missing cells.'')')
     2 NE, ENA, ENB
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' Means for Model AMMI'', I1, ''.'')')  ENA, ENB, MD
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 75(''-''), /, A1, A3, ''   Name       Mean'',
     1 ''      Count    |  Index  Name       Mean      Count'', /,
     2 75(''-''))')  ENA, ENB
      DO 105 NED=1,NE
      NCA = L(KCE+NED)
      SA = '  '
      IF (NCA .LT. NG)  SA = ' I'
      NB = L(KIE+NED)
      NCB = L(KCE+NB)
      SB = '  '
      IF (NCB .LT. NG)  SB = ' I'
  105 WRITE (3, '(I5, 2X, A4, F15.5, I7, A2, ''  |'', I7, 2X, A4,
     1 F15.5, I7, A2)')  NED, C(KNE+NED), V(KBE+NED), NCA, SA, NB,
     2 C(KNE+NB), V(KBE+NB), NCB, SB
      WRITE (3, '(75(''-''))')
      WRITE (3, '(/, ''Grand mean '', F15.5, 2X, A20)')  GM, NMUN
      IF (KBAL .EQ. -1)  WRITE (3, '(/, ''I = Imputed data for ('',
     1 I5, '' - Count) '', A1, A3, '' with missing cells.'')')
     2 NG, GNA, GNB
C     Write IPCA scores.
      DO 110 ID=1,MD
      KG = KPGN + ((ID - 1) * NG)
      KE = KPEN + ((ID - 1) * NE)
      CALL INDEX(NG, KG, KIG)
      CALL INDEX(NE, KE, KIE)
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' IPCA Axis'', I2, '' Scores for Model AMMI'', I1,
     2 ''.'')')  GNA, GNB, ID, MD
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 67(''-''), /, A1, A3, ''   Name           '',
     1 ''Score      |  Index  Name           Score'', /, 67(''-''))')
     2 GNA, GNB
      DO 111 NGD=1,NG
      NB = L(KIG+NGD)
  111 WRITE (3, '(I5, 2X, A4, F20.7, ''  |'', I7, 2X, A4, F20.7)')
     1 NGD, C(KNG+NGD), V(KG+NGD), NB, C(KNG+NB), V(KG+NB)
      WRITE (3, '(67(''-''))')
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' IPCA Axis'', I2, '' Scores for Model AMMI'', I1,
     2 ''.'')')  ENA, ENB, ID, MD
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 67(''-''), /, A1, A3, ''   Name           '',
     1 ''Score      |  Index  Name           Score'', /, 67(''-''))')
     2 ENA, ENB
      DO 112 NED=1,NE
      NB = L(KIE+NED)
  112 WRITE (3, '(I5, 2X, A4, F20.7, ''  |'', I7, 2X, A4, F20.7)')
     1 NED, C(KNE+NED), V(KE+NED), NB, C(KNE+NB), V(KE+NB)
  110 WRITE (3, '(67(''-''))')
C     Write ANOVA table.
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''ANOVA Table for Model AMMI'', I1, ''.'')')  MD
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 79(''-''), /, ''Source'', 12X, ''df'', 15X,
     1 ''SS'', 18X, ''MS'', 10X, ''Probability'', /, 79(''-''))')
      WRITE (3, '(''Total         '', I7, F22.5, F20.5)')  NFT, SST,
     1 SST / DBLE(NFT)
      CALL FTEST(NFR, SSR, NFE, SSE)
      WRITE (3, '(2X, A4, 8X, I7, F22.5, F20.5, F12.7, A4)')
     1 TNM, NFR, SSR, SMS, PF, SI
      CALL FTEST(NFGEN, SSGEN, NFE, SSE)
      WRITE (3, '(4X, A1, A3, 6X, I7, F22.5, F20.5, F12.7, A4)')
     1 GNA, GNB, NFGEN, SSGEN, SMS, PF, SI
      CALL FTEST(NFENV, SSENV, NFE, SSE)
      WRITE (3, '(4X, A1, A3, 6X, I7, F22.5, F20.5, F12.7, A4)')
     1 ENA, ENB, NFENV, SSENV, SMS, PF, SI
      CALL FTEST(NFGE, SSGE, NFE, SSE)
      WRITE (3, '(4X, A1, '' x '', A1, 5X, I7, F22.5, F20.5, F12.7,
     1 A4)')  GNA, ENA, NFGE, SSGE, SMS, PF, SI
      NF = NG + NE - 1
      NFL = NFGE
      SSL = SSGE
      IF (MD .EQ. 0)  GO TO 10
      DO 120 LD=1,MD
      NF = NF - 2
      SSD = SSP(LD)
      CALL FTEST(NF, SSD, NFE, SSE)
      WRITE (3, '(''      IPCA'', I2, 2X, I7, F22.5, F20.5, F12.7,
     1 A4)')  LD, NF, SSD, SMS, PF, SI
      NFL = NFL - NF
  120 SSL = SSL - SSD
      IF (NFL .LE. 0)  GO TO 10
      CALL FTEST(NFL, SSL, NFE, SSE)
      WRITE (3, '(''      Residual'', I7, F22.5, F20.5, F12.7, A4)')
     1 NFL, SSL, SMS, PF, SI
   10 IF (NFE .GT. 0)  WRITE (3, '(''  Error       '', I7, F22.5,
     1 F20.5)')  NFE, SSE, EMS
      WRITE (3, '(79(''-''))')
      WRITE (3, '(/, ''Grand mean'', F15.5, 2X, A20)')     GM, NMUN
      IF (KBAL .GE. 0)  THEN
      WRITE (3, '(/, ''There are no missing cells.'')')
      ELSE
      WRITE (3, '(/, ''There are'', I7, '' missing cells supplied'',
     1 '' imputed values.'')')  NMCL
      ENDIF
C     Write commentary on pattern and noise.
      IF (MD .GT. 0  .AND.  NR .GT. 1)  THEN
      WRITE (3, '(/, ''The noise SS in the interaction can be'',
     1 '' estimated as '', A1, ''x'', A1, '' df times Error MS.'')')
     2 GNA, ENA
      DFMS = FLOAT(NFGE) * EMS
      IF (DFMS .GT. SSGE)  THEN
      WRITE (3, '(''This value'', F22.5, '' exceeds the '', A1, ''x'',
     1 A1, '' SS,'', /, ''so this interaction is probably mostly'',
     2 '' noise.'')')  DFMS, GNA, ENA
      ELSE
      WRITE (3, '(''Accordingly, the '', A1, ''x'', A1, '' SS'',
     1 '' contains approximately:'')')  GNA, ENA
      R = SSGE - DFMS
      X = 100.0D0 * (R / SSGE)
      WRITE (3, '(A1, ''x'', A1, '' Pattern'', F25.5, '' or'', F7.2,
     1 ''%'')')  GNA, ENA, R, X
      X = 100.0D0 * (DFMS / SSGE)
      WRITE (3, '(A1, ''x'', A1, '' Noise  '', F25.5, '' or'', F7.2,
     1 ''%'')')  GNA, ENA, DFMS, X
      WRITE (3, '(A1, ''x'', A1, '' Total  '', F25.5)')  GNA, ENA, SSGE
      WRITE (3, '(''Ordinarily, there is considerable selectivity,'',
     1 '' with pattern recovered mostly'', /, ''in early IPCA axes,'',
     2 '' and noise recovered mostly in late IPCA axes.'')')
      WRITE (3, '(''Consequently, these estimates provide a rough'',
     1 '' guide for model diagnosis by'', /, ''keeping early axes'',
     2 '' that are mostly pattern and relegating the others to a'', /,
     3 ''discarded residual.  MATMODEL in validation mode provides a'',
     4 '' more'', /, ''definitive diagnosis that frequently confirms'',
     5 '' this much simpler method.'')')
      ENDIF
      WRITE (3, '(''The above calculation of the Error MS ignores'',
     1 '' the experimental design, so it'', /, ''may give a large'',
     2 '' estimate.  If a more appropriate estimate is'',
     3 '' available from'', /, ''other software, it may be'',
     4 '' substituted in the above reasoning.'')')
      ENDIF
C     Write machine-readable file if requested.
      IF (KQ .EQ. 0)  RETURN
      OPEN (4, FILE=FNMCR, STATUS='NEW')
C     Write AMMI model parameters.
      WRITE (4, '(''AMMI'', I1, '' model for data with'',
     1 3(I6, 1X, A1, A3), '' and grand mean'', F17.5, ''.'')')
     2 MFIP, NG, GNA, GNB, NE, ENA, ENB, NR, RNA, RNB, GM
      DO 130 NGD=1,NG
  130 WRITE (4, '(I5, 2X, A4, F20.5, 7F20.7)')  NGD, C(KNG+NGD),
     1 V(KBG+NGD), (V(KPGN+((NB-1)*NG)+NGD), NB=1,MFIP)
      DO 131 NED=1,NE
  131 WRITE (4, '(I5, 2X, A4, F20.5, 7F20.7)')  NED, C(KNE+NED),
     1 V(KBE+NED), (V(KPEN+((NB-1)*NE)+NED), NB=1,MFIP)
C     Write model expected values, beginning with AMMI1.
      IF (KQ .EQ. 1)  GO TO 20
      IF (MD .GE. 1)  THEN
      DO 200 ND=1,MD
      WRITE (4, '(/, ''AMMI'', I1, '' Expected Values For '',
     1 A4, ''.'')')  ND, NMDB
      DO 201 NED=1,NE
      WRITE (4, '(A4)')  C(KNE+NED)
      EMGM = V(KBE+NED) - GM
      DO 202 NGD=1,NG
      R = EMGM + V(KBG+NGD)
      DO 203 NXD=1,ND
  203 R = R + (V(KPGN+((NXD-1)*NG)+NGD) * V(KPEN+((NXD-1)*NE)+NED))
  202 V(KSG+NGD) = R
  201 WRITE (4, FZ)  (V(KSG+NGD), NGD=1,NG)
  200 CONTINUE
      ENDIF
   20 CLOSE (4)
      RETURN
      END
      SUBROUTINE OUT2
C     Write biplots and model expected values.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA,   G(75), IM
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM,   SI
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /MESW/  KME
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Write AMMI1 biplot of main effect and IPCA1.
      IF (MD .EQ. 0)  GO TO 12
      VXMIN = V(KBG+1)
      VXMAX = VXMIN
      DO 100 LD=1,NG
      V(KSG+LD) = V(KPGN+LD)
      VX = V(KBG+LD)
      IF (VX .LT. VXMIN)  VXMIN = VX
  100 IF (VX .GT. VXMAX)  VXMAX = VX
      DO 101 LD=1,NE
      V(KSE+LD) = V(KPEN+LD)
      VX = V(KBE+LD)
      IF (VX .LT. VXMIN)  VXMIN = VX
  101 IF (VX .GT. VXMAX)  VXMAX = VX
      FX = 75.0D0 / (VXMAX - VXMIN)
      NGPE = NG + NE
      CALL INDEX(NGPE, KSG, KIG)
      VYMIN = V(KSG+L(KIG+NGPE))
      VYMAX = V(KSG+L(KIG+1))
      FY = 50.0D0 / (VYMAX - VYMIN)
      LXB = MIN0(75, INT(FX * (GM - VXMIN)) + 1)
      LYB = MAX0(1, 50 + INT(FY * VYMIN))
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''Biplot of Means and IPCA1.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, ''The abscissa (X-axis) plots means from'',
     1 F17.5, '' to'', F17.5, ''.'', /, ''The ordinate (Y-axis)'',
     2 '' plots IPCA1 from'', F17.5, '' to'', F17.5, ''.'')')
     3 VXMIN, VXMAX, VYMIN, VYMAX
      WRITE (3, '(''Means are in units of '', A20,
     1 '' and IPCA1 the square root of this.'')')  NMUN
      WRITE (3, '(''The points are '', A1, '' or '', A1, '' or * for '',
     1 A1, A3, '' or '', A1, A3, '' or multiple, and + for'',
     2 '' centroid.'')')  GNA, ENA, GNA, GNB, ENA, ENB
      R = SSGEN + SSENV + SSP(1)
      X = 100.0D0 * (R / SSR)
      R = DSQRT((SSR - R) / DFLOAT(NX * NAGE))
      WRITE (3, '(''This graph accounts for'', F7.2, ''% of the '',
     1 A4, '' sum of squares.'')')  X, TNM
      WRITE (3, '(''The root mean square residual for the AMMI1'',
     1 '' model is'', F15.5, ''.'')')  R
      WRITE (3, '(/, 1X, 75(''-''))')
      LD = 1
      LI = L(KIG+1)
      LX = MIN0(75, INT(FX * (V(KBG+LI) - VXMIN)) + 1)
      LY = MAX0(1, 50 - INT(FY * (V(KSG+LI) - VYMIN)))
      DO 102 LN=1,50
      DO 103 MDD=1,75
  103 G(MDD) = ' '
   10 IF (LY .EQ. LN)  THEN
      IF (G(LX) .NE. ' ')  THEN
      G(LX) = '*'
      ELSE
      IF (LI .LE. NG)  THEN
      G(LX) = GNA
      ELSE
      G(LX) = ENA
      ENDIF
      ENDIF
      LD = LD + 1
      LY = 0
      IF (LD .GT. NGPE)  GO TO 10
      LI = L(KIG+LD)
      LX = MIN0(75, INT(FX * (V(KBG+LI) - VXMIN)) + 1)
      LY = MAX0(1, 50 - INT(FY * (V(KSG+LI) - VYMIN)))
      GO TO 10
      ELSE
      IF (G(LXB) .EQ. ' ')  G(LXB) = '|'
      IF (LN .NE. LYB)  GO TO 11
      DO 104 MDD=1,75
  104 IF (G(MDD) .EQ. ' ')  G(MDD) = '-'
      IF (G(LXB) .EQ. '|')  G(LXB) = '+'
   11 WRITE (3, '(''|'', 75A1, ''|'')')  G
      ENDIF
  102 CONTINUE
      WRITE (3, '(1X, 75(''-''))')
C     Write AMMI2 biplot of IPCA1 and IPCA2.
      IF (MD .LT. 2)  GO TO 12
      VXMIN = VYMIN
      VXMAX = VYMAX
      DO 200 LD=1,NG
  200 V(KSG+LD) = V(KPGN+NG+LD)
      DO 201 LD=1,NE
  201 V(KSE+LD) = V(KPEN+NE+LD)
      FX = 75.0D0 / (VXMAX - VXMIN)
      NGPE = NG + NE
      CALL INDEX(NGPE, KSG, KIG)
      VYMIN = V(KSG+L(KIG+NGPE))
      VYMAX = V(KSG+L(KIG+1))
      FY = 50.0D0 / (VYMAX - VYMIN)
      LXB = MIN0(75, INT(-FX * VXMIN) + 1)
      LYB = MAX0(1, 50 + INT(FY * VYMIN))
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''Biplot of IPCA1 and IPCA2.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, ''The abscissa (X-axis) plots IPCA1 from'',
     1 F17.5, '' to'', F17.5, ''.'', /, ''The ordinate (Y-axis)'',
     2 '' plots IPCA2 from'', F17.5, '' to'', F17.5, ''.'')')
     3 VXMIN, VXMAX, VYMIN, VYMAX
      WRITE (3, '(''The units are the square root of '', A20,
     1 '' for both axes.'')')  NMUN
      WRITE (3, '(''The points are '', A1, '' or '', A1, '' or * for '',
     1 A1, A3, '' or '', A1, A3, '' or multiple, and + for'',
     2 '' centroid.'')')  GNA, ENA, GNA, GNB, ENA, ENB
      X = 100.0D0 * ((SSP(1) + SSP(2)) / SSGE)
      WRITE (3, '(''This graph accounts for'', F7.2, ''% of the '',
     1 A1, ''x'', A1, '' sum of squares.'')')  X, GNA, ENA
      R = SSGEN + SSENV + SSP(1) + SSP(2)
      R = DSQRT((SSR - R) / DFLOAT(NX * NAGE))
      WRITE (3, '(''The root mean square residual for the AMMI2'',
     1 '' model is'', F15.5, ''.'')')  R
      WRITE (3, '(/, 1X, 75(''-''))')
      LD = 1
      LI = L(KIG+1)
      IF (LI .LE. NG)  THEN
      R = V(KPGN+LI)
      ELSE
      R = V(KPEN+LI-NG)
      ENDIF
      LX = MIN0(75, INT(FX * (R - VXMIN)) + 1)
      LY = MAX0(1, 50 - INT(FY * (V(KSG+LI) - VYMIN)))
      DO 202 LN=1,50
      DO 203 MDD=1,75
  203 G(MDD) = ' '
   20 IF (LY .EQ. LN)  THEN
      IF (G(LX) .NE. ' ')  THEN
      G(LX) = '*'
      ELSE
      IF (LI .LE. NG)  THEN
      G(LX) = GNA
      ELSE
      G(LX) = ENA
      ENDIF
      ENDIF
      LD = LD + 1
      LY = 0
      IF (LD .GT. NGPE)  GO TO 20
      LI = L(KIG+LD)
      IF (LI .LE. NG)  THEN
      R = V(KPGN+LI)
      ELSE
      R = V(KPEN+LI-NG)
      ENDIF
      LX = MIN0(75, INT(FX * (R - VXMIN)) + 1)
      LY = MAX0(1, 50 - INT(FY * (V(KSG+LI) - VYMIN)))
      GO TO 20
      ELSE
      IF (G(LXB) .EQ. ' ')  G(LXB) = '|'
      IF (LN .NE. LYB)  GO TO 21
      DO 204 MDD=1,75
  204 IF (G(MDD) .EQ. ' ')  G(MDD) = '-'
      IF (G(LXB) .EQ. '|')  G(LXB) = '+'
   21 WRITE (3, '(''|'', 75A1, ''|'')')  G
      ENDIF
  202 CONTINUE
      WRITE (3, '(1X, 75(''-''))')
C     Perform mega-environment analysis for AMMI1 model.
   12 IF (MD .GE. 1  .AND.  KME .EQ. 1)  CALL MEGAENV
C     Write model expected values and residuals.
      IF (KPAM .EQ. -1)  RETURN
      KS = 0
      DO 105 NED=1,NE
      X = V(KBE+NED) - GM
      DO 106 NGD=1,NG
      KS = KS + 1
      VH = X + V(KBG+NGD)
      IF (MD .EQ. 0)  GO TO 106
      DO 107 ND=1,MD
      NM = ND - 1
  107 VH = VH + (V(KPGN+(NM*NG)+NGD) * V(KPEN+(NM*NE)+NED))
  106 V(KS) = VH
  105 CONTINUE
      XLOSS = 0.0D0
      XGAIN = 0.0D0
      XDEM = 0.0D0
      NDFW = 0
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''Listing of data, AMMI'', I1, '' model, and residuals.'')')  MD
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, ''Results are arranged by '', A1, A3, '' and'',
     1 '' imputed data (if any) are marked with "I".'')')  ENA, ENB
      RMSR = NX
      RMSR = DSQRT(SSL / (RMSR * FGE))
      P05  = 1.959963985D0 * RMSR
      P01  = 2.575829304D0 * RMSR
      P001 = 3.290526731D0 * RMSR
      WRITE (3, '(/, ''In '', A20, '' the root mean square residual'',
     1 '' is'', F17.5, ''.'')')  NMUN, RMSR
      WRITE (3, '(''Large residuals exceeding this by a factor of'',
     1 '' 1.960, 2.576, or 3.291'', /, ''are marked with *, **, or'',
     2 '' *** respectively.  Assuming normality,'', /, ''5%, 1%, and'',
     3 '' 0.1% of the residuals exceed these limits.'', /)')
      KS = 0
      DO 108 NED=1,NE
      NS = KKGE + ((NED - 1) * NG)
      KSAM = (NED - 1) * NG
      KSDT = KDGE + KSAM
      CALL INDEX(NG, KSAM, KCG)
      CALL INDEX(NG, KSDT, KIG)
      NWA = L(KCG+1)
      NWD = L(KIG+1)
      XGN = 0.0D0
      IF (NWA .EQ. NWD)  GO TO 13
      NDFW = NDFW + 1
      XGN = V(KSAM+NWA) - V(KSAM+NWD)
      XGAIN = XGAIN + XGN
      XLOSS = XLOSS + (V(KSDT+NWD) - V(KSDT+NWA))
      IF (XGN .LE. XDEM)  GO TO 13
      XDEM = XGN
      NDEM = NED
      NGAM = NWA
      NGDT = NWD
   13 WRITE (3, '(/, 1X, A1, A3, I7, 2X, A4, ''   Mean'', F16.5,
     1 ''    AMMI'', I1, '' Gain'', F16.5)')  ENA, ENB, NED,
     2 C(KNE+NED), V(KBE+NED), MD, XGN
      WRITE (3, '(/, 1X, A1, A3, ''  Name   Count       DATA'', 15X,
     1 ''AMMI'', I1, 12X, ''Residual'')')  GNA, GNB, MD
      DO 109 NGD=1,NG
      KS = KS + 1
      VA = V(KS)
      VT = V(KDGE+KS)
      R = VT - VA
      NCT = M(KKGE+KS)
      IM = ' '
      IF (NCT .EQ. 0)  IM = 'I'
      RA = DABS(R)
      SI = '    '
      IF (RA .LT. P05)   GO TO 109
      SI = ' *  '
      IF (RA .GE. P01)   SI = ' ** '
      IF (RA .GE. P001)  SI = ' ***'
  109 WRITE (3, '(I5, 2X, A4, I7, F17.5, 1X, A1, 2F17.5, A4)')
     1 NGD, C(KNG+NGD), NCT, VT, IM, VA, R, SI
      WRITE (3, '(/, '' Rank  |  Index  Name       DATA'', 9X,
     1 ''|  Index  Name       AMMI'', I1)')  MD
      DO 110 NGD=1,NG
      IND = L(KIG+NGD)
      INA = L(KCG+NGD)
      NCT = M(NS+IND)
      IM = ' '
      IF (NCT .EQ. 0)  IM = 'I'
  110 WRITE (3, '(I5, ''  |'', I7, 2X, A4, F16.5, 1X, A1, ''  |'', I7,
     1 2X, A4, F16.5)')  NGD, IND, C(KNG+IND), V(KSDT+IND), IM, INA,
     2 C(KNG+INA), V(KSAM+INA)
  108 CONTINUE
      XLOSS = XLOSS / FE
      XGAIN = XGAIN / FE
      NSAW = NE - NDFW
      WRITE (3, '(/, /, ''AMMIF and AMMI'', I1, '' pick the same'',
     1 '' winner in'', I6, 1X, A1, A3, '' or'', F7.2, ''%,'', /,
     2 ''but they pick different winners in'', I10, 1X, A1, A3,
     3 '' or'', F7.2, ''%.'')')  MD, NSAW, ENA, ENB,
     4 100.0D0 * (DBLE(NSAW) / FE), NDFW, ENA, ENB,
     5 100.0D0 * (DBLE(NDFW) / FE)
      IF (NDFW .EQ. 0)  RETURN
      WRITE (3, '(/, ''Were AMMIF rankings more to be trusted, the'',
     1 '' average LOSS from selecting'', /, ''AMMI'', I1,
     2 '' winners would be'', F15.5, '', or'', F8.3,
     3 ''% of the grand mean.'')')  MD, XLOSS, 100.0D0 * (XLOSS / GM)
      WRITE (3, '(/, ''Were AMMI'', I1, '' rankings more to be'',
     1 '' trusted, the average GAIN from selecting'', /, ''AMMI'',
     2 I1, '' winners would be'', F15.5, '', or'', F8.3, ''% of the'',
     3 '' grand mean.'')')  MD, MD, XGAIN, 100.0D0 * (XGAIN / GM)
      WRITE (3, '(/, ''The largest AMMI'', I1, '' gain of'', F15.5,
     1 '', or'', F8.3, ''% of the grand mean,'', /, ''occurs in '',
     2 A1, A3, I6, 1X, A4, '' where AMMIF picks '', A1, A3, I6, 1X, A4,
     3 /, ''but AMMI'', I1, '' picks '', A1, A3, I6, 1X, A4,
     4 '' instead.'')')  MD, XDEM, 100.0D0 * (XDEM / GM), ENA, ENB,
     5 NDEM, C(KNE+NDEM), GNA, GNB, NGDT, C(KNG+NGDT), MD, GNA, GNB,
     6 NGAM, C(KNG+NGAM)
      RETURN
      END
      SUBROUTINE MEGAENV
C     Perform mega-environment analysis for AMMI1 model.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000),  SEP(100)
      INTEGER*4  L(8000000),  NGW(100)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Initially accept all genotypes.
      DO 100 NGD=1,NG
  100 L(KIG+NGD) = 1
C     Find genotype with highest mean.
      GHM = 0.0D0
      DO 101 NGD=1,NG
      R = V(KBG+NGD)
      IF (R .LE. GHM)  GO TO 101
      GHM = R
      NGH = NGD
  101 CONTINUE
C     Identify genotype left and right winners.
      NN = 0
      DO 102 NGD=1,NG
      R = V(KPGN+NGD)
      NN = NN + 1
      IF (NN .EQ. 1)  THEN
      GXL = R
      GXR = R
      NGXL = NGD
      NGXR = NGD
      ELSE
      IF (R .LT. GXL)  THEN
      GXL = R
      NGXL = NGD
      ENDIF
      IF (R .GT. GXR)  THEN
      GXR = R
      NGXR = NGD
      ENDIF
      ENDIF
  102 CONTINUE
C     Find all switches.
      IGW = 1
      NGW(IGW) = NGXR
      L(KIG+NGXR) = -1
      AR = V(KBG+NGXR)
      BR = V(KPGN+NGXR)
   20 NGSW = 0
      DO 110 NGD=1,NG
      IF (L(KIG+NGD) .NE. 1)  GO TO 110
      S = (AR - V(KBG+NGD)) / (V(KPGN+NGD) - BR)
      IF (IGW .GE. 2  .AND. S .GE. SR)  GO TO 110
      IF (NGSW .EQ. 0)  THEN
      SN = S
      NGSW = NGD
      ELSE
      IF (S.GT.SN .OR. (S.EQ.SN .AND. V(KPGN+NGD).LT.V(KPGN+NGSW))) THEN
      SN = S
      NGSW = NGD
      ENDIF
      ENDIF
  110 CONTINUE
      IGW = IGW + 1
      IF (IGW .EQ. 101)  GO TO 70
      NGW(IGW) = NGSW
      SEP(IGW) = SN
      L(KIG+NGSW) = -1
      IF (NGSW .EQ. NGXL)  GO TO 30
      AR = V(KBG+NGSW)
      BR = V(KPGN+NGSW)
      SR = SN
      GO TO 20
C     Rank environment IPCA1 scores; find max and min.
   30 CALL INDEX(NE, KPEN, KIE)
      NEMIN = L(KIE+NE)
      NEMAX = L(KIE+1)
      EMIN = V(KPEN+NEMIN)
      EMAX = V(KPEN+NEMAX)
C     Determine winner for each environment.
      KSCR = 2 * KSE
      NLD = 1
      NGWIN = NGW(NLD)
      SWITCH = SEP(NLD+1)
      DO 131 NED=1,NE
      NER = L(KIE+NED)
      EP1 = V(KPEN+NER)
      IF (EP1 .GE. SWITCH  .OR.  NLD .EQ. IGW)  GO TO 36
   35 NLD = NLD + 1
      NGWIN = NGW(NLD)
      SWITCH = SEP(NLD+1)
      IF (EP1 .LT. SWITCH  .AND.  NLD .LT. IGW)  GO TO 35
   36 L(KSCR+NER) = NGWIN
  131 CONTINUE
      NGMIN = L(KSE+NEMIN)
      NGMAX = L(KSE+NEMAX)
C     Write switch points.
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  Mega-environments'',
     1 '' for AMMI1 Model.'')')
      WRITE (3, '(/, A1, A3, '' switch points, including'',
     1 '' hypothetical winners.'')')  GNA, GNB
      WRITE (3, '(/, 73(''-''), /, A1, A3, '' Winner       '', A1, A3,
     1 ''IPCA1 Switch     Nominal Value         IPCA1 Score'', /,
     2 73(''-''))')  GNA, GNB, ENA, ENB
      DO 135 NLD=1,IGW
      NWN = NGW(NLD)
      IF     (NWN .EQ. NGMIN)  THEN
      YN = V(KBG+NGMIN) + (V(KPGN+NGMIN) * EMIN)
      WRITE (3, '(I5, 2X, A4, 24X, F15.5, '' at'', F20.7)')
     1 NGMIN, C(KNG+NGMIN), YN, EMIN
      ELSEIF (NWN .EQ. NGH)  THEN
      WRITE (3, '(I5, 2X, A4, 24X, F15.5, '' at'', 11X, ''0.0'')')
     1 NGH, C(KNG+NGH), GHM
      ELSEIF (NWN .EQ. NGMAX)  THEN
      YN = V(KBG+NGMAX) + (V(KPGN+NGMAX) * EMAX)
      WRITE (3, '(I5, 2X, A4, 24X, F15.5, '' at'', F20.7)')
     1 NGMAX, C(KNG+NGMAX), YN, EMAX
      ELSE
      WRITE (3, '(I5, 2X, A4)')  NWN, C(KNG+NWN)
      ENDIF
      IF (NLD .EQ. IGW)  GO TO 135
      R = SEP(NLD+1)
      IF (R .GE. EMIN  .AND.  R .LE. EMAX)  THEN
      YN = V(KBG+NWN) + (V(KPGN+NWN) * SEP(NLD+1))
      WRITE (3, '(11X, F20.7, F19.5)')  R, YN
      ELSE
      WRITE (3, '(11X, F20.7, ''      (Outside range.)'')')  R
      ENDIF
  135 CONTINUE
      WRITE (3, '(73(''-''))')
C     Write winner for each environment; count winners.
      WRITE (3, '(/, /, ''Winner for each environment.'')')
      WRITE (3, '(/, 59(''-''), /, A1, A3, ''   Name'', 9X,
     1 ''IPCA1 Score   AMMIF         Advantage'', /,
     2 59(''-''))')  ENA, ENB
      MEGA = 0
      NWNP = 0
      DO 140 NGD=1,NG
  140 L(KIG+NGD) = 0
      DO 150 NED=1,NE
      NERK = L(KIE+NED)
      NWN = L(KSCR+NERK)
      IF (NWNP .NE. NWN)  THEN
      MEGA = MEGA + 1
      IF (MEGA .GT. 1)  WRITE (3, '('' '')')
      WRITE (3, '(''Mega-environment'', I4, '':  '', A1, A3, I6,
     1 2X, A4, ''  wins AMMI1 model.'', /)')  MEGA, GNA, GNB, NWN,
     2 C(KNG+NWN)
      ENDIF
      NWNP = NWN
      NAF = NWN
      R = V(KDGE+((NERK-1)*NG)+NAF)
      DO 151 NGD=1,NG
      X = V(KDGE+((NERK-1)*NG)+NGD)
      IF (X .LE. R)  GO TO 151
      NAF = NGD
      R = X
  151 CONTINUE
      L(KIG+NAF) = 1
      IF (NWN .EQ. NGH)  THEN
      ADV = 0.0D0
      ELSE
      A1NWN = V(KBG+NWN) + V(KBE+NERK)-GM + (V(KPGN+NWN)*V(KPEN+NERK))
      A1NGH = V(KBG+NGH) + V(KBE+NERK)-GM + (V(KPGN+NGH)*V(KPEN+NERK))
      ADV = A1NWN - A1NGH
      ENDIF
  150 WRITE (3, '(I5, 2X, A4, F20.7, 4X, A4, F20.5)')  NERK,
     1 C(KNE+NERK), V(KPEN+NERK), C(KNG+NAF), ADV
      WRITE (3, '(59(''-''))')
      WRITE (3, '(/, ''The advantage from subdivision into'',
     1 '' mega-environments equals'', /, ''the AMMI1 estimate'',
     2 '' for the mega-environment winner minus'', /,
     3 ''the AMMI1 estimate for the main effect winner, '',
     4 A1, A3, I6, '' ('', A4, '').'')')  GNA, GNB, NGH, C(KNG+NGH)  
      NWAF = 0
      DO 160 NGD=1,NG
  160 NWAF = NWAF + L(KIG+NGD)
      IF (MEGA .LT. NWAF)  THEN
      WRITE (3, '(/, ''AMMI1 has'', I4, '' winners, whereas'', /,
     1 ''AMMIF has'', I4, '' winners.'', /, ''Noise makes things'',
     2 '' seem more complicated than they really are.'')')
     3 MEGA, NWAF
      ENDIF
      WRITE (3, '(/, ''Ordinarily, any small mega-environment,'',
     1 '' with few members'', /, ''or little advantage over other'',
     2 '' near winners, is ignored'', /, ''and its members are'',
     3 '' reassigned to a nearby larger mega-environment.'')')
      RETURN
   70 WRITE (3, '(/, ''Over 100 winners render this analysis'',
     1 '' pointless.'')')
      RETURN
      END
      SUBROUTINE OUT3
C     Write further commentary on table and analyze residuals.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000),   LC(40)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA,   HP, H(50)
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Write commentary regarding LSD05 and order statistics.
      IF (KPAM .EQ. -1)  RETURN
      IF (NFE .EQ. 0)  GO TO 10
      WRITE (3, '(/, /, ''The following values may be useful for'',
     1 '' assessing the significance of'', /, ''mean separations'',
     2 '' in the above listing of data estimates.'')')
      WRITE (3, '(/, ''Unweighted grand mean without imputed data'',
     1 F25.5)')  GMU
      WRITE (3, '(''With'', I6, '' df the root error mean square'',
     1 F27.5)')  NFE, REMS
      SEM = REMS / DSQRT(DBLE(NX))
      WRITE (3, '(''With'', I6, 1X, A1, A3, '' the standard error of '',
     1 A4, '' means'', F20.5)')  NX, RNA, RNB, TNM, SEM
      SED = SEM * 1.414213562D0
      CV = 100.0D0 * (SEM / GMU)
      WRITE (3, '(''Coefficient of variation for '', A4, '' means'',
     1 F28.5, ''% of GM'')')  TNM, CV
      WRITE (3, '(/, ''Standard error of difference between two '',
     1 A4, '' means'', F16.5)')  TNM, SED
      T05 = TTEST(NFE)
      WRITE (3, '(''With'', I6, '' df t05 of'', F9.5, '' giving LSD05'',
     1 F25.5)')  NFE, T05, T05 * SED
   10 WRITE (3, '(/, /, ''The average value of the maximum of N'',
     1 '' normal realizations is the'', /, ''first order statistic.'',
     2 ''  This value times the standard error of the '', A4, /,
     3 ''means provides a typical indication of the largest upward'',
     4 '' bias present'', /, ''in the data, which is most likely'',
     5 '' to affect the highest means.'')')  TNM
      WRITE (3, '(/, 56(''-''), /, ''Scope'', 17X, ''Count   First'',
     1 '' Order   Typical Bias'', /, 56(''-''))')
      R = XORDER(NG)
      X = R * SEM
      WRITE (3, '(A1, A3, '' within one '', A1, A3, I7, F12.5, F17.5)')
     1 GNA, GNB, ENA, ENB, NG, R, X
      R = XORDER(NE)
      X = R * SEM
      WRITE (3, '(A1, A3, '' within one '', A1, A3, I7, F12.5, F17.5)')
     1 ENA, ENB, GNA, GNB, NE, R, X
      R = XORDER(NGE)
      X = R * SEM
      WRITE (3, '(''All '', A4, 12X, I7, F12.5, F17.5)')  TNM, NGE, R, X
      WRITE (3, '(56(''-''))')
C     Write histogram of residuals.
      RSMX = 0.0D0
      KS = 0
      DO 100 NED=1,NE
      DO 101 NGD=1,NG
      KS = KS + 1
      R = DABS(V(KDGE+KS) - V(KS))
      V(KS) = R
      IF (R .LE. RSMX)  GO TO 101
      RSMX = R
      NEMX = NED
      NGMX = NGD
  101 CONTINUE
  100 CONTINUE
      IF ((RSMX / GM) .LT. 1.0D-10)  RETURN
      RMRA = NAGE * NX
      RMRA = DSQRT(SSL / RMRA)
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''Histogram of Residuals.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, ''The largest residual in absolute value is'',
     1 F15.5, /, ''for '', A1, A3, I6, 1X, A4, '' and '', A1, A3, I6,
     2 1X, A4, '' for model AMMI'', I1, ''.'')')  RSMX, GNA, GNB,
     3 NGMX, C(KNG+NGMX), ENA, ENB, NEMX, C(KNE+NEMX), MD
      WRITE (3, '(/, ''The'', I7, '' actual '', A4,
     1 '' residuals, ignoring'', I6, '' missing cells,'', /,
     2 ''have a root mean square of'', F15.5, ''.'')')
     3 NAGE, TNM, NMCL, RMRA
      WRITE (3, '(/, ''A histogram for the AMMI'', I1,
     1 '' residuals follows.'')')  MD
      WRITE (3, '(/, /, 79(''-''), /, ''Interval  Maximum   Count'',
     2 ''    Histogram'', /, 79(''-''))')
      LN = 20
      IF (NAGE .GE. 100)  LN = 30
      IF (NAGE .GE. 200)  LN = 40
      F = DBLE(LN) / RSMX
      DO 102 LD=1,LN
  102 LC(LD) = 0
      DO 103 IGE=1,NGE
      IF (M(KKGE+IGE) .EQ. 0)  GO TO 103
      LS = F * V(IGE)
      LS = LS + 1
      IF (LS .GT. LN)  LS = LN
      LC(LS) = LC(LS) + 1
  103 CONTINUE
      LMX = 0
      DO 104 LD=1,LN
      LCD = LC(LD)
  104 IF (LCD .GT. LMX)  LMX = LCD
      F = RSMX / DBLE(LN)
      FL = 50.0D0 / DBLE(LMX)
      DO 105 LD=1,LN
      R = LD
      R = R * F
      LCD = (DBLE(LC(LD)) * FL) + 0.9999999999D0
      LCD = LCD + 1
      HP = '*'
      DO 106 ND=1,50
      IF (ND .EQ. LCD)  HP = ' '
  106 H(ND) = HP
  105 WRITE (3, '(I2, F16.5, I7, ''  |'', 50A1, ''|'')')
     1 LD, R, LC(LD), H
      WRITE (3, '(79(''-''))')
C     Issue terminal progress message.
      WRITE (*, '(/, '' Completed AMMI analysis for run'', I5,
     1 '' of'', I5, ''.'')')  IRUN, MRUN
      RETURN
      END
      SUBROUTINE PCA
C     Compute and write PCA model.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM, SI
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /SGNF/  SMS, PF, SI
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Subtract grand mean from the data.
      DO 100 IGE=1,NGE
  100 V(IGE) = V(KDGE+IGE) - GM
C     Initialize PCA GEN scores with arbitrary vectors.
      X = (FG - 1.0D0) / 2.0D0
      DO 101 NGD=1,NG
      DO 102 MA=1,NXP
      KS = KPGN + ((MA - 1) * NG) + NGD
  102 V(KS) = X
  101 X = X - 1.0D0
C     Calculate PCA for axes 1 to NXP, using the power method.
      MD = NXP
      CALL IPCA(-1, 0, 0, 0)
C     Write PCA scores.
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' PCA Scores.'')')  GNA, GNB
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, ''The grand mean'', F16.5, '' is subtracted'',
     1 '' from the data before applying PCA.'')')  GM
      IF (KBAL .EQ. -1)  WRITE (3, '(/, ''Imputed data from AMMI'',
     1 I1, '' have been used to fill'', I6, '' otherwise missing'',
     2 '' cells.'')')  MFIP, NMCL
      WRITE (3, '(/, 79(''-''), /, A1, A3, ''   Name       PCA 1'',
     1 12X, ''PCA 2'', 12X, ''PCA 3'', 12X, ''PCA 4'', /, 79(''-''))')
     2 GNA, GNB
      DO 110 NGD=1,NG
  110 WRITE (3, '(I5, 2X, A4, 4F17.7)')  NGD, C(KNG+NGD),
     1 (V(KPGN+((ND-1)*NG)+NGD), ND=1,NXP)
      WRITE (3, '(79(''-''))')
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' PCA Scores.'')')  ENA, ENB
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, ''The grand mean'', F15.5, '' is subtracted'',
     1 '' from the data before applying PCA.'')')  GM
      IF (KBAL .EQ. -1)  WRITE (3, '(/, ''Imputed data from AMMI'',
     1 I1, '' have been used to fill'', I6, '' otherwise missing'',
     2 '' cells.'')')  MFIP, NMCL
      WRITE (3, '(/, 79(''-''), /, A1, A3, ''   Name       PCA 1'',
     1 12X, ''PCA 2'', 12X, ''PCA 3'', 12X, ''PCA 4'', /, 79(''-''))')
     2 ENA, ENB
      DO 111 NED=1,NE
  111 WRITE (3, '(I5, 2X, A4, 4F17.7)')  NED, C(KNE+NED),
     1 (V(KPEN+((ND-1)*NE)+NED), ND=1,NXP)
      WRITE (3, '(79(''-''))')
C     Write ANOVA table for model PCA.
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''ANOVA Table for Model PCA.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 79(''-''), /, ''Source'', 12X, ''df'', 15X,
     1 ''SS'', 18X, ''MS'', 10X, ''Probability'', /, 79(''-''))')
      WRITE (3, '(''Total         '', I7, F22.5, F20.5)')  NFT, SST,
     1 SST / DBLE(NFT)
      CALL FTEST(NFR, SSR, NFE, SSE)
      WRITE (3, '(2X, A4, 8X, I7, F22.5, F20.5, F12.7, A4)')
     1 TNM, NFR, SSR, SMS, PF, SI
      NF = NG + NE - 1
      NFL = NFR
      SSL = SSR
      DO 112 ND=1,NXP
      SSD = SSP(ND)
      CALL FTEST(NF, SSD, NFE, SSE)
      WRITE (3, '(''    PCA'', I2, I12, F22.5, F20.5, F12.7, A4)')
     1 ND, NF, SSD, SMS, PF, SI
      NFL = NFL - NF
      NF = NF - 2
  112 SSL = SSL - SSD
      IF (NFL .LE. 0)  GO TO 10
      CALL FTEST(NFL, SSL, NFE, SSE)
      WRITE (3, '(''    Residual'', I9, F22.5, F20.5, F12.7, A4)')
     1 NFL, SSL, SMS, PF, SI
   10 IF (NFE .GT. 0)  WRITE (3, '(''  Error     '', I9, F22.5,
     1 F20.5)')  NFE, SSE, EMS
      WRITE (3, '(79(''-''))')
      WRITE (3, '(/, ''Grand mean'', F15.5, 2X, A20)')  GM, NMUN
      IF (KBAL .GE. 0)  THEN
      WRITE (3, '(/, ''There are no missing cells.'')')
      ELSE
      WRITE (3, '(/, ''Imputed data from AMMI'', I1, '' have been'',
     1 '' used to fill'', I6, '' otherwise missing cells.'')')
     2 MFIP, NMCL
      ENDIF
C     Write expected values for PCA models.
      IF (KPPC .EQ. -1)  GO TO 20
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''Expected Values for PCA Models.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 70(''-''), /, A1, A3, 2X, A1, A3, 6X, ''PCA 1'',
     1 ''          PCA 2          PCA 3          PCA 4'', /,
     2 70(''-''))')  ENA, ENB, GNA, GNB
      DO 120 NED=1,NE
      DO 121 NGD=1,NG
      R = GM
      DO 122 NXD=1,NXP
      NS = NXD - 1
      R = R + (V(KPGN+NS*NG+NGD) * V(KPEN+NS*NE+NED))
  122 V(NXD) = R
  121 WRITE (3, '(2I5, 4F15.5)')  NED, NGD, (V(NXD), NXD=1,NXP)
  120 CONTINUE
      WRITE (3, '(70(''-''))')
C     Issue terminal progress message.
   20 WRITE (*, '(/, '' Completed PCA analysis.'')')
      RETURN
      END
      SUBROUTINE LNR
C     Compute and write linear regression model.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM, SI
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /SGNF/  SMS, PF, SI
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Compute GEN regressions and correlations.
      FX = NX
      BBG = 0.0D0
      RK = 0.0D0
      XXE = SSENV / (FX * FG)
      DO 100 NGD=1,NG
      R = V(KBG+NGD)
      XY = 0.0D0
      RYY = 0.0D0
      DO 101 NED=1,NE
      NS = KDGE + ((NED - 1) * NG) + NGD
      X = V(KBE+NED) - GM
      RY = V(NS) - R - X
      XY = XY + (X * RY)
  101 RYY = RYY + (RY * RY)
      B = XY / XXE
      V(KSG+NGD) = B
      BBG = BBG + (B * B)
      RK = RK + (B * (R - GM))
      R = 0.0D0
      IF (DABS(B) .GE. 1.0D-10)  R = B * (XY / RYY)
  100 Y(KIG+NGD) = R
C     Compute ENV regressions and correlations.
      BBE = 0.0D0
      XXG = SSGEN / (FX * FE)
      RK = RK / XXG
      DO 102 NED=1,NE
      R = V(KBE+NED)
      XY = 0.0D0
      RYY = 0.0D0
      NS = KDGE + ((NED - 1) * NG)
      DO 103 NGD=1,NG
      X = V(KBG+NGD) - GM
      RY = V(NS+NGD) - R - X
      XY = XY + (X * RY)
  103 RYY = RYY + (RY * RY)
      B = XY / XXG
      V(KSE+NED) = B
      BBE = BBE + (B * B)
      R = 0.0D0
      IF (DABS(B) .GE. 1.0D-10)  R = B * (XY / RYY)
  102 Y(KIE+NED) = R
C     Compute df and SS values, and rank slopes.
      SSJR = FX * RK * XXG * RK * XXE
      SSGR = DABS((FX * BBG * XXE) - SSJR)
      NFGR = NG - 2
      SSER = DABS((FX * BBE * XXG) - SSJR)
      NFER = NE - 2
      SSRL = SSGE - SSJR - SSGR - SSER
      NFRL = NFGE - 1 - NFGR - NFER
      CALL INDEX(NG, KSG, KCG)
      CALL INDEX(NE, KSE, KCE)
C     Write means and slopes.
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' Linear Regressions.'')')  GNA, GNB
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 71(''-''), /, A1, A3, ''   Name       Mean'',
     1 ''       Slope   R-squared  | Index  Name   Slope'', /,
     2 71(''-''))')  GNA, GNB
      DO 110 NGD=1,NG
      NS = L(KCG+NGD)
  110 WRITE (3, '(I5, 2X, A4, F15.5, 2F10.5, ''  |'', I6, 2X, A4,
     1 F10.5)')  NGD, C(KNG+NGD), V(KBG+NGD), V(KSG+NGD), Y(KIG+NGD),
     2 NS, C(KNG+NS), V(KSG+NS)
      WRITE (3, '(71(''-''))')
      WRITE (3, '(/, ''Grand Mean'', F16.5)')  GM
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 A1, A3, '' Linear Regressions.'')')  ENA, ENB
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 71(''-''), /, A1, A3, ''   Name       Mean'',
     1 ''       Slope   R-squared  | Index  Name   Slope'', /,
     2 71(''-''))')  ENA, ENB
      DO 111 NED=1,NE
      NS = L(KCE+NED)
  111 WRITE (3, '(I5, 2X, A4, F15.5, 2F10.5, ''  |'', I6, 2X, A4,
     1 F10.5)')  NED, C(KNE+NED), V(KBE+NED), V(KSE+NED), Y(KIE+NED),
     2 NS, C(KNE+NS), V(KSE+NS)
      WRITE (3, '(71(''-''))')
      WRITE (3, '(/, ''Grand Mean'', F16.5)')  GM
C     Write ANOVA table for linear regressions.
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''ANOVA Table for Linear Regressions.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 79(''-''), /, ''Source'', 12X, ''df'', 15X,
     1 ''SS'', 18X, ''MS'', 10X, ''Probability'', /, 79(''-''))')
      WRITE (3, '(''Total         '', I7, F22.5, F20.5)')  NFT, SST,
     1 SST / DBLE(NFT)
      CALL FTEST(NFR, SSR, NFE, SSE)
      WRITE (3, '(2X, A4, 8X, I7, F22.5, F20.5, F12.7, A4)')
     1 TNM, NFR, SSR, SMS, PF, SI
      CALL FTEST(NFGEN, SSGEN, NFE, SSE)
      WRITE (3, '(4X, A1, A3, 6X, I7, F22.5, F20.5, F12.7, A4)')
     1 GNA, GNB, NFGEN, SSGEN, SMS, PF, SI
      CALL FTEST(NFENV, SSENV, NFE, SSE)
      WRITE (3, '(4X, A1, A3, 6X, I7, F22.5, F20.5, F12.7, A4)')
     1 ENA, ENB, NFENV, SSENV, SMS, PF, SI
      CALL FTEST(NFGE, SSGE, NFE, SSE)
      WRITE (3, '(4X, A1, '' x '', A1, 5X, I7, F22.5, F20.5, F12.7,
     1 A4)')  GNA, ENA, NFGE, SSGE, SMS, PF, SI
      CALL FTEST(1, SSJR, NFE, SSE)
      WRITE (3, '(''      Joint Regr    1'', F22.5, F20.5, F12.7, A4)')
     1 SSJR, SMS, PF, SI
      CALL FTEST(NFGR, SSGR, NFE, SSE)
      WRITE (3, '(6X, A1, A3, '' Regrs'', I5, F22.5, F20.5, F12.7, A4)')
     1 GNA, GNB, NFGR, SSGR, SMS, PF, SI
      CALL FTEST(NFER, SSER, NFE, SSE)
      WRITE (3, '(6X, A1, A3, '' Regrs'', I5, F22.5, F20.5, F12.7, A4)')
     1 ENA, ENB, NFER, SSER, SMS, PF, SI
      CALL FTEST(NFRL, SSRL, NFE, SSE)
      WRITE (3, '(6X, ''Residual'', I7, F22.5, F20.5, F12.7, A4)')
     1 NFRL, SSRL, SMS, PF, SI
      IF (NFE .GT. 0)  WRITE (3, '(''  Error       '', I7, F22.5,
     1 F20.5)')  NFE, SSE, EMS
      WRITE (3, '(79(''-''))')
      WRITE (3, '(/, ''Grand mean'', F15.5, 2X, A20)')     GM, NMUN
      WRITE (3, '(/, ''Joint regression K'', D15.7)')  RK
      IF (KBAL .GE. 0)  THEN
      WRITE (3, '(/, ''There are no missing cells.'')')
      ELSE
      WRITE (3, '(/, ''There are'', I7, '' missing cells supplied'',
     1 '' imputed values.'')')  NMCL
      ENDIF
C     Write expected values for regression models.
      IF (KPLR .EQ. -1)  GO TO 10
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  '',
     1 ''Expected Values for Regression Models.'')')
      WRITE (3, '(/, A72)')  NMRN
      WRITE (3, '(/, 75(''-''), /, A1, A3, 2X, A1, A3, 4X,
     1 ''Additive    Joint Regr   '', A1, A3, '' Regrs   '', A1, A3,
     2 '' Regrs   Both Regrs'', /, 75(''-''))')  ENA, ENB, GNA, GNB,
     3 GNA, GNB, ENA, ENB
      DO 120 NED=1,NE
      AE = V(KBE+NED) - GM
      BE = V(KSE+NED)
      DO 121 NGD=1,NG
      AG = V(KBG+NGD) - GM
      BG = V(KSG+NGD)
      VA = GM + AG + AE
      VJ = VA + (RK * AG * AE)
      VG = VA + (BG * AE)
      VE = VA + (BE * AG)
      VB = VA + (BG * AE) + (BE * AG) - (RK * AG * AE)
  121 WRITE (3, '(2I5, 5F13.5)')  NED, NGD, VA, VJ, VG, VE, VB
  120 CONTINUE
      WRITE (3, '(75(''-''))')
C     Issue terminal progress message.
   10 WRITE (*, '(/, '' Completed linear regression analysis.'')')
      RETURN
      END
      SUBROUTINE STATS(MX)
C     Compute prediction statistics.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /PRED/  RPD(8,3), RPDT, KRPD(8), NRPD(2)
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
      KS = 0
      KR = 0
C     Compute prediction statistics given no missing cells.
      IF (KBAL .EQ. -1)  GO TO 10
      DO 100 NED=1,NE
      DO 101 NGD=1,NG
      KS = KS + 1
      DO 102 NRD=1,NR
      KR = KR + 1
      IF (M(KKRGE+KR) .GE. 0)  GO TO 102
      RY = Y(KYRGE+KR)
      MT = M(KKGE+KS)
      NS = 1
      IF (MT .LT. 1)  NS = 2
      VHT = V(KBE+NED) + V(KBG+NGD) - GM
      R = RY - VHT
      RPD(1,NS) = RPD(1,NS) + (R * R)
      DO 103 MDD=1,MX
      MP = MDD + 1
      KG = KPGN + ((MDD - 1) * NG)
      KE = KPEN + ((MDD - 1) * NE)
      VHT = VHT + (V(KG+NGD) * V(KE+NED))
      R = RY - VHT
  103 RPD(MP,NS) = RPD(MP,NS) + (R * R)
      NRPD(NS) = NRPD(NS) + 1
      IF (NS .EQ. 2)  GO TO 102
      R = RY - V(KDGE+KS)
      RPDT = RPDT + (R * R)
  102 CONTINUE
  101 CONTINUE
  100 CONTINUE
      DO 104 MDD=1,MX+1
  104 KRPD(MDD) = 1
      GO TO 20
C     Compute prediction statistics given missing cells.
   10 MP = MD + 1
      DO 110 NED=1,NE
      DO 111 NGD=1,NG
      KS = KS + 1
      DO 112 NRD=1,NR
      KR = KR + 1
      IF (M(KKRGE+KR) .GE. 0)  GO TO 112
      RY = Y(KYRGE+KR)
      MT = M(KKGE+KS)
      NS = 1
      IF (MT .LT. 1)  NS = 2
      VHT = V(KBE+NED) + V(KBG+NGD) - GM
      IF (MD .EQ. 0)  GO TO 11
      DO 113 MDD=1,MD
      KG = KPGN + ((MDD - 1) * NG)
      KE = KPEN + ((MDD - 1) * NE)
  113 VHT = VHT + (V(KG+NGD) * V(KE+NED))
   11 R = RY - VHT
      RPD(MP,NS) = RPD(MP,NS) + (R * R)
      IF (MD .LT. MX)  GO TO 112
      NRPD(NS) = NRPD(NS) + 1
      IF (NS .EQ. 2)  GO TO 112
      R = RY - V(KDGE+KS)
      RPDT = RPDT + (R * R)
  112 CONTINUE
  111 CONTINUE
  110 CONTINUE
      KRPD(MP) = 1
      IF (MD .LT. MX)  RETURN
   20 IF (IRUN .LT. MRUN)  RETURN
      FP = NRPD(1)
      FI = NRPD(2)
      R = FP + FI
      DO 120 MDD=1,MX+1
      RPD(MDD,3) = DSQRT((RPD(MDD,1) + RPD(MDD,2)) / R)
      IF (FP .GT. 0.0D0)  RPD(MDD,1) = DSQRT(RPD(MDD,1) / FP)
  120 IF (FI .GT. 0.0D0)  RPD(MDD,2) = DSQRT(RPD(MDD,2) / FI)
      IF (FP .GT. 0.0D0)  RPDT = DSQRT(RPDT / FP)
      RETURN
      END
      SUBROUTINE PREDS(MX)
C     Write prediction statistics for KVAL=1 data splitting.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA,   ST
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM,   EXRD
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      COMMON  /PRED/  RPD(8,3), RPDT, KRPD(8), NRPD(2)
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Compute error df and SS based upon all available data.
      NFE = 0
      IF (NR .EQ. 1)  GO TO 20
      SSE = 0.0D0
      NS = KKRGE
      ND = KYRGE
      NA = 0
      GMU = 0.0D0
      DO 100 IGE=1,NGE
      NC = 0
      VB = 0.0D0
      DO 101 NRD=1,NR
      IF (M(NS+NRD) .EQ. 0)  GO TO 101
      NC = NC + 1
      R = Y(ND+NRD)
      VB = VB + R
  101 CONTINUE
      IF (NC .EQ. 0)  GO TO 10
      R = NC
      VB = VB / R
      NA = NA + 1
      GMU = GMU + VB
      DO 102 NRD=1,NR
      IF (M(NS+NRD) .EQ. 0)  GO TO 102
      R = Y(ND+NRD)
      R = R - VB
      SSE = SSE + (R * R)
  102 CONTINUE
      NFE = NFE + NC - 1
   10 NS = NS + NR
  100 ND = ND + NR
      R = NFE
      EMS = 0.0D0
      IF (NFE .GE. 1)  EMS = SSE / R
      REMS = DSQRT(EMS)
      R = NA
      GMU = GMU / R
C     Write prediction statistics summary.
   20 WRITE (3, '(/, /,  ''MATMODEL Version 3.0:'',
     1 ''  Prediction Statistics.'')')
      WRITE (3, '(/, A72)')  NMRN
      NP = NRPD(1)
      MX1 = MX + 1
      WRITE (3, '(/, ''The root mean square prediction differences'',
     1 '' in units of '', A20, /, ''are based on'', I6,
     2 '' runs having'', I10, '' validations with actual '', A4,
     3 '' data,'', /, I10, '' validations with imputed data, and'',
     4 I10, '' validations total.'')')  NMUN, MRUN, NP, TNM, NRPD(2),
     5 NP+NRPD(2)
      WRITE (3, '(/, 61(''-''), /, ''Model        Actual'', 15X,
     1 ''Imputed           Total'', /, 61(''-''))')
      NS = 0
      IF (NP .EQ. 0)  GO TO 21
      DO 110 MDD=1,MX1
      IF (KRPD(MDD) .EQ. 0)  GO TO 110
      R = RPD(MDD,1)
      IF (NS .EQ. 0)  THEN
      NS = MDD
      XL = R
      ELSE
      IF (R .GE. XL)  GO TO 110
      XL = R
      NS = MDD
      ENDIF
  110 CONTINUE
      IF (RPDT .LT. XL)  NS = -1
   21 NML = NS - 1
      DO 111 MDD=1,MX1
      IF (KRPD(MDD) .EQ. 0)  GO TO 111
      ML = MDD - 1
      ST = ' '
      IF (NS .EQ. MDD)  ST = '*'
      WRITE (3, '(''AMMI'', I1, F17.5, 1X, A1, F20.5, F17.5)')
     1 ML, RPD(MDD,1), ST, RPD(MDD,2), RPD(MDD,3)
  111 CONTINUE
      IF (NP .EQ. 0)  GO TO 22
      ST = ' '
      IF (NS .EQ. -1)  ST = '*'
      WRITE (3, '(''AMMIF'', F17.5, 1X, A1)')  RPDT, ST
   22 WRITE (3, '(61(''-''))')
      WRITE (3, '(/, ''The unweighted grand mean with all actual'',
     1 '' data is'', F19.5, ''.'')')  GMU
      IF (NFE .EQ. 0)  RETURN
      WRITE (3, '(/, ''For comparison, with'', I7, '' df the root'',
     1 '' EMS is'', F22.5, ''.'')')  NFE, REMS
      X = NX
      RDT = DSQRT(EMS + (EMS / X))
      X = REMS / DSQRT(X)
      WRITE (3, '(''For'', I6, '' replications, the standard error'',
     1 '' is'', F23.5, ''.'')')  NX, X
      IF (NP .EQ. 0)  GO TO 24
      R = 100.0D0 * ((RDT - RPDT) / RPDT)
      EXRD = 'more'
      IF (R .LT. 0.0D0)  THEN
      R = -R
      EXRD = 'less'
      ENDIF
      WRITE (3, '(/, ''Consequently the expected RMSPD for AMMIF is'',
     1 F24.5, '','', /, ''which is'', F7.2, ''% '', A4,
     2 '' than the empirical value'', F22.5, ''.'')')
     3 RDT, R, EXRD, RPDT
      IF (NS .NE. -1)  GO TO 23
      WRITE (3, '(/, ''None of these AMMI models is superior to'',
     1 '' AMMIF.'')')
      RETURN
   23 WRITE (3, '(/, ''Model AMMI'', I1, '' has the smallest actual'',
     1 '' RMSPD of'', F24.5, '','')')  NML, XL
      WRITE (3, '(''based upon'', I12, '' validations with the'', I6,
     1 1X, A4, '' having actual data.'')')  NP, NAGE, TNM
      IF (XL .LE. REMS)  GO TO 24
      R = DSQRT((XL * XL) - EMS)
      X = REMS / R
      X = X * X
      AGN = NX
      AGN = X / AGN
      PCM = 100.0D0 * (R / GMU)
      WRITE (3, '(''From statistical reasoning, the root mean square'',
     1 '' prediction error'', /, ''between AMMI'', I1,
     2 '' predictions and the true '', A4, '' means is'', F16.5,
     3 '','', /, ''which is'', F7.2, ''% of the unweighted grand'',
     4 '' mean.'')')  NML, TNM, R, PCM
      WRITE (3, '(/, ''The same standard error requires '', A4,
     1 '' means based upon'', F7.2, '' replications.'', /,
     2 ''Accordingly the AMMI gain factor is'', F7.2, ''.'')')
     3 TNM, X, AGN
      R = NX
      R = X - R
      WRITE (3, '(''Given'', I5, '' replications this amounts to'',
     1 F7.2, '' free replications.'')')  NX, R
      NF = FGE * R
      WRITE (3, '(''Given'', I5, 1X, A4, '' this amounts to'', I7,
     1 '' free observations.'')')  NGE, TNM, NF
   24 IF (NRPD(2) .EQ. 0  .OR.  NS .EQ. 0)  GO TO 25
      R = RPD(NS,2)
      IF (R .LE. REMS)  GO TO 25
      R = DSQRT ((R * R) - EMS)
      X = REMS / R
      X = X * X
      WRITE (3, '(''AMMI'', I1, '' with imputed data has RMSPD'',
     1 F15.5, ''.'')')  NS, R
      WRITE (3, '(''This is equivalent to '', A4,
     1 '' means based upon'', F7.2, '' replications.'')')  TNM, X
   25 CONTINUE
      RETURN
      END
      SUBROUTINE INDIR
C     Analyze indirect and direct information.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      INTEGER*2  M(16000000)
      CHARACTER*1   GNA, ENA, RNA
      CHARACTER*3   GNB, ENB, RNB, EXDS
      CHARACTER*4   C(8000000), NMDB, NMVB, TNM,   BETR
      CHARACTER*20  NMUN
      CHARACTER*60  FNDAT, FNVAL, FNOUT, FNMCR, FZ
      CHARACTER*72  NMDA, NMVA, NMRN
      COMMON  /DATA/  Y(8000000)
      COMMON  /PRMI/  NG, NE, NR, NGE, NRG, NRGE, KDGE, KBG, KBE, KSG,
     1 KSE, KPGN, KPEN, KYRGE, KCG, KCE, KIG, KIE, KNG, KNE, KKGE, KQ,
     2 KKRGE, KTL, MXIPCA, MXCYCL, MXITER, MNIP, MXIP, MFIP, LNGTH4,
     3 MXRUN, MRUN, IRUN, KVAL, NVAL, KBAL, NMCL, MODE, KRTL, NAGE,
     4 KPC, KLR, KPPC, NXP, KPLR, KPAM, KPII, KII, MIIP, KNUL, KLV, MD
      COMMON  /PRMD/  FG, FE, FR, FGE, FRGE, RSHFT, DEL, YMV
      COMMON  /PRMC/ FZ, FNDAT, FNVAL, FNOUT, FNMCR, NMDA, NMVA, NMDB,
     1 NMVB, NMRN, NMUN, TNM, GNA, GNB, ENA, ENB, RNA, RNB, EXDS
      COMMON  /SSQR/  SST, SSE, SSR, SSGEN, SSENV, SSGE, SSP(7), SSL,
     1 EMS, REMS, EMS0, REMS0, GM, GMU, GMU0, NFT, NFE, NFE0, NFR,
     2 NFGEN, NFENV, NFGE, NX
      EQUIVALENCE  (V(1), Y(1), L(1), C(1), M(1))
C     Write headers.
      WRITE (3, '(/, /, ''MATMODEL Version 3.0:  Indirect'',
     1 '' Information.'')')
      WRITE (3, '(/, A72)')  NMRN
      IF (KPII .EQ. 1)  WRITE (3, '(/, /, 79(''-''), /, ''       N  '',
     1 A1, A3, 2X, A1, A3, '' Count      Impute             AMMI'', I1,
     2 13X, ''DATA'', /, 79(''-''), /)')  ENA, ENB, GNA, GNB, MIIP
C     Fit an initial AMMI model to all modelling data.
C     Save the results in a temporary file for recall later.
      IRUN = 0
      MRUN = NGE
      WRITE (*, '('' Indirect information analysis fits an initial'',
     1 '' AMMI model to all modelling data.'')')
      CALL AMMI(MIIP, MIIP)
      OPEN (4, STATUS='SCRATCH')
      DO 100 NED=1,NE
      R = V(KBE+NED) - GM
      DO 101 NGD=1,NG
      VIA = R + V(KBG+NGD)
      IF (MIIP .EQ. 0)  GO TO 101
      DO 102 MDD=1,MIIP
      KG = KPGN + ((MDD - 1) * NG)
      KE = KPEN + ((MDD - 1) * NE)
  102 VIA = VIA + (V(KG+NGD) * V(KE+NED))
  101 WRITE (4, '(D25.15)')  VIA
  100 CONTINUE
      REWIND 4
C     Analyze indirect information.
C     Set TRT status to the special value of -2 for withheld data.
      NSIW = 0
      NSIV = 0
      NSDV = 0
      NSAV = 0
      RSIW = 0.0D0
      RSIV = 0.0D0
      RSDV = 0.0D0
      RSAV = 0.0D0
      KS = -NR
      NED = 1
      NGD = 0
      DO 103 ND=1,NGE
      IRUN = ND
      KS = KS + NR
      NGD = NGD + 1
      IF (NGD .LE. NG)  GO TO 10
      NED = NED + 1
      NGD = 1
   10 READ (4, '(D25.15)')  VIA
      MKTR = M(KKGE+ND)
      IF (MKTR .GT. 0)  GO TO 11
      VII = VIA
      DO 104 NA=1,NR
      NB = KS + NA
      MT = M(KKRGE+NB)
      IF (MT .EQ. 0)  GO TO 104
      R = Y(KYRGE+NB)
      R = R - VII
      R = R * R
      NSIV = NSIV + 1
      RSIV = RSIV + R
      NSAV = NSAV + 1
      RSAV = RSAV + R
  104 CONTINUE
      IF (KPII .EQ. 1)  WRITE (3, '(I8, 2I6, I5, 2F18.7)')
     1 ND, NED, NGD, MKTR, VII, VIA
      IF (KLV .EQ. 2)  WRITE (*, '('' Run'', I5, '' of'', I5,
     1 '', no AMMI calculations needed.'')')  IRUN, MRUN
      IF (KLV .EQ. 3)  WRITE (*, '('' Run'', I5, '' of'', I5,
     1 '', model'', I5, '' of'', I5, '' to'', I5, '','', /,
     2 ''     no AMMI calculations needed.'')')
     3 IRUN, MRUN, MD, MD, MD
      GO TO 103
   11 M(KKGE+ND) = 0
      DO 105 NA=1,NR
      NB = KKRGE + KS + NA
  105 IF (M(NB) .EQ. 1)  M(NB) = -2
      VID = V(KDGE+ND)
      CALL ANOVA(IRUN)
      CALL AMMI(MIIP, MIIP)
      VII = V(KDGE+ND)
      DO 106 NA=1,NR
      NB = KS + NA
      MT = M(KKRGE+NB)
      IF (MT .EQ. 0)  GO TO 106
      VY = Y(KYRGE+NB)
      RI = VY - VII
      RI = RI * RI
      IF (MT .EQ. -2)  GO TO 12
      RD = VY - VID
      RD = RD * RD
      RA = VY - VIA
      RA = RA * RA
      NSIV = NSIV + 1
      RSIV = RSIV + RI
      NSDV = NSDV + 1
      RSDV = RSDV + RD
      NSAV = NSAV + 1
      RSAV = RSAV + RA
      GO TO 106
   12 NSIW = NSIW + 1
      RSIW = RSIW + RI
  106 CONTINUE
      IF (KPII .EQ. 1)  WRITE (3, '(I8, 2I6, I5, 3F18.7)')
     1 ND, NED, NGD, MKTR, VII, VIA, VID
      M(KKGE+ND) = MKTR
      DO 107 NA=1,NR
      NB = KKRGE + KS + NA
  107 IF (M(NB) .EQ. -2)  M(NB) = 1
      V(KDGE+ND) = VID
      WRITE (*, '('' Indirect information, cell'', I7,
     1 '' of'', I7, ''.'')')  ND, NGE
  103 CONTINUE
      IF (KPII .EQ. 1)  WRITE (3, '(79(''-''))')
      CLOSE (4)
C     Write summary statistics.
      IF (NVAL .EQ. 0)  GO TO 30
      IF (NFE0 .GT. 0)  THEN
      WRITE (3, '(/, 79(''-''), /, ''Information    Count      '',
     1 ''Difference RMS       Prediction RMS    Effective Reps'', /,
     2 79(''-''))')
      ELSE
      WRITE (3, '(/, 41(''-''), /, ''Information    Count     '',
     1 ''Difference RMS'', /, 41(''-''))')
      ENDIF
      R = NSDV
      R = RSDV / R
      RMSD = DSQRT(R)
      RMSPDR = 0.0D0
      IF (NFE0 .EQ. 0)  GO TO 20
      P = R - EMS0
      IF (P .LE. 0.0D0)  GO TO 20
      RMSPDR = DSQRT(P)
      DEFR = EMS0 / (R - EMS0)
      WRITE (3, '(''Direct'', I14, 2F21.5, F12.2)')  NSDV, RMSD,
     1 RMSPDR, DEFR
      GO TO 21
   20 WRITE (3, '(''Direct'', I14, F21.5)')  NSDV, RMSD
   21 NVW = NSIV + NSIW
      R = NVW
      R = (RSIV + RSIW) / R
      RMSD = DSQRT(R)
      IF (NFE0 .EQ. 0)  GO TO 22
      P = R - EMS0
      IF (P .LE. 0.0D0)  GO TO 22
      RMSP = DSQRT(P)
      EFR = EMS0 / (R - EMS0)
      WRITE (3, '(/, ''Indirect'', I12, 2F21.5, F12.2)')  NVW, RMSD,
     1 RMSP, EFR
      GO TO 23
   22 WRITE (3, '(/, ''Indirect'', I12, F21.5)')  NVW, RMSD
   23 R = NSIV
      R = RSIV / R
      RMSD = DSQRT(R)
      IF (NFE0 .EQ. 0)  GO TO 24
      P = R - EMS0
      IF (P .LE. 0.0D0)  GO TO 24
      RMSP = DSQRT(P)
      EFR = EMS0 / (R - EMS0)
      WRITE (3, '(''  Validation'', I8, 2F21.5, F12.2)')  NSIV, RMSD,
     1 RMSP, EFR
      GO TO 25
   24 WRITE (3, '(''  Validation'', I8, F21.5)')  NSIV, RMSD
   25 R = NSIW
      R = RSIW / R
      RMSD = DSQRT(R)
      IF (NFE0 .EQ. 0)  GO TO 26
      P = R - EMS0
      IF (P .LE. 0.0D0)  GO TO 26
      RMSP = DSQRT(P)
      EFR = EMS0 / (R - EMS0)
      WRITE (3, '(''  Withheld'', I10, 2F21.5, F12.2)')  NSIW, RMSD,
     1 RMSP, EFR
      GO TO 27
   26 WRITE (3, '(''  Withheld'', I10, F21.5)')  NSIW, RMSD
   27 EFR = 0.0D0
      R = NSAV
      R = RSAV / R
      RMSD = DSQRT(R)
      RMSP = 0.0D0
      IF (NFE0 .EQ. 0)  GO TO 28
      P = R - EMS0
      IF (P .LE. 0.0D0)  GO TO 28
      RMSP = DSQRT(P)
      EFR = EMS0 / (R - EMS0)
      WRITE (3, '(/, ''Both (AMMI'', I1, '')'', I8, 2F21.5, F12.2, /,
     1 79(''-''))')  MIIP, NSAV, RMSD, RMSP, EFR
      GO TO 29
   28 WRITE (3, '(/, ''Both (AMMI'', I1, '')'', I8, F21.5, /,
     1 41(''-''))')  MIIP, NSAV, RMSD
   29 WRITE (3, '(/, ''Grand mean'', F19.5, 3X, A20)')  GMU0, NMUN
      IF (NFE0 .EQ. 0)  GO TO 30
      WRITE (3, '(''Error RMS'', F20.5)')  REMS0
      IF (EFR .EQ. 0.0D0)  GO TO 30
      DAR = NSIW
      DAR = DAR / FGE
      WRITE (3, '(/, ''The actual number of direct replications'',
     1 '' used for modelling is'', F7.2, ''.'')')  DAR
      R = EFR / DAR
      WRITE (3, '(''Therefore the AMMI'', I1, '' gain factor is'',
     1 F7.2, ''.'')')  MIIP, R
      NF = FGE * (EFR - DAR)
      WRITE (3, '(''Given'', I5, 1X, A4, '' this amounts to'', I7,
     1 '' free observations.'')')  NGE, TNM, NF
      IF (RMSPDR .GT. 0.0D0  .AND.  RMSP .GT. 0.0D0)  THEN
      BETR = 'more'
      IF (RMSPDR .LE. RMSP)  BETR = 'less'
      WRITE (3, '(/, ''Note that AMMI'', I1, '' is '', A4,
     1 '' predictively accurate than direct '', A4, '' means.'')')
     2 MIIP, BETR, TNM
      ENDIF
      WRITE (3, '(/, ''If the direct effective replications agree'',
     1 '' with the actual within about 10%,'', /, ''if the count for'',
     2 '' direct information is at least about 100,'', /,
     3 ''and if the error RMS also has at least about 100 df,'', /,
     4 ''then these values merit reasonable confidence.'')')
      IF ((DABS(DEFR - DAR) / DAR) .LE. 0.1D0  .AND.
     1 NSDV .GE. 100  .AND.  NFE0 .GE. 100)  THEN
      WRITE (3, '(''These conditions are met.'')')
      ELSE
      WRITE (3, '(''These conditions are not met.'')')
      ENDIF
      RETURN
   30 WRITE (3, '(/, ''Grand mean'', F19.5, 3X, A20)')  GMU0, NMUN
      IF (NFE0 .GE. 1)  WRITE (3, '(''Error RMS'', F20.5)')  REMS0
      R = NSIW
      R = RSIW / R
      RMSD = DSQRT(R)
      WRITE (3, '(/, ''The RMS for'', I8, '' differences between'',
     1 '' withheld data'', /, ''and AMMI'', I1, '' imputed values is'',
     2 F21.5, ''.'')')  NSIW, MIIP, RMSD
      IF (NFE0 .EQ. 0)  RETURN
      P = R - EMS0
      IF (P .LE. 0.0D0)  RETURN
      RMSP = DSQRT(P)
      EFR = EMS0 / (R - EMS0)
      WRITE (3, '(/, ''The implied prediction RMS for imputed'',
     1 '' values is'', F21.5, '','', /, ''which equates to'', F7.2,
     2 '' effective replications.'')')  RMSP, EFR
      WRITE (3, '(/, ''If the number of differences is at least'',
     1 '' about 100,'', /, ''and if the error RMS estimate also has'',
     2 '' at least about 100 df,'', /, ''then these values merit'',
     3 '' consideration.'')')
      IF (NSIW .GE. 100  .AND.  NFE0 .GE. 100)  THEN
      WRITE (3, '(''These conditions are met.'')')
      ELSE
      WRITE (3, '(''These conditions are not met.'')')
      ENDIF
      RETURN
      END
      SUBROUTINE FTEST(NFS, SSS, NFE, SSE)
C     Given a source df and SS of NFS and SSS, and an error df and SS
C     of NFE and SSE, this subroutine returns the source MS of SMS,
C     F-test probability of PF, and significance annotation of SI.
C     This subroutine is adapted from William J. Kennedy, Jr. and
C     James E. Gentle, 1980, Statistical Computing, Marcel Dekker,
C     New York, pages 112-116.
C     Modified December 1990:  Limit NFS and NFE to 1000 in order
C     to avoid math overflow.
      IMPLICIT  REAL*8  (A-H, O-Z)
      CHARACTER*4  SI
      COMMON  /SGNF/  SMS, PF, SI
      PF = 1.0D0
      SI = ' NA '
      SMS = NFS
      SMS = SSS / SMS
      IF (NFE .EQ. 0)  RETURN
      EMS = NFE
      EMS = SSE / EMS
      Z = SMS / EMS
      N1 = NFS
      IF (N1 .GT. 1000)  N1 = 1000
      AN1 = N1
      N2 = NFE
      IF (N2 .GT. 1000)  N2 = 1000
      AN2 = N2
      A = AN1 * Z / (AN1 * Z + AN2)
      A1 = 1.0D0 - A
      IF (A1 .LT. 0.1D-36)  A1 = 0.1D-36
      D1 = AN1 * 0.5D0
      D2 = AN2 * 0.5D0
      D3 = D1 + D2 - 1.0D0
      R = 0.0D0
      S1 = 0.0D0
      S2 = 0.0D0
      DL = 1.0D0
      XM = 1.0D0
      XK = 1.0D0
      CC = 0.25D0
      PI = 3.141592653589793D0
      N = N2
   10 MM = D2
      MM = 2 * MM
      IF (MM .NE. N)  GO TO 12
      N = D2 - 1
      IF (N .EQ. 0)  GO TO 11
      DO 100 I=1,N
      S1 = DL + S1 * R
      D2 = D2 - 1.0D0
      D3 = D3 - 1.0D0
      TEM = A1 / D2
      R = D3 * TEM
      S2 = (R + TEM) * S2
  100 CONTINUE
   11 CONTINUE
      S1 = DL + S1 * R
      DL = 0.0D0
      T = -1.0D0
      D3 = -1.0D0
      S2 = A * S2
      CC = CC + 0.5D0
      GO TO 14
   12 CONTINUE
      N = D2
      IF (N .EQ. 0)  GO TO 13
      DO 101 I=1,N
      S1 = DL + S1 * R
      D2 = D2 - 1.0D0
      D3 = D3 - 1.0D0
      TEM = A1 / D2
      R = D3 * TEM
      S2 = (R + TEM) * S2
  101 CONTINUE
   13 S1 = XK * S1
      S2 = XK * S2
      ART = DSQRT(A1)
      XM = XM * ART
      T = (XM - ART) / A1
      D3 = -0.5D0
      XK = 2.0D0 / PI
      CC = CC * 2.0D0
   14 IF (CC .GT. 0.875D0)  GO TO 15
      D2 = D1
      D3 = D2 + D3
      S2 = S1
      S1 = 0.0D0
      A1 = A
      IF (A1 .LT. 0.1D-36)  A1 = 0.1D-36
      N = N1
      GO TO 10
   15 IF (CC .LT. 1.125D0)  DL = 4.0D0 / PI * DATAN(T)
      PDFF = XM * (S2 - S1) - DL
      IF (PDFF .LT. 0.0D0)  PDFF = 0.0D0
      IF (PDFF .GT. 1.0D0)  PDFF = 1.0D0
      PF = 1.0D0 - PDFF
      SI = ' ***'
      IF (PF .LE. 0.001D0)  RETURN
      SI = ' ** '
      IF (PF .LE. 0.01D0)  RETURN
      SI = ' *  '
      IF (PF .LE. 0.05D0)  RETURN
      SI = '    '
      RETURN
      END
      FUNCTION TTEST(N)
C     This function returns the two-tailed t-test statistic for the
C     0.05 level with N df.  The accuracy is 4 to 5 significant digits.
      REAL*4  T(39)
      REAL*8  TTEST
      DATA  T  /12.706, 4.3027, 3.1825, 2.7764, 2.5706, 2.4469,
     1  2.3646, 2.3060, 2.2622, 2.2281, 2.2010, 2.1788, 2.1604,
     2  2.1448, 2.1315, 2.1199, 2.1098, 2.1009, 2.0930, 2.0860,
     3  2.0796, 2.0739, 2.0687, 2.0639, 2.0595, 2.0555, 2.0518,
     4  2.0484, 2.0452, 2.0423, 2.0211, 2.008,  2.0003, 1.994,
     5  1.989,  1.986,  1.982,  1.981,  1.9799/
      IF     (N .LE. 30)  THEN
      TTEST = T(N)
      RETURN
      ELSEIF (N .LE. 119)  THEN
      NL = N / 10
      X = N - (10 * NL)
      X = X / 10.0
      NL = NL + 27
      TTEST = ((1.0 - X) * T(NL)) + (X * T(NL+1))
      RETURN
      ELSE
      X = N
      X = 120.0 / X
      TTEST = (X * 1.9799) + ((1.0 - X) * 1.9600)
      RETURN
      ENDIF
      END
      FUNCTION XORDER(N)
C     This function returns the first order statistic for N
C     realizations from the standard normal distribution, that is,
C     the average value of the maximum from a sample of N
C     realizations.  For N=1 to 30, table look up is used;
C     thereafter polynomial approximation, up to N=1000000000
C     (10**9).  For N outside this range, XORDER is set to -1.
C     The accuracy is about 5 significant digits.
      IMPLICIT  REAL*8  (C, D, X)
      REAL*8  XT(30), C0(8), C1(8), C2(8), C3(8)
      DATA  XT     /0.0000000D0, 0.5641896D0, 0.8462844D0, 1.0293754D0,
     1 1.1629645D0, 1.2672064D0, 1.3521784D0, 1.4236003D0, 1.4850132D0,
     2 1.5387527D0, 1.5864364D0, 1.6292276D0, 1.6679902D0, 1.7033816D0,
     3 1.7359134D0, 1.7659914D0, 1.7939420D0, 1.8200319D0, 1.8444815D0,
     4 1.8674751D0, 1.8891679D0, 1.9096923D0, 1.9291617D0, 1.9476741D0,
     5 1.9653146D0, 1.9821578D0, 1.9982693D0, 2.0137069D0, 2.0285221D0,
     6 2.0427608D0/
      DATA  C0  / 0.153657340D0,  0.315908870D0,  0.548553833D0,
     1  0.756410952D0,  0.9391111995D0,  1.105615262D0,  1.254237509D0,
     2  1.397281002D0/
      DATA  C1  / 1.669508346D0,  1.435243047D0,  1.198252998D0,
     1  1.040503789D0,  0.9300320966D0,  0.846385472D0,  0.782150732D0,
     2  0.731715560D0/
      DATA  C2  /-0.315716991D0, -0.202358990D0, -0.121463553D0,
     1 -0.081439260D0, -0.0591313929D0, -0.045105790D0, -0.035844206D0,
     2 -0.029928135D0/
      DATA  C3  / 0.034724278D0,  0.016336195D0,  0.007085678D0,
     1  0.003691021D0,  0.0021866870D0,  0.001401746D0,  0.000956271D0,
     2  0.000725439D0/
      XORDER = -1.0D0
      IF (N .LE. 0  .OR.  N .GT. 1000000000)  RETURN
      IF (N .GE. 31)  GO TO 10
      XORDER = XT(N)
      RETURN
   10 X = DLOG10(DBLE(N))
      X2 = X * X
      X3 = X * X2
      NS = X
      IF (NS .EQ. 9)  NS = 8
      XORDER = C0(NS) + (C1(NS) * X) + (C2(NS) * X2) + (C3(NS) * X3)
      RETURN
      END
      SUBROUTINE RANDOM
C     Generate a random number uniformly distributed between 0 and 1.
C     Before the first call, IX, IY, and IZ should be set to integer
C     values between 1 and 30000.  This generator has a period of
C     6.95 x 10**12 before it repeats.  This subroutine is based
C     upon algorithm AS 183 presented in:  Wichmann, B.A. and
C     Hill, I.D.  1982.  An efficient and portable pseudo-random
C     number generator.  Applied Statistics 31:188-190.  (Also see
C     the 1984 correction in Applied Statistics 33:123, and the
C     1986 remark in Applied Statistics 35:89).
      REAL*8  RAN
      COMMON  /RAND/  RAN, IXS, IYS, IZS, IX, IY, IZ
      IX = MOD(171*IX, 30269)
      IY = MOD(172*IY, 30307)
      IZ = MOD(170*IZ, 30323)
      RAN = AMOD(FLOAT(IX) / 30269.0 + FLOAT(IY) / 30307.0 +
     1 FLOAT(IZ) / 30323.0, 1.0)
      RETURN
      END
      SUBROUTINE INDEX(N, KBV, KBL)
C     Index N elements of vector V with base address KBV, placing
C     the indices in decending order in vector L with base address
C     KBL, using the Heapsort sorting algorithm with indirect
C     indexing through L in all references to V.
C     Hence the first index, L(KBL+1), points to V(KBV+L(KBL+1))
C     containing the largest value in vector V.
C     This subroutine is adapted from William H. Press, Brian P.
C     Flannery, Saul A. Teukolsky, and William T. Vetterling,
C     1986, Numerical Recipes, Cambridge University Press,
C     Cambridge, England, pages 232-233.
      IMPLICIT  REAL*8  (A-H, O-X)
      REAL*8     V(4000000)
      INTEGER*4  L(8000000)
      COMMON  /DATA/  Y(8000000)
      EQUIVALENCE  (V(1), Y(1), L(1))
      DO 100 J=1,N
      L(KBL+J) = J
  100 CONTINUE
      K = (N / 2) + 1
      IR = N
   10 CONTINUE
      IF (K .GT. 1)  THEN
      K = K - 1
      LT = L(KBL+K)
      X = V(KBV+LT)
      ELSE
      LT = L(KBL+IR)
      X = V(KBV+LT)
      L(KBL+IR) = L(KBL+1)
      IR = IR - 1
      IF (IR .EQ. 1)  THEN
      L(KBL+1) = LT
      GO TO 30
      ENDIF
      ENDIF
      I = K
      J = K + K
   20 IF (J .LE. IR)  THEN
      IF (J .LT. IR)  THEN
      KS = KBL + J
      IF (V(KBV+L(KS)) .LT. V(KBV+L(KS+1)))  J = J + 1
      ENDIF
      IF (X .LT. V(KBV+L(KBL+J)))  THEN
      L(KBL+I) = L(KBL+J)
      I = J
      J = J + J
      ELSE
      J = IR + 1
      ENDIF
      GO TO 20
      ENDIF
      L(KBL+I) = LT
      GO TO 10
   30 KS = KBL + N + 1
      DO 101 J=1,N/2
      I = KBL + J
      K = KS - J
      LT = L(I)
      L(I) = L(K)
  101 L(K) = LT
      RETURN
      END
